/*
 * $Id: BigInteger.java,v 1.8 1999/01/12 03:12:06 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.math/BigInteger.java,v $
 * $Revision: 1.8 $
 * $Date: 1999/01/12 03:12:06 $
 * $State: Exp $
 */

package au.net.aba.math;

import au.net.aba.security.SecureRandom;

/**
 * A class that provides the capability of performing arithmetic
 * on integers that exceed the size of the built-in Java data types.
 * This class is a compatibility class for the JDK 1.1 BigInteger version.
 * <p>
 * Immutable arbitrary-precision integers.  All operations behave as
 * if BigIntegers were represented in two's-complement notation (like
 * Java's primitive integer types).  BigIntegers provide analogues to
 * all of Java's primitive integer operators, and all relevant static
 * methods from java.lang.Math.  Additionally, BigIntegers provide
 * operations for modular arithmetic, GCD calculation, primality
 * testing, prime generation, single-bit manipulation, and a few other
 * odds and ends.
 * 
 * <P>Semantics of arithmetic operations exactly mimic those of Java's
 * integer arithmetic operators, as defined in The Java Language
 * Specification.  For example, division by zero throws an
 * ArithmeticException, and division of a negative by a positive
 * yields a negative (or zero) remainder.  All of the details in the
 * Spec concerning overflow are ignored, as BigIntegers are made as
 * large as necessary to accommodate the results of an operation.
 * <P>Semantics of shift operations extend those of Java's shift
 * operators to allow for negative shift distances.  A right-shift
 * with a negative shift distance results in a left shift, and
 * vice-versa.  The unsigned right shift operator (>>>) is omitted, as
 * this operation makes little sense in combination with the "infinite
 * word size" abstraction provided by this class.
 * 
 * <P>Semantics of bitwise logical operations are are exactly mimic
 * those of Java's bitwise integer operators.  The Binary operators
 * (and, or, xor) implicitly perform sign extension on the shorter of
 * the two operands prior to performing the operation.
 * 
 * <P>Comparison operations perform signed integer comparisons,
 * analogous to those performed by Java's relational and equality
 * operators.
 * 
 * <P>Modular arithmetic operations are provided to compute residues,
 * perform exponentiation, and compute multiplicative inverses.  These
 * methods always return a non-negative result, between 0 and (modulus
 * - 1), inclusive.
 * 
 * <P>Single-bit operations operate on a single bit of the
 * two's-complement representation of their operand.  If necessary,
 * the operand is sign extended so that it contains the designated
 * bit.  None of the single-bit operations can produce a number with a
 * different sign from the the BigInteger being operated on, as they
 * affect only a single bit, and the "infinite word size" abstraction
 * provided by this class ensures that there are infinitely many
 * "virtual sign bits" preceding each BigInteger.
 * 
 * <P><b>NOTE:</b> Not all methods present in the real BigInteger are
 * supported and differences in behaviour should be noted in the
 * documentation.
 *
 * @see BigDecimal
 */
public class BigInteger 
{
	public final static String ident = "$Id: BigInteger.java,v 1.8 1999/01/12 03:12:06 leachbj Exp $";

	//==================================
	// Constructors
	//==================================

	/**
	 * Translates a byte array containing the two's-complement
	 * representation of a (signed) integer into a BigInteger. The
	 * input array is assumed to be big-endian (i.e., the most
	 * significant byte is in the [0] position). (The most significant
	 * bit of the most significant byte is the sign bit.) The array
	 * must contain at least one byte or a NumberFormatException will
	 * be thrown. 
	*/
	public BigInteger(byte val[])
	throws NumberFormatException
	{
		assignBytes(val, 0, val.length);
	}

	/**
	 * Translates the sign-magnitude representation of an integer into
	 * a BigInteger. The sign is represented as an integer signum value
	 * (-1 for negative, 0 for zero, 1 for positive). The magnitude is
	 * represented as a big-endian byte array (i.e., the most
	 * significant byte is in the [0] position). An invalid signum
	 * value or a 0 signum value coupled with a nonzero magnitude will
	 * result in a NumberFormatException. A zero length magnitude array
	 * is permissible, and will result in in a value of 0 (irrespective
	 * of the given signum value). 
	 */
	public BigInteger(
		int signum,
		byte magnitude[])
	throws NumberFormatException
	{
		switch (signum)
		{
			case 0:
				number = new long[1];
				number[0] = 0;
				exp = 0;
				break;

			case -1:
				negative = true;
				// fall thru
			case 1:
				assignBytes(magnitude, 0, magnitude.length);
				break;

			default:
				throw new NumberFormatException("Invalid sign-magititude " +
					signum);
		}
	}

	/**
	 * Translates a string containing an optional minus sign followed
	 * by a sequence of one or more digits in the specified radix into
	 * a BigInteger. The character-to-digit mapping is provided by
	 * Character.digit. Any extraneous characters (including
	 * whitespace), or a radix outside the range from
	 * Character.MIN_RADIX(2) to Character.MAX_RADIX(36), inclusive,
	 * will result in a NumberFormatException. 
	 */
	public BigInteger(
		String val,
		int radix)
	throws NumberFormatException
	{
		assignString(val, radix);
	}

	/**
	 * Translates a string containing an optional minus sign followed
	 * by a sequence of one or more decimal digits into a BigInteger.
	 * The character-to-digit mapping is provided by Character.digit.
	 * Any extraneous characters (including whitespace) will result in
	 * a NumberFormatException. 
	 */
	public BigInteger(String val)
	throws NumberFormatException
	{
		assignString(val, 10);
	}

	/**
	 * Returns a random number uniformly distributed on [0, 2**numBits
	 * - 1] (assuming a fair source of random bits is provided in
	 * rndSrc). Note that this constructor always returns a
	 * non-negative BigInteger. Throws an IllegalArgumentException if
	 * numBits < 0. 
	 */
	public BigInteger(
		int numBits,
		SecureRandom rndSrc)
	throws IllegalArgumentException
	{
		assignRandom(numBits, rndSrc);
	}

	/**
	 * Returns a randomly selected BigInteger with the specified
	 * bitLength that is probably prime.  The certainty parameter is a
	 * measure of the uncertainty that the caller is willing to
	 * tolerate: the probability that the number is prime will exceed 1
	 * - 1/2**certainty. The execution time is proportional to the
	 * value of the certainty parameter. The given random number
	 * generator is used to select candidates to be tested for
	 * primality. Throws an ArithmeticException if bitLength < 2. 
	 */
/* XXX
	public BigInteger(int bitLength, int certainty, SecureRandom rnd)
	{
		if (bitLength < 2)
		{
			throw new ArithmeticException("Invalid bitLength.");
		}

		assignRandom(bitLength, rnd);

		// XXX
		System.err.println("Warning prime checking not implemented!");
	}
*/

	//==================================
	// Private Constructors
	//==================================

	private BigInteger()
	{
		number = new long[1];
		number[0] = 0;
		exp = 0;
		negative = false;
	}

	private BigInteger(BigInteger value)
	{
		number = new long[value.number.length];
		exp = value.exp;
		negative = value.negative;

		System.arraycopy(value.number, 0, number, 0, number.length);
	}

	//==================================
	// Public Interface
	//==================================

	/**
	 * Returns a BigInteger with the specified value. This factory is
	 * provided in preference to a (long) constructor because it allows
	 * for reuse of frequently used BigIntegers (like 0 and 1),
	 * obviating the need for exported constants. 
	 */
	public static BigInteger valueOf(long val)
	{
		BigInteger res = new BigInteger();

		// determine the number of units required
		long count = val / BASE;
		int num = 1;

		while (count != 0)
		{
			num++;
			count /= BASE;
		}

		res.number = new long[num];

		if (val < 0)
		{
			res.negative = true;
			val = -val;
		}
		else
		{
			res.negative = false;
		}

		if (val == 0)
		{
			res.exp = 0;
			res.number[0] = 0;

			return res;
		}

		// build the number
		long carry = val;

		res.exp = -1;
		while (carry != 0)
		{
			res.exp++;
			res.number[res.exp] = (carry % BASE);
			carry /= BASE;
		}

		return res;
	}

	/**
	 * Returns a BigInteger whose value is (this + val). 
	 */
	public BigInteger add(BigInteger val)
	throws ArithmeticException
	{
		BigInteger res = new BigInteger(this);

		if (negative != val.negative)
		{
			// add(-, +) || add(+, -)
			if (unsignedCompareTo(val) > 0)
			{
				res.unsignedSubtract(val); 
				res.negative = negative;
			}
			else
			{
				res = new BigInteger(val);
				res.unsignedSubtract(this); 
				res.negative = val.negative;
			}

			if (res.unsignedCompareTo(BigInteger.ZERO) == 0)
			{
				res.negative = false;
			}
		}
		else
		{
			// add(-, -) || add(+, +)
			res.unsignedAdd(val); 
			res.negative = negative;
		}

		return res;
	}

	/**
	 * Returns a BigInteger whose value is (this - val). 
	 */
	public BigInteger subtract(BigInteger val)
	{
		return add(val.negate());
	}

	/**
	 * Returns a BigInteger whose value is (this * val). 
	 */
	public BigInteger multiply(BigInteger val)
	{
		if (BigInteger.isZero(val)
			|| BigInteger.isZero(this))
		{
			return BigInteger.ZERO;
		}

		BigInteger res = new BigInteger(this);

		// do the unsigned multiplication
		res.baseMul(val);

		// set the sign bit
		if ((negative || val.negative) && (negative != val.negative))
		{
			res.negative = true;
		}
		else
		{
			res.negative = false;
		}

		return res;
	}

	/**
	 * Returns a BigInteger whose value is (this / val). Throws an
	 * ArithmeticException if val == 0. 
	 */
	public BigInteger divide(BigInteger val)
	throws ArithmeticException
	{
		if (BigInteger.isZero(val))
		{
			throw new ArithmeticException("division by zero.");
		}

		if (BigInteger.isZero(this))
		{
			return BigInteger.ZERO;
		}

		BigInteger res = new BigInteger(this);

		// do the unsigned multiplication
		res.baseDiv(val);

		// set the sign bit
		if ((negative || val.negative) && (negative != val.negative))
		{
			res.negative = true;
		}
		else
		{
			res.negative = false;
		}

		return res;
	}

	/**
	 * Returns a BigInteger whose value is (this % val). Throws an
	 * ArithmeticException if val == 0.
	 */
	public BigInteger remainder(BigInteger val)
	throws ArithmeticException
	{
		long numBits1 = significantBits();
		long numBits2 = val.significantBits();

		if (numBits2 == 0)
		{
			throw new ArithmeticException("mod by zero");
		}

		long shift = numBits1 - numBits2;

		/*
		 * are we smaller than the modulo number
		 */
		if (shift < 0)
		{
			return new BigInteger(this);
		}

		BigInteger shiftVal = new BigInteger(val);
		shiftVal = shiftVal.shiftLeft(shift);

		BigInteger res = new BigInteger(this);

		// System.out.println("shift == " + shift);
		// System.out.println("shiftVal == " + shiftVal.toString(16));
		/*
		 * calculate first part of modulus
		 */
		for (long i = shift; i >= 1; i--)
		{
			// System.out.println("shiftVal == " + shiftVal.toString(16));
			while (res.unsignedCompareTo(shiftVal) >= 0)
			{
				// System.out.println("res1 == " + res.toString(16));
				res.unsignedSubtract(shiftVal);
			}
			shiftVal.shiftR();
		}

		/*
		 * last shift
		 */
		while (res.compareTo(shiftVal) >= 0)
		{
			// System.out.println("res2 == " + res.toString(16));
			res.unsignedSubtract(shiftVal);
		}

		while (res.exp > 0 && res.number[res.exp] == 0)
		{
			res.exp--;
		}

		return res;
	}

	/**
	 * Returns an array of two BigIntegers. The first ([0]) element of
	 * the return value is the quotient (this / val), and the second
	 * ([1]) element is the remainder (this % val). Throws an
	 * ArithmeticException if val == 0. 
	 */
	public BigInteger[] divideAndRemainder(BigInteger val)
	throws ArithmeticException
	{
		BigInteger[] res = new BigInteger[2];

		res[0] = divide(new BigInteger(val));
		res[1] = remainder(new BigInteger(val));

		return res;
	}

	/**
	 * Returns a BigInteger whose value is (this ** exponent). Throws
	 * an ArithmeticException if exponent < 0 (as the operation would
	 * yield a non-integer value). Note that exponent is an integer
	 * rather than a BigInteger. 
	 */
	public BigInteger pow(int exponent)
	throws ArithmeticException
	{
		return pow(BigInteger.valueOf(exponent));
	}

	/**
	 * Returns a BigInteger whose value is the greatest common
	 * denominator of abs(this) and abs(val). Returns 0 if this == 0 &&
	 * val == 0. 
	 */
	public BigInteger gcd(BigInteger val)
	{
		BigInteger u1Out = new BigInteger();
		BigInteger u2Out = new BigInteger();

		return BigInteger.extEuclid(this, val, u1Out, u2Out);
	}

	/**
	 * Returns a BigInteger whose value is the absolute value of this
	 * number. 
	 */
	public BigInteger abs()
	{
		BigInteger res = new BigInteger(this);
		res.negative = false;

		return res;
	}

	/**
	 * Returns a BigInteger whose value is (-1 * this). 
	 */
	public BigInteger negate()
	{
		BigInteger res = new BigInteger(this);
		res.negative = !res.negative;

		return res;
	}

	/**
	 * Returns the signum function of this number (i.e., -1, 0 or 1 as
	 * the value of this number is negative, zero or positive). 
	 */
	public int signum()
	{
		if (negative)
		{
			return -1;
		}

		if (significantBits() == 0)
		{
			return 0;
		}

		return 1;
	}

	/**
	 * Returns a BigInteger whose value is this mod m. Throws an
	 * ArithmeticException if m <= 0. 
	 */
	public BigInteger mod(BigInteger m)
	{
		return remainder(m);
	}

	/**
	 * Returns a BigInteger whose value is (this ** exponent) mod m.
	 * (If exponent == 1, the returned value is (this mod m). If
	 * exponent < 0, the returned value is the modular multiplicative
	 * inverse of (this ** -exponent).) Throws an ArithmeticException
	 * if m <= 0.  
	 * <p><b>NOTE:</b> there is no support for negative exponents.
	 */
	public BigInteger modPow(
		BigInteger exponent,
		BigInteger m)
	{
		if (m.compareTo(BigInteger.ZERO) <= 0)
		{
			throw new ArithmeticException("modulus not positive.");
		}

		if (exponent.compareTo(BigInteger.ZERO) == 0)
		{
			return BigInteger.ONE;
		}

		if (exponent.compareTo(BigInteger.ZERO) < 0)
		{
			throw new ArithmeticException("no support for negative exponent.");
		}

		BigInteger res = new BigInteger(BigInteger.ONE);
		BigInteger tmp = new BigInteger(this);
		BigInteger powV = new BigInteger(exponent);

		/*
		 * the raising to the power is handled using a modified
		 * version of algorithm A (page 442, "The Art Of Computer
		 * Programming - Semi-Numerical Algorithms"), with the
		 * modulo been done progressively.
		 */
		for (int i = 0; i <= powV.exp; i++)
		{
			int count = 0;

			for (long val = powV.number[i]; val != 0; val >>= 1)
			{
				if ((val & 1) != 0)
				{
					res = res.multiply(tmp);
					res = res.remainder(m);
				}
				tmp = tmp.multiply(tmp);
				tmp = tmp.remainder(m);
				count++;
			}

			if (i != powV.exp)
			{
				for (int j = count; j < BASE_BITS; j++)
				{
					tmp = tmp.multiply(tmp);
					tmp = tmp.remainder(m);
				}
			}
		}

		return res;
	}

	/**
	 * Returns modular multiplicative inverse of this, mod m. Throws an
	 * ArithmeticException if m <= 0 or this has no multiplicative
	 * inverse mod m (i.e., gcd(this, m) != 1). 
	 */
	public BigInteger modInverse(BigInteger m)
	throws ArithmeticException
	{
		if (m.compareTo(BigInteger.ZERO) <= 0)
		{
			throw new ArithmeticException("modulus not positive.");
		}

		BigInteger x = new BigInteger();
		BigInteger y = new BigInteger();

		BigInteger gcd = BigInteger.extEuclid(this, m, x, y);

		if (!gcd.equals(BigInteger.ONE))
		{
			throw new ArithmeticException("Numbers not relatively prime.");
		}

		if (x.compareTo(BigInteger.ZERO) < 0)
		{
			x = x.add(m);
		}

		return x;
	}

	/**
	 * Returns a BigInteger whose value is (this << n). (Computes
	 * floor(this * 2**n).) 
	 */
	public BigInteger shiftLeft(int n)
	{
		return shiftLeft((long)n);
	}

	/**
	 * Returns a BigInteger whose value is (this >> n). Sign extension
	 * is performed. (Computes floor(this / 2**n).) 
	 */
	public BigInteger shiftRight(int n)
	{
		BigInteger res = new BigInteger(this);
		int step = (int)(n / BASE_BITS);
		int shift = (int)(n % BASE_BITS);

		long carry = 0;

		for (int i = exp; i >= 0; i--)
		{
			res.number[i] = 0;
		}

		for (int i = exp; i >= step; i--)
		{
			long val = number[i];
			res.number[i - step] += (val >> shift) + carry;
			carry = (val << (BASE_BITS - shift)) & BASE;
		}

		while (res.exp > 0 && res.number[res.exp] == 0)
		{
			res.exp--;
		}

		return res;
	}

	/**
	 * Returns a BigInteger whose value is (this & val). (This method
	 * returns a negative number iff this and val are both negative.) 
	 */
/* XXX
	public BigInteger and(BigInteger val)
	{
		throw new RuntimeException("and() not implemented.");
	}
*/

	/**
	 * Returns a BigInteger whose value is (this | val). (This method
	 * returns a negative number iff either this or val is negative.) 
	 */
/* XXX
	public BigInteger or(BigInteger val)
	{
		throw new RuntimeException("or() not implemented.");
	}
*/

	/**
	 * Returns a BigInteger whose value is (this ^ val). (This method
	 * returns a negative number iff exactly one of this and val are
	 * negative.) 
	 */
/* XXX
	public BigInteger xor(BigInteger val)
	{
		throw new RuntimeException("xor() not implemented.");
	}
*/

	/*
	 * Returns a BigInteger whose value is (~this). (This method
	 * returns a negative value iff this number is non-negative.) 
	 */
/* XXX
	public BigInteger not()
	{
		throw new RuntimeException("not() not implemented.");
	}
*/

	/**
	 * Returns a BigInteger whose value is (this & ~val). This method,
	 * which is equivalent to and(val.not()), is provided as a
	 * convenience for masking operations. (This method returns a
	 * negative number iff this is negative and val is positive.) 
	 */
/* XXX
	public BigInteger andNot(BigInteger val)
	{
		throw new RuntimeException("andNot() not implemented.");
	}
*/

	/**
	 * Returns true iff the designated bit is set. (Computes ((this &
	 * (1 << n)).)
	 */
/* XXX
	public boolean testBit(int n)
	throws ArithmeticException
	{
		throw new RuntimeException("testBit() not implemented.");
	}
*/

	/**
	 * Returns a BigInteger whose value is equivalent to this number
	 * with the designated bit set.  (Computes (this | (1<<n)).)
	 * Throws an ArithmeticException if n < 0.
	 */
/* XXX
	public BigInteger setBit(int n)
	throws ArithmeticException
	{
		throw new RuntimeException("setBit() not implemented.");
	}
*/

	/**
	 * Returns a BigInteger whose value is equivalent to this number
	 * with the designated bit cleared. (Computes (this & ~(1<<n)).)
	 * Throws an ArithmeticException if n < 0.
	 */
/* XXX
	public BigInteger clearBit(int n)
	throws ArithmeticException
	{
		throw new RuntimeException("clearBit() not implemented.");
	}
 */

	/**
	 * Returns a BigInteger whose value is equivalent to this number
	 * with the designated bit flipped. (Computes (this ^ (1<<n)).)
	 * Throws an ArithmeticException if n < 0.
	 */
/* XXX
	public BigInteger flipBit(int n)
	throws ArithmeticException
	{
		throw new RuntimeException("flipBit() not implemented.");
	}
*/

	/**
	 * Returns the index of the rightmost (lowest-order) one bit in
	 * this number (i.e., the number of zero bits to the right of the
	 * rightmost one bit). Returns -1 if this number contains no one
	 * bits.  (Computes (this==0? -1 : log2(this & -this)).) 
	 */
/* XXX
	public int getLowestSetBit()
	{
		throw new RuntimeException("getLowestSetBit() not implemented.");
	}
*/

	/**
	 * Returns the number of bits in the minimal two's-complement
	 * representation of this number, *excluding* a sign bit, i.e.,
	 * (ceil(log2(this < 0 ? -this : this + 1))). (For positive
	 * numbers, this is equivalent to the number of bits in the
	 * ordinary binary representation.) 
	 */
	public int bitLength()
	{
		int nShifts = 0;
		long last = number[exp];
		while (last != 0)
		{
			last >>>= 1;
			nShifts++;
		}

		return (exp * BASE_BITS + nShifts);
	}

	/**
	 * Returns the number of bits in the two's complement
	 * representation of this number that differ from its sign bit.
	 * This method is useful when implementing bit-vector style sets
	 * atop BigIntegers. 
	 */
/* XXX
	public int bitCount()
	{
		throw new RuntimeException("bitCount() not implemented.");
	}
*/

	/**
	 * Returns true if this BigInteger is probably prime, false if it's
	 * definitely composite. The certainty parameter is a measure of
	 * the uncertainty that the caller is willing to tolerate: the
	 * method returns true if the probability that this number is is
	 * prime exceeds 1 - 1/2**certainty.  The execution time is
	 * proportional to the value of the certainty parameter. 
	 */
/* XXX
	public boolean isProbablePrime(int certainty)
	{
		throw new RuntimeException("isProbablePrime() not implemented.");
	}
*/

	/**
	 * Returns -1, 0 or 1 as this number is less than, equal to, or
	 * greater than val. This method is provided in preference to
	 * individual methods for each of the six boolean comparison
	 * operators (<, ==, >, >=, !=, <=). The suggested idiom for
	 * performing these comparisons is: (x.compareTo(y) 0), where <op>
	 * is one of the six comparison operators. 
	 */
	public int compareTo(BigInteger val)
	{
		int sign = negative ? -1 : 1;

		if (negative != val.negative)
		{
			return sign;
		}

		return sign * unsignedCompareTo(val);
	}

	/**
	 * Returns true iff x is a BigInteger whose value is equal to this
	 * number. This method is provided so that BigIntegers can be used
	 * as hash keys. 
	 *
	 * @overrides: equals in class Object 
	 */
	public boolean equals(Object x)
	{
		if (x instanceof BigInteger)
			return compareTo((BigInteger)x) == 0;

		return false;
	}

	/**
	 * Returns the BigInteger whose value is the lesser of this and
	 * val. If the values are equal, either may be returned. 
	 */
	public BigInteger min(BigInteger val)
	{
		if (compareTo(val) < 0)
		{
			return this;
		}
		else
		{
			return val;
		}
	}

	/**
	 * Returns the BigInteger whose value is the greater of this and
	 * val. If the values are equal, either may be returned. 
	 */
	public BigInteger max(BigInteger val)
	{
		if (compareTo(val) > 0)
		{
			return this;
		}
		else
		{
			return val;
		}
	}

	/**
	 * Computes a hash code for this object. 
	 * 
	 * @overrides: hashCode in class Object 
	 */
/* XXX
	public int hashCode()
	{
		return super.hashCode();
	}
*/

	/**
	 * Returns the string representation of this number in the given
	 * radix. If the radix is outside the range from
	 * Character.MIN_RADIX(2) to Character.MAX_RADIX(36) inclusive, it
	 * will default to 10 (as is the case for Integer.toString). The
	 * digit-to-character mapping provided by Character.forDigit is
	 * used, and a minus sign is prepended if appropriate. (This
	 * representation is compatible with the (String, int)
	 * constructor.) 
	 */
	public String toString(int radix)
	{
		if (equals(BigInteger.ZERO))
		{
			return "0";
		}

		StringBuffer s = new StringBuffer();

		/*
		 * we handle 16 especially as it is common, and easy to process
		 */
		if (radix == 16)
		{
			String[] digits =
				{ "0", "1", "2", "3", "4", "5", "6", "7",
				"8", "9", "a", "b", "c", "d", "e", "f" };

			long num = number[exp];
			boolean allZeros = true;

			for (int j = 1; j <= BASE_BYTES * 2; j++)
			{
				long val = num >> ((BASE_BYTES * 2 - j) * 4);

				if (val != 0 || !allZeros)
				{
					allZeros = false;
					s.append(digits[(int)(val & 0xf)]);
				}
			}

			for (int i = exp - 1; i >= 0; i--)
			{
				num = number[i];
				for (int j = 1; j <= BASE_BYTES * 2; j++)
				{
					s.append(digits[(int)(num >>
						((BASE_BYTES * 2 - j) * 4)) & 0xf]);
				}
			}
		}
		else
		{
			BigInteger num = new BigInteger(this);
			num.negative = false;

			while (num.compareTo(BigInteger.ZERO) > 0)
			{
				BigInteger[] res = num.integerDivide(radix);
				s.insert(0, Character.forDigit((int)res[1].number[0], radix));
				num = res[0];
			}
		}

		if (negative)
		{
			s.insert(0, '-');
		}

		return s.toString();
	}

	/**
	 * Returns the string representation of this number, radix 10. The
	 * digit-to-character mapping provided by Character.forDigit is
	 * used, and a minus sign is prepended if appropriate. (This
	 * representation is compatible with the (String) constructor, and
	 * allows for string concatenation with Java's + operator.) 
	 * 
	 * @overrides: toString in class Object 
	 */
	public String toString()
	{
		return toString(10);
	}

	/**
	 * Returns the two's-complement representation of this number. The
	 * array is big-endian (i.e., the most significant byte is in the
	 * [0] position). The array contains the minimum number of bytes
	 * required to represent the number (ceil((this.bitLength() +
	 * 1)/8)). (This representation is compatible with the (byte[])
	 * constructor.) 
	 */
	public byte[] toByteArray()
	{
		byte[] b = new byte[lengthInBytes()];

		/*
		 * leave room for "sign bit" (Jdk 1.1 Compatibility)
		 */
		if (bitLength() % BITS_IN_BYTE == 0)
		{
			b[0] = 0;
			toBytes(b, 1, b.length - 1);
		}
		else
		{
			toBytes(b, 0, b.length);
		}

		return b;
	}

	/**
	 * Converts this number to an int. Standard narrowing primitive
	 * conversion as per The Java Language Specification. 
	 * 
	 * @overrides: intValue in class Number 
	 */
/* XXX
	public int intValue()
	{
		throw new RuntimeException("intValue() not implemented.");
	}
*/

	/**
	 * Converts this number to a long. Standard narrowing primitive
	 * conversion as per The Java Language Specification. 
	 * 
	 * @overrides: longValue in class Number 
	 */
/* XXX
	public long longValue()
	{
		throw new RuntimeException("longValue() not implemented.");
	}
*/

	/**
	 * Converts this number to a float. Similar to the double-to-float
	 * narrowing primitive conversion defined in The Java Language
	 * Specification: if the number has too great a magnitude to
	 * represent as a float, it will be converted to infinity or
	 * negative infinity, as appropriate. 
	 * 
	 * @overrides: floatValue in class Number 
	 */
/* XXX
	public float floatValue()
	{
		throw new RuntimeException("floatValue() not implemented.");
	}
*/

	/**
	 * Converts the number to a double. Similar to the double-to-float
	 * narrowing primitive conversion defined in The Java Language
	 * Specification: if the number has too great a magnitude to
	 * represent as a double, it will be converted to infinity or
	 * negative infinity, as appropriate. 
	 * 
	 * @overrides: doubleValue in class Number 
	 */
/* XXX
	public double doubleValue()
	{
		throw new RuntimeException("doubleValue() not implemented.");
	}
*/


	//==================================
	// Private Interface
	//==================================

	/*
	 * The significant bytes within each unit of the number.
	 */
	private static final int BITS_IN_BYTE = 8;
	private static final int BASE_BYTES = 7;
	private static final int BASE_BITS = BASE_BYTES * BITS_IN_BYTE;
	private static final long BASE = 0x00ffffffffffffffL;
	private static final int HALF_BASE_BITS = BASE_BITS / 2;
	private static final long HALF_BASE = 0x0fffffff;

	/*
	 *
	 */
	private static final BigInteger ZERO = BigInteger.valueOf(0);
	private static final BigInteger ONE = BigInteger.valueOf(1);

	/*
	 * The internal representation of the number.
	 */
	private long[] number;

	/*
	 * The index of the most significant unit of the number
	 * within the number array.
	 */
	private int exp;

	/*
	 * are we negative?
	 */
	private boolean  negative;

	/**
	 * Assign a string in the specified numeric base to the current
	 * BigInteger.
	 *
	 * @param s the string to use.
	 * @param base the numeric base of the string.
	 * @throws ArithmeticException When an arithmetic error occurs.
	 * @throws NumberFormatException When the string
	 *       contains invalid characters.
	 */
	private void assignString(
		String s,
		int base)
	throws ArithmeticException, NumberFormatException
	{
		int len = s.length();

		if (s == null || len <= 0)
		{
			throw new NumberFormatException("Invalid BigInteger string.");
		}

		/*
		 * set our value to zero
		 */
		number = BigInteger.ZERO.number;
		exp = BigInteger.ZERO.exp;
		negative = BigInteger.ZERO.negative;

		/*
		 * read in the number
		 */
		int i = 0;
		boolean neg = false;
		if (s.charAt(0) == '-')
		{
			neg = true;
			i++;
		}

		while (i < len)
		{
			long digit = Character.digit(s.charAt(i), base);
			if (digit < 0)
			{
				throw new NumberFormatException(s);
			}
			baseMul(BigInteger.valueOf(base));
			unsignedAdd(BigInteger.valueOf(digit));
			i++;
		}

		negative = neg;
	}

	/**
	 * Read an array of bytes (in MSB order) into a BigInteger.
	 *
	 * @param b the source array
	 * @param off the offset into the source array where the
	 * number starts.
	 * @param len the number of bytes making up the number.
	 * @throws ArrayIndexOutOfBoundsException If the array
	 * does not contain enough bytes.
	 */
	private void assignBytes(
		byte[] b,
		int off,
		int len)
	throws ArrayIndexOutOfBoundsException
	{
		if ((off < 0) || (off+len > b.length))
		{
			throw new ArrayIndexOutOfBoundsException();
		}

		/*
		 * determine the number of units required to store
		 * the specified section of the byte[]
		 */
		int totalBits = len * BITS_IN_BYTE;
		int units = (totalBits + BASE_BITS - 1) / BASE_BITS;

		number = new long[units];

		/*
		 * build the number
		 */

		int cap = off + len - 1;
		int index;

		for (index = 0; index < units - 1; index++)
		{
			number[index] = (long)(b[cap] & 0xff)
				+ ((long)(b[cap - 1] & 0xff)
				<< BITS_IN_BYTE)
				+ ((long)(b[cap - 2] & 0xff)
				<< (BITS_IN_BYTE * 2))
				+ ((long)(b[cap - 3] & 0xff)
				<< (BITS_IN_BYTE * 3))
				+ ((long)(b[cap - 4] & 0xff)
				<< (BITS_IN_BYTE * 4))
				+ ((long)(b[cap - 5] & 0xff)
				<< (BITS_IN_BYTE * 5))
				+ ((long)(b[cap - 6] & 0xff)
				<< (BITS_IN_BYTE * 6));
			cap -= 7;
		}

		number[index] = 0;
		for (int i = cap - off; i >= 0; i--)
		{
			number[index] |= ((long)(b[cap - i] & 0xff)
				<< (BITS_IN_BYTE * i));
		}

		exp = number.length - 1;
		while (exp > 0 && number[exp] == 0)
		{
			exp--;
		}
	}

	/**
	 *
	 */
	private void assignRandom(
		int numBits,
		SecureRandom rndSrc)
	throws IllegalArgumentException
	{
		if (numBits < 0)
		{
			throw new IllegalArgumentException("Invalid negative number of bits.");
		}

		int nBytes = numBits/BITS_IN_BYTE;
		if (numBits % BITS_IN_BYTE == 0)
		{
			nBytes++;
		}
		byte[] randBytes = new byte[nBytes];
		rndSrc.nextBytes(randBytes);
		assignBytes(randBytes, 0, randBytes.length);
	}

	/**
	 * Resize to the given exp.
	 */
	private void resize(int valExp)
	{
		if (valExp > exp && valExp >= number.length)
		{
			/* +2 to allow extra room */
			long[] newNumber = new long[valExp + 2];
			System.arraycopy(number, 0, newNumber, 0, number.length);
			number = newNumber;
		}
	}

	/**
	 * add another BigInteger to the current one 
	 * (assumes everything is unsigned).
	 *
	 * @param value the BigInteger to be added
	 */
	private void unsignedAdd(BigInteger value)
	{
		/*
		 * make sure we are likely to be big enough
		 */
		resize(value.exp);

		/*
		 * do the first stage of the addition
		 */
		for (int i = 0; i != value.exp; i++)
		{
			long val = number[i] + value.number[i];
			number[i] = val & BASE;
			if (val > BASE)
			{
				number[i + 1] += 1;
			}
		}

		/*
		 * final add and extend array, due to carry, if necessary
		 */
		int index = value.exp;
		long val = number[index] + value.number[index];
		number[index] = (val & BASE);

		while (val > BASE)
		{
			index++;
			if (index == number.length)
			{
				resize(index);
			}
			val = number[index] + 1;
			number[index] = (val & BASE);
		}

		/*
		 * update exponent
		 */
		if (exp < index)
		{
			exp = index;
		}
	}

	/**
	 * subtract another BigInteger from the current one (assumes
	 * both are unsigned). If the BigInteger being subtracted is 
	 * larger than the current one, it returns true.
	 *
	 * @param value the BigInteger to be subtracted
	 * @return true if negative, false if positive
	 */
	private boolean unsignedSubtract(BigInteger value)
	{
		// System.out.println("unsignedSubtract(" + this + ", " + value + ")");

		/*
		 * make sure we are likely to be big enough
		 */
		resize(value.exp);

		/*
		 * do the first stage of the subtraction
		 */
		int carry = 0;
		for (int i = 0; i <= value.exp; i++)
		{
			long val = number[i] - value.number[i] - carry;
			if (val < 0)
			{
				carry = 1;
				number[i] = val + BASE + 1;
			}
			else
			{
				carry = 0;
				number[i] = val;
			}
		}

		/*
		 * second stage
		 */
		if (carry != 0 && value.exp < exp)
		{
			for (int i = value.exp + 1; i <= exp; i++)
			{
				long val = number[i] - carry;
				if (val < 0)
				{
					carry = 1;
					number[i] = (val + BASE + 1);
				}
				else
				{
					carry = 0;
					number[i] = val;
					break;	/* no need to go further */
				}
			}
		}

		if (carry != 0)
		{
			number[exp] = -(number[exp] - (BASE + 1));
		}
			
		exp = number.length - 1;
		while (exp > 0 && number[exp] == 0)
		{
			exp--;
		}

		return (carry == 1);
	}

	/**
	 * multiplies the current BigInteger by another, in an unsigned fashion.
	 *
	 * @param value the BigInteger to be multiplied
	 */
	private void baseMul(BigInteger value)
	{
		long[] newNumber = new long[exp + value.exp + 2];

		/*
		 * multiplication of the two longs is handled by splitting
		 * the longs into smaller values, and then multiplying the
		 * components together.
		 */
		for (int i = 0; i <= value.exp; i++)
		{
			long a1 = value.number[i] & HALF_BASE;
			long a2 = value.number[i] >> HALF_BASE_BITS;

			for (int j = 0; j <= exp; j++)
			{
				long b1 = number[j] & HALF_BASE;
				long b2 = number[j] >> HALF_BASE_BITS;
				long val = a1 * b1;

				newNumber[i + j] += val;

				val = a2 * b1;
				newNumber[i + j] += ((val << HALF_BASE_BITS)
					& BASE);
				newNumber[i + j + 1] += val >> HALF_BASE_BITS;

				val = a1 * b2;
				newNumber[i + j] += ((val << HALF_BASE_BITS)
					& BASE);
				newNumber[i + j + 1] += val >> HALF_BASE_BITS;

				val = a2 * b2;
				newNumber[i + j + 1] += val;
			}
		}

		for (int i = 0; i < newNumber.length - 1; i++)
		{
			long val = newNumber[i];
			newNumber[i] = val & BASE;
			newNumber[i + 1] += val >> BASE_BITS;
		}

		number = newNumber;
		exp = number.length - 1;
		while (exp > 0 && number[exp] == 0)
		{
			exp--;
		}
	}

	/**
	 * Divide the current BigInteger by another.
	 *
	 * @param val the value we are dividing by.
	 */
	private void baseDiv(BigInteger val)
	{
		if (BigInteger.isZero(this))
		{
			throw new ArithmeticException("div by zero");
		}

		long numBits1 = significantBits();
		long numBits2 = val.significantBits();

		long shift = numBits1 - numBits2;

		/*
		 * is the divisor to big?
		 */
		if (shift < 0)
		{
			// reset this to zero
			this.number = BigInteger.ZERO.number;
			this.exp = BigInteger.ZERO.exp;
			this.negative = BigInteger.ZERO.negative;

			return;
		}

		BigInteger res = new BigInteger(BigInteger.ZERO);
		BigInteger shiftVal = new BigInteger(val);

		shiftVal = shiftVal.shiftLeft(shift);

		/*
		 * calculate first part of the result
		 */
		for (long i = shift; i >= 1; i--)
		{
			while (unsignedCompareTo(shiftVal) >= 0)
			{
				res.unsignedAdd(BigInteger.ONE);
				unsignedSubtract(shiftVal);
			}
			shiftVal.shiftR();
			res = res.shiftLeft(1);
		}

		/*
		 * last pass
		 */
		while (unsignedCompareTo(shiftVal) >= 0)
		{
			res.unsignedAdd(BigInteger.ONE);
			unsignedSubtract(shiftVal);
		}

		this.number = res.number;
		this.exp = res.exp;
		this.negative = res.negative;
	}

	/**
	 * Returns an array of two BigIntegers. The first ([0]) element of
	 * the return value is the quotient (this / val), and the second
	 * ([1]) element is the remainder (this % val). Throws an
	 * ArithmeticException if val == 0. 
	 */
/*
	private BigInteger[] divideAndRemainder(int value)
	throws ArithmeticException
	{
		BigInteger[] res = new BigInteger[2];

		if (value <= 127 && value > 0)
		{
			long carry = 0;
			BigInteger divRes = new BigInteger(this);

			for (int i = divRes.exp; i >= 0; i--)
			{
				long tmp = (long)(divRes.number[i]
					+ (carry << BASE_BITS));
				divRes.number[i] = (tmp / value);
				carry = tmp % value;
			}

			while (divRes.exp > 0 && divRes.number[divRes.exp] == 0)
			{
				divRes.exp--;
			}

			res[0] = divRes;
			res[1] = BigInteger.valueOf(carry);
		}
		else
		{
			res[0] = divide(BigInteger.valueOf(value));
			res[1] = remainder(BigInteger.valueOf(value));
		}

		return res;
	}
*/

	/**
	 * Returns a BigInteger whose value is (this ** exponent). Throws
	 * an ArithmeticException if exponent < 0 (as the operation would
	 * yield a non-integer value). Note that exponent is an integer
	 * rather than a BigInteger. 
	 */
	private BigInteger pow(BigInteger exponent)
	throws ArithmeticException
	{
		BigInteger res = new BigInteger(BigInteger.ONE);
		BigInteger mult = new BigInteger(this);
		BigInteger shifter = new BigInteger(exponent);

		/*
		 * the raising to the power is handled using a modified
		 * version of algorithm A (page 442, "The Art Of Computer
		 * Programming - Semi-Numerical Algorithms").
		 */
		while (shifter.compareTo(BigInteger.ZERO) > 0)
		{
			if ((shifter.number[0] & 1) != 0)
			{
				res = res.multiply(mult);
			}
			mult = mult.multiply(mult);
			shifter.shiftR();
		}

		return res;
	}

	/**
	 * Calculate the numbers u1, u2, and u3 such that:
	 *
	 * u1 * a + u2 * b = u3
	 *
	 * where u3 is the greatest common divider of a and b.
	 * a and b using the extended Euclid algorithm (refer p. 323
	 * of The Art of Computer Programming vol 2, 2nd ed).
	 * This also seems to have the side effect of calculating
	 * some form of multiplicative inverse.
	 *
	 * @param a    First number to calculate gcd for
	 * @param b    Second number to calculate gcd for
	 * @param u1Out      the return object for the u1 value
	 * @param u2Out      the return object for the u2 value
	 * @return     The greatest common divisor of a and b
	 */
	private static BigInteger extEuclid(
		BigInteger a,
		BigInteger b,
		BigInteger u1Out,
		BigInteger u2Out)
	{
		BigInteger  res;

		BigInteger u1 = new BigInteger(BigInteger.ONE);
		BigInteger u3 = new BigInteger(a);
		BigInteger v1 = new BigInteger(BigInteger.ZERO);
		BigInteger v3 = new BigInteger(b);

		while (v3.compareTo(BigInteger.ZERO) > 0)
		{
			BigInteger q, tn, tv;

			q = u3.divide(v3);

			tn = new BigInteger(u1);
			tv = new BigInteger(v1);

			// System.out.println(tn + " - " + tv + " * " + q);
			// System.out.println(tv + " * " + q + " == " + tv.multiply(q));

			tn = tn.subtract(tv.multiply(q));
			u1 = v1;
			v1 = tn;

			// System.out.println("v1 == " + v1);

			tn = new BigInteger(u3);
			tv = new BigInteger(v3);
			tn = tn.subtract(tv.multiply(q));
			u3 = v3;
			v3 = tn;

			// System.out.println("v3 == " + v3);
		}

		res = new BigInteger(u1);
		u1Out.exp = res.exp;
		u1Out.number = res.number;
		u1Out.negative = res.negative;

		res = new BigInteger(u3);
		res = res.subtract(u1.multiply(a)).divide(b);
		u2Out.exp = res.exp;
		u2Out.number = res.number;
		u2Out.negative = res.negative;

		return u3;
	}

	/**
	 * Convert the current BigInteger into an array of bytes in MSB
	 * order.
	 *
	 * @param b the array to write into.
	 * @param off the offset into the array to being writing.
	 * @param len the number of bytes to write.
	 * @throws ArrayIndexOutOfBoundsException If the array
	 * 	is not large enough.
	 */
	private void toBytes(
		byte[] b,
		int off,
		int length)
	throws ArrayIndexOutOfBoundsException
	{
		long value;
		int len = 0;
		int bitLen = bitLength();

		if (bitLen > 0)
		{
			len = ((bitLength() + BITS_IN_BYTE - 1)
				/ BITS_IN_BYTE) % BASE_BYTES;

			if (len == 0)     // we are an exact multiple
			{
				len = 7;
			}
		}

		value = number[exp];
		for (int j = 0; j != len; j++)
		{
			b[j + off] = (byte)(value >> ((len - j - 1)
				* BITS_IN_BYTE));
		}

		off -= BASE_BYTES - len;

		for (int i = exp - 1; i >= 0; i--)
		{
			value = number[i];
			for (int j = 0; j != BASE_BYTES; j++)
			{
				b[(exp - i) * BASE_BYTES + j + off]
					= (byte)(value
					>> ((BASE_BYTES - j - 1)
					* BITS_IN_BYTE));
			}
		}
	}

	/**
	 * Right shift the current BigInt by one bit. This routine
	 * should only be used for internal optimisation.
	 */
	private void shiftR()
	{
		long carry = 0;

		for (int i = exp; i >= 1; i--)
		{
			number[i] += carry;;
			if ((number[i] & 1) == 1)
			{
				carry = (long)1 << BASE_BITS;
			}
			else
			{
				carry = 0;
			}
			number[i] >>= 1;
		}

		number[0] += carry;;
		number[0] >>= 1;

		while (exp > 0 && number[exp] == 0)
		{
			exp--;
		}
	}

	/**
	 * calculate the length in bytes.
	 *
	 * @return the length in bytes.
	 */
	private int lengthInBytes()
	{
		int base = exp * BASE_BITS;
		for (long top = number[exp]; top != 0; top >>= 1)
		{
			base++;
		}

		return (base + BITS_IN_BYTE) / BITS_IN_BYTE;
	}

	/**
	 * Division by an integer.
	 *
	 * @param value the long to be divided by.
	 * @return two BigIntegers, the first being the result of the
	 *          division, the second being the remainder.
	 */
	public BigInteger[] integerDivide(int value)
	{
		BigInteger[] res = new BigInteger[2];

		if (value <= 127 && value > 0)
		{
			BigInteger divRes = new BigInteger(this);
			long carry = 0;

			for (int i = divRes.exp; i >= 0; i--)
			{
				long tmp = (long)(divRes.number[i] + (carry << BASE_BITS));
				divRes.number[i] = (tmp / value);
				carry = tmp % value;
			}

			while (divRes.exp > 0 && divRes.number[divRes.exp] == 0)
			{
				divRes.exp--;
			}

			res[0] = divRes;
			res[1] = BigInteger.valueOf(carry);
		}
		else
		{
			res[0] = divide(BigInteger.valueOf(value));
			res[1] = remainder(BigInteger.valueOf(value));
		}

		return res;
	}

	/**
	 * returns the number of bits in the current BigInteger
	 *
	 * @return the number of bits needed to contain the BigInteger
	 */
	private int significantBits()
	{
		int nShifts = 0;
		long last = number[exp];

		while (last != 0)
		{
			last >>>= 1;
			nShifts++;
		}

		return (exp * BASE_BITS + nShifts);
	}

	private int unsignedCompareTo(BigInteger val)
	{
		if (exp > val.exp)
		{
			return 1;
		}
		else if (exp < val.exp)
		{
			return -1;
		}

		/*
		 * at this point we just compare index values
		 */

		int index;
		for (index = exp; index != 0; index--)
		{
			if (number[index] != val.number[index])
			{
				break;
			}
		}

		if (number[index] > val.number[index])
		{
			return 1;
		}
		else if (number[index] < val.number[index])
		{
			return -1;
		}

		return 0;
	}

	private BigInteger shiftLeft(long n)
	{
		int step = (int)(n / BASE_BITS);
		int shift = (int)(n % BASE_BITS);

		BigInteger res = new BigInteger(BigInteger.ZERO);
		res.number = new long[step + exp + 2];

		for (int i = 0; i <= exp; i++)
		{
			long val = (long)number[i];
			res.number[i + step + 1] = (val >> (BASE_BITS - shift));
			res.number[i + step] += ((val << shift) & BASE);
		}

		res.exp = res.number.length - 1;
		while (res.exp > 0 && res.number[res.exp] == 0)
		{
			res.exp--;
		}

		return res;
	}

	/**
	 * Returns true if the given value is zero.
	 */
	private final static boolean isZero(BigInteger val)
	{
		return val.exp == 0 && val.number[0] == 0;
	}
}

