/*
 * $Id: AlgorithmParameters.java,v 1.4 1998/10/26 01:46:50 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/AlgorithmParameters.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/26 01:46:50 $
 * $State: Exp $
 */

package au.net.aba.security;

import java.io.*;
import java.security.*;

import au.net.aba.security.spec.*;

/**
 * This class is used as an opaque representation of cryptographic
 * parameters. 
 * <p>
 * An AlgorithmParameters object for managing the parameters for a
 * particular algorithm can be obtained by calling one of the getInstance
 * factory methods (static methods that return instances of a given class). 
 * <p>
 * There are two ways to request such an implementation: by specifying
 * either just an algorithm name, or both an algorithm name and a package
 * provider. 
 * <p>
 *   If just an algorithm name is specified, the system will determine if
 *   there is an AlgorithmParameters implementation for the algorithm
 *   requested available in the environment, and if there is more than
 *   one, if there is a preferred one. 
 * <p>
 *   If both an algorithm name and a package provider are specified, the
 *   system will determine if there is an implementation in the package
 *   requested, and throw an exception if there is not. 
 * <p>
 * Once an AlgorithmParameters object is returned, it must be
 * initialised via a call to init, using an appropriate parameter
 * specification or parameter encoding. 
 * <p>
 * A transparent parameter specification is obtained from an
 * AlgorithmParameters object via a call to getParameterSpec, and
 * a byte encoding of the parameters is obtained via a call to getEncoded. 
 *
 * @see AlgorithmParameterSpec
 * @see DSAParameterSpec
 * @see KeyPairGenerator
 */
public class AlgorithmParameters
{
	public final static String ident = "$Id: AlgorithmParameters.java,v 1.4 1998/10/26 01:46:50 leachbj Exp $";

	private AlgorithmParametersSpi	paramSpi;
	private Provider		provider;
	private String			algorithm;

	/**
	 * Creates an AlgorithmParameters object.
	 *
	 * @param keyFacSpi the delegate
	 * @param provider the provider
	 * @param algorithm the algorithm
	 */
	protected AlgorithmParameters(
		AlgorithmParametersSpi	paramSpi,
		Provider		provider,
		String			algorithm)
	{
		this.paramSpi = paramSpi;
		this.provider = provider;
		this.algorithm = algorithm;
	}

	/**
	 * Returns the name of the algorithm associated with the parameter
	 * set.
	 *
	 * @return the algorithm name.
	 */
	public final String getAlgorithm()
	{
		return algorithm;
	}

	/**
	 * create an instance of a AlgorithmParameters object given the provider.
	 *
	 * @param prov the provider object.
	 * @param algorithm the algorithm we want.
	 */
	private static final AlgorithmParameters getInstance(
		String		algorithm,
		Provider	prov)
		throws NoSuchAlgorithmException
	{
		AlgorithmParameters	param;
		AlgorithmParametersSpi	spi;
		String			className;

		className = prov.getProperty("AlgorithmParameters." + algorithm);
		if (className == null)
		{
			throw new NoSuchAlgorithmException(algorithm);
		}

		try
		{
			spi = (AlgorithmParametersSpi)Class.forName(
						className).newInstance();
		}
		catch (Exception e)
		{
			throw new NoSuchAlgorithmException(
						algorithm + ": " + e);
		}

		return new AlgorithmParameters(spi, prov, algorithm);
	}

	/**
	 * Generates an AlgorithmParameters object that implements the
	 * specified digest algorithm. If the default provider package provides
	 * an implementation of the requested digest algorithm, an instance of
	 * AlgorithmParameters containing that implementation is returned.
	 * If the algorithm is not available in the default package, other
	 * packages are searched.
	 *
	 * @param algorithm the name of the algorithm requested.
	 * @return the new AlgorithmParameters object.
	 * @exception NoSuchAlgorithmException if the algorithm is not
	 *    available in the environment.
	 */
	public static final AlgorithmParameters getInstance(
		String algorithm)
		throws NoSuchAlgorithmException
	{
		Provider[] prov = Security.getProviders();

		for (int i = 0; i != prov.length; i++)
		{
			try
			{
				return getInstance(algorithm, prov[i]);
			}
			catch (Exception e)
			{
				// ignore
			}
		}

		throw new NoSuchAlgorithmException(
			"No AlgorithmParameters could be found for algorithm "
				+ algorithm);
	}

	/**
	 * Generates an AlgorithmParameters object for the specified
	 * algorithm, as supplied from the specified provider, if such an
	 * algorithm is available from the provider.
	 *
	 * @param algorithm - the name of the algorithm requested.
	 * @param provider - the name of the provider.
	 *
	 * @return the new AlgorithmParameters object.
	 *
	 * @exception NoSuchAlgorithmException if the algorithm is not
	 * available in the package supplied by the requested provider.
	 * @exception NoSuchProviderException if the provider is not available
	 * in the environment.
	 * @see Provider
	 */
	public static final AlgorithmParameters getInstance(
		String algorithm,
		String provider)
		throws NoSuchAlgorithmException, NoSuchProviderException
	{
		Provider	prov;

		prov = Security.getProvider(provider);
		if (prov == null)
		{
			throw new NoSuchProviderException(
					"Provider " + provider + " not found");
		}

		return getInstance(algorithm, prov);
	}

	/**
	 * Returns the provider of this algorithm parameter object.
	 *
	 * @return the provider of this algorithm parameter object
	 */
	public final Provider getProvider()
	{
		return provider;
	}

	/**
	 * Initialises this parameters object using the parameters specified in
	 * paramSpec.
	 *
	 * @param paramSpec the parameter specification.
	 * @exception InvalidParameterSpecException if the given parameter
	 * specification is inappropriate for the initialisation of this
	 * parameter object.
	 */
	public final void init(
		AlgorithmParameterSpec paramSpec)
                throws InvalidParameterSpecException
	{
		paramSpi.engineInit(paramSpec);
	}

	/**
	 * Imports the specified parameters and decodes them according to the
	 * primary decoding format for parameters. The primary decoding
	 * format for parameters is ASN.1, if an ASN.1 specification for this
	 * type of parameters exists.
	 * 
	 * @param params the encoded parameters.
	 * @exception IOException on decoding errors
	 */
	public final void init(
		byte[] params)
		throws IOException
	{
		paramSpi.engineInit(params);
	}

	/**
	 * Imports the parameters from params and decodes them according
	 * to the specified decoding format. If format is null, the primary
	 * decoding format for parameters is used. The primary decoding
	 * format is ASN.1, if an ASN.1 specification for these parameters
	 * exists.
	 *
	 * @param params the encoded parameters.
	 * @param format the name of the decoding format
	 * @exception IOException on decoding errors
	 */
	public final void init(
		byte[]	params,
		String	format)
		throws IOException
	{
		paramSpi.engineInit(params, format);
	}

	/**
	 * Returns a (transparent) specification of this parameters object.
	 * paramSpec identifies the specification class in which the
	 * parameters should be returned. It could, for example, be
	 * DSAParameterSpec.class, to indicate that the parameters
	 * should be returned in an instance of the DSAParameterSpec class.
	 *
	 * @param paramSpec - the specification class in which the parameters
	 * should be returned.
	 *
	 * @return the parameter specification.
	 *
	 * @exception InvalidParameterSpecException - if the requested parameter
	 * specification is inappropriate for this parameter object.
	 */
	public final AlgorithmParameterSpec getParameterSpec(
		Class paramSpec)
		throws InvalidParameterSpecException
	{
		return paramSpi.engineGetParameterSpec(paramSpec);
	}

	/**
	 * Returns the parameters in their primary encoding format. The
	 * primary encoding format for parameters is ASN.1, if an ASN.1
	 * specification for this type of parameters exists.
	 *
	 * @return the parameters encoded using the specified encoding scheme.
	 *
	 * @exception IOException - on encoding errors.
	 */
	public final byte[] getEncoded()
		throws IOException
	{
		return paramSpi.engineGetEncoded();
	}

	/**
	 * Returns the parameters encoded in the specified format. If format
	 * is null, the primary encoding format for parameters is used. The
	 * primary encoding format is ASN.1, if an ASN.1 specification for
	 * these parameters exists.
	 *
	 * @param format - the name of the encoding format.
	 *
	 * @return the parameters encoded using the specified encoding scheme.
	 *
	 * @exception IOException - on encoding errors.
	 */
	public final byte[] getEncoded(
		String format)
		throws IOException
	{
		return paramSpi.engineGetEncoded(format);
	}

	/**
	 * Returns a formatted string describing the parameters.
	 *
	 * @return a formatted string describing the parameters, or null if this
	 * object has not been initialised.
	 *
	 */
	public final String toString()
	{
		return paramSpi.engineToString();
	}
}
