/*
 * $Id: DigestInputStream.java,v 1.3 1998/10/26 01:46:10 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.security/DigestInputStream.java,v $
 * $Revision: 1.3 $
 * $Date: 1998/10/26 01:46:10 $
 * $State: Exp $
 */

package au.net.aba.security;

import java.io.*;

/**
 *
 * <p>A transparent stream that updates the associated message digest
 * using the bits going through the stream.
 *
 * <p>To complete the message digest computation, call one of the
 * digest methods on the associated message digest after your calls to
 * one of this digest input stream's read methods.
 *
 * <p>It is possible to turn this stream on or off (see on). When it
 * is on, a call to read results in an update on the message digest.
 * But when it is off, the message digest is not updated. The default
 * is for the stream to be on.
 *
 * <p>Note that digest objects can compute only one digest (see
 * MessageDigest), so that in order to compute intermediate digests, a
 * caller should retain a handle onto the digest object, and clone it
 * for each digest to be computed, leaving the original digest
 * untouched.
 *
 * @see MessageDigest, DigestOutputStream
 */
public class DigestInputStream
	extends FilterInputStream
{
	public final static String ident = "$Id: DigestInputStream.java,v 1.3 1998/10/26 01:46:10 leachbj Exp $";

	/** The message digest associated with this stream. */
	protected MessageDigest digest;

	private boolean on;

	/**
	 * Creates a digest input stream, using the specified input stream
	 * and message digest.
	 *
	 * @param stream - the input stream. 
	 * @param digest - the message digest to associate with this stream. 
	 */
	public DigestInputStream(InputStream stream,
		MessageDigest digest)
	{
		super(stream);
		this.digest = digest;
		this.on = true;
	}

	/**
	 * Returns the message digest associated with this stream. 
	 * 
	 * @return the message digest associated with this stream.
	 */
	public MessageDigest getMessageDigest()
	{
		return digest;
	}

	/**
	 * Associates the specified message digest with this stream. 
	 * 
	 * @param digest - the message digest to be associated with this stream. 
	 */
	public void setMessageDigest(MessageDigest digest)
	{
		this.digest = digest;
	}

	/**
	 * Reads a byte, and updates the message digest (if the digest
	 * function is on). That is, this method reads a byte from the
	 * input stream, blocking until the byte is actually read. If the
	 * digest function is on (see on), this method will then call
	 * update on the message digest associated with this stream,
	 * passing it the byte read.
	 * 
	 * @returns the byte read.
	 * @exception IOException if an I/O error occurs.
	 * @override read in class FilterInputStream
	 * @see update
	 */
	public int read() throws IOException
	{
		int b = super.read();

		if (b != -1 && on)
		{
			digest.update((byte)b);
		}
		return b;
	}


	/**
	 * Reads into a byte array, and updates the message digest (if the
	 * digest function is on). That is, this method reads up to len
	 * bytes from the input stream into the array b, starting at offset
	 * off. This method blocks until the data is actually read. If the
	 * digest function is on (see on), this method will then call
	 * update on the message digest associated with this stream,
	 * passing it the data.
	 * 
	 * @param b - the array into which the data is read. 
	 * @param off - the starting offset into b of where the data should
	 * 	be placed.
	 * @param len - the maximum number of bytes to be read from the
	 * 	input stream into b, starting at offset off.
	 * @return the actual number of bytes read. This is less than len
	 * 	if the end of the stream is reached prior to reading len bytes.
	 * 	-1 is returned if no bytes were read because the end of the
	 * 	stream had already been reached when the call was made
	 * @exception IOException if an I/O error occurs.
	 * @overrides read in class FilterInputStream
	 * @see update
	 */
	public int read(
		byte[] b,
		int off,
		int len)
	throws IOException
	{
		int n = super.read(b, off, len);
		if (n != -1 && on)
		{
			digest.update(b, off, n);
		}
		return n;
	}


	/**
	 * Turns the digest function on or off. The default is on. When it
	 * is on, a call to read results in an update on the message
	 * digest. But when it is off, the message digest is not updated.
	 * 
	 * @param on - true to turn the digest function on, false to turn it off. 
	 */
	public void on(boolean on)
	{
		this.on = on;
	}


	/**
	 * Prints a string representation of this digest input stream and
	 * its associated message digest object.
	 * 
	 * @override toString in class Object
	 */
	public String toString()
	{
		return "stream: "
			+ this.toString()
			+ "digest: "
			+ digest.toString();
	}
}

