/*
 * $Id: DigestOutputStream.java,v 1.3 1998/10/26 01:46:10 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.security/DigestOutputStream.java,v $
 * $Revision: 1.3 $
 * $Date: 1998/10/26 01:46:10 $
 * $State: Exp $
 */

package au.net.aba.security;

import java.io.*;

/**
 *
 * <p>A transparent stream that updates the associated message digest
 * using the bits going through the stream.
 * 
 * <p>To complete the message digest computation, call one of the
 * digest methods on the associated message digest after your calls to
 * one of this digest output stream's write methods.
 * 
 * <p>It is possible to turn this stream on or off (see on). When it
 * is on, a call to write results in an update on the message digest.
 * But when it is off, the message digest is not updated. The default
 * is for the stream to be on.
 * 
 * @see MessageDigest, DigestInputStream
 */
public class DigestOutputStream
	extends FilterOutputStream
{
	public final static String ident = "$Id: DigestOutputStream.java,v 1.3 1998/10/26 01:46:10 leachbj Exp $";

	/** The message digest associated with this stream. */
	protected MessageDigest digest;

	private boolean on;

	/**
	 * Creates a digest output stream, using the specified output
	 * stream and message digest.
	 * 
	 * @param stream - the output stream. 
	 * @param digest - the message digest to associate with this stream. 
	 */
	public DigestOutputStream(
		OutputStream stream,
		MessageDigest digest)
	{
		super(stream);
		this.digest = digest;
		this.on = true;
	}

	/**
	 * Returns the message digest associated with this stream. 
	 * 
	 * @return the message digest associated with this stream.
	 */
	public MessageDigest getMessageDigest()
	{
		return digest;
	}

	/**
	 * Associates the specified message digest with this stream. 
	 * 
	 * @param digest - the message digest to be associated with this
	 *	stream. 
	 */
	public void setMessageDigest(MessageDigest digest)
	{
		this.digest = digest;
	}

	/**
	 * Updates the message digest (if the digest function is on) using
	 * the specified byte, and in any case writes the byte to the
	 * output stream. That is, if the digest function is on (see on),
	 * this method calls update on the message digest associated with
	 * this stream, passing it the byte b. This method then writes the
	 * byte to the output stream, blocking until the byte is actually
	 * written.
	 * 
	 * @param b - the byte to be used for updating and writing to the
	 * 	output stream.
	 * @exception IOException if an I/O error occurs.
	 * @override write in class FilterOutputStream
	 * @see update
	 */
	public void write(int b)
	throws IOException
	{
		if (on)
		{
			digest.update((byte)b);
		}
		super.write(b);
	}

	/**
	 * Updates the message digest (if the digest function is on) using
	 * the specified subarray, and in any case writes the subarray to
	 * the output stream. That is, if the digest function is on (see
	 * on), this method calls update on the message digest associated
	 * with this stream, passing it the subarray specifications. This
	 * method then writes the subarray bytes to the output stream,
	 * blocking until the bytes are actually written.
	 * 
	 * b - the array containing the subarray to be used for updating
	 * 	and writing to the output stream.
	 * off - the offset into b of the first byte to be updated and written. 
	 * len - the number of bytes of data to be updated and written from
	 * 	b, starting at offset off.
	 * @throws IOException if an I/O error occurs.
	 * @overrides write in class FilterOutputStream
	 * @see update
	 */
	public void write(
		byte[] b,
		int off,
		int len)
	throws IOException
	{
		if (on)
		{
			digest.update(b, off, len);
		}
		super.write(b, off, len);
	}

	/**
	 * Turns the digest function on or off. The default is on. When it
	 * is on, a call to write results in an update on the message
	 * digest. But when it is off, the message digest is not updated.
	 * 
	 * @param on - true to turn the digest function on, false to turn
	 *	it off. 
	 */
	public void on(boolean on)
	{
		this.on = on;
	}

	/**
	 * Prints a string representation of this digest output stream and
	 * its associated message digest object.
	 * 
	 * @override toString in class Object
	 */
	public String toString()
	{
		return "stream: "
			+ this.toString()
			+ "digest: "
			+ digest.toString();
	}
}

