/*
 * $Id: KeyFactory.java,v 1.6 1998/10/27 05:10:46 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.security/KeyFactory.java,v $
 * $Revision: 1.6 $
 * $Date: 1998/10/27 05:10:46 $
 * $State: Exp $
 */

package au.net.aba.security;

import au.net.aba.security.spec.*;

/**
 * A factory class for creating Keys.
 */
public abstract class KeyFactory
{
	public final static String ident = "$Id: KeyFactory.java,v 1.6 1998/10/27 05:10:46 leachbj Exp $";

	public KeyFactory()
	{
	}

	/**
	 * Generates a private key object from the provided key specification
	 * (key material).
	 *
	 * @param keySpec	The specification (key material of the private key.
	 * @returns the private key.
	 * @exception InvalidKeySpecException	if the given key specification
	 * 	is inappropriate for this key factory to produce a private key.
	 */
	public PrivateKey generatePrivate(
		KeySpec	keySpec)
	throws InvalidKeySpecException
	{
		return engineGeneratePrivate(keySpec);
	}

	/**
	 * Generates a public key object from the provided key specification
	 * (key material). 
	 *
	 *
	 * @param keySpec - the specification (key material) of the
	 * 	 public key.
	 * @returns the public key.
	 * @exception InvalidKeySpecException - if the given key
	 * 	specification is inappropriate for this key factory to
	 * 	produce a public key.
	 */
	public PublicKey generatePublic(
		KeySpec	keySpec)
	throws InvalidKeySpecException
	{
		return engineGeneratePublic(keySpec);
	}

	/**
	 * Generates a KeyFactory object for the specified algorithm. If the
	 * default provider package contains a KeyFactory subclass for the
	 * algorithm, an instance of that subclass is returned. If such a key
	 * factory is not available in the default provider package, other
	 * provider packages are searched. 
	 *
	 * @param algorithm the name of the requested key algorithm. See
	 * 	Appendix A in the Java Cryptography Architecture API
	 * 	Specification & Reference for information about standard
	 * 	algorithm names.
	 * @return a KeyFactory object for the specified algorithm.
	 * @exception NoSuchAlgorithmException if the algorithm is not
	 * 	available from the specified provider.
	 */
	public static KeyFactory getInstance(
		String algorithm)
	throws NoSuchAlgorithmException
	{
		Object	obj;
		String	name;

		name = "au.net.aba.crypto.provider." + algorithm + "KeyFactory";

		try
		{
			obj = Class.forName(name).newInstance();
		}
		catch (Exception e)
		{
			throw new NoSuchAlgorithmException(name + " : " + e);
		}

		return (KeyFactory)obj;
	}

	/**
	 * Generates a KeyFactory object for the specified algorithm from the
	 * specified provider. 
	 *
	 * @param algorithm the name of the requested key algorithm. See
	 * 	Appendix A in the Java Cryptography Architecture API
	 * 	Specification & Reference for information about standard
	 * 	algorithm names.
	 * @param provider the name of the provider.
	 * @return a KeyFactory object for the specified algorithm.
	 * @exception NoSuchAlgorithmException if the algorithm is not
	 * 	available from the specified provider.
	 * @exception NoSuchProviderException if the provider has not
	 * 	been configured.
	 */
	public static KeyFactory getInstance(
		String	algorithm,
		String	provider)
	throws NoSuchAlgorithmException, NoSuchProviderException
	{
		if (!provider.equals("ABA"))
		{
			throw new NoSuchProviderException("ABA only provider");
		}

		return getInstance(algorithm);
	}

	/**
	 * Returns a specification (key material) of the given key object.
	 * keySpec identifies the specification class in which the key material
	 * should be returned. It could, for example, be DSAPublicKeySpec.class,
	 * to indicate that the key material should be returned in an instance
	 * of the DSAPublicKeySpec class. 
	 *
	 * @param key	the key
	 * @param keySpec	the specification class in which the key material 
	 * 	should be returned.
	 */
	public KeySpec getKeySpec(
		Key	key,
		Class	keySpec)
	{
		return engineGetKeySpec(key, keySpec);
	}

/*
	public Provider getProvider()
*/

	/**
	 * Translates a key object, whose provider may be unknown or
	 * potentially untrusted, into a corresponding key object of this key
	 * factory. 
	 */
	public Key translateKey(
		Key	key)
	{
		return engineTranslateKey(key);
	}

	/**
	 * SPI: Generates a private key object from the provided key
	 * specification (key material). 
	 *
	 * @exception InvalidKeySpecException - if the given key
	 * 	specification is inappropriate for this key factory to
	 * 	produce a private key.
	 */
	protected abstract PrivateKey engineGeneratePrivate(KeySpec keySpec)
		throws InvalidKeySpecException;

	/**
	 * SPI: Generates a public key object from the provided key
	 * specification (key material). 
	 *
	 * @exception InvalidKeySpecException - if the given key
	 * 	specification is inappropriate for this key factory to
	 * 	produce a public key.
	 */
	protected abstract PublicKey engineGeneratePublic(KeySpec keySpec)
		throws InvalidKeySpecException;

	/**
	 * SPI: Returns a specification (key material) of the given key object.
	 * keySpec identifies the specification class in which the key material
	 * should be returned. It could, for example, be DSAPublicKeySpec.class,
	 * to indicate that the key material should be returned in an instance
	 * of the DSAPublicKeySpec class. 
	 */
	protected abstract KeySpec engineGetKeySpec(Key key, Class keySpec);

	/**
	 * SPI: Translates a key object, whose provider may be unknown or
	 * potentially untrusted, into a corresponding key object of this key
	 * factory. 
	 */
	protected abstract Key engineTranslateKey(Key key);
}
