/*
 * $Id: KeyFactory.java,v 1.9 1998/10/27 05:11:12 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/KeyFactory.java,v $
 * $Revision: 1.9 $
 * $Date: 1998/10/27 05:11:12 $
 * $State: Exp $
 */

package au.net.aba.security;

import java.security.*;
import au.net.aba.security.spec.*;

/**
 * A factory class for creating Keys.
 */
public class KeyFactory
{
	public final static String ident = "$Id: KeyFactory.java,v 1.9 1998/10/27 05:11:12 leachbj Exp $";

	KeyFactorySpi	keyFacSpi;
	Provider	provider;
	String		algorithm;

	/**
	 * Creates a KeyFactory object.
	 * 
	 * @param keyFacSpi the delegate
	 * @param provider the provider
	 * @param algorithm the algorithm
	 */
	protected KeyFactory(
		KeyFactorySpi	keyFacSpi,
		Provider	provider,
		String		algorithm)
	{
		this.keyFacSpi = keyFacSpi;
		this.provider = provider;
		this.algorithm = algorithm;
	}

	/**
	 * create an instance of a key factory object given the provider.
	 *
	 * @param prov the provider object.
	 * @param algorithm the algorithm we want.
	 */

	/**
	 * Generates a KeyFactory object for the specified algorithm
	 * from the specified provider.
	 * @param algorithm - the name of the requested key algorithm.
	 * 	See Appendix A in the Java Cryptography Architecture API
	 * 	Specification & Reference for information about standard
	 * 	algorithm names.
	 * @param provider - the name of the provider.
	 * @returns a KeyFactory object for the specified algorithm.
	 * @exception NoSuchAlgorithmException - if the algorithm is not
	 * 	available from the specified provider.
	 * @exception NoSuchProviderException - if the provider has not
	 * 	been configured.
	 * @see Provider
	 */
	private static final KeyFactory getInstance(
		String		algorithm,
		Provider	prov)
		throws NoSuchAlgorithmException
	{
		KeyFactory	fact;
		KeyFactorySpi	spi;
		String		className;

		className = prov.getProperty("KeyFactory." + algorithm);
		if (className == null)
		{
			throw new NoSuchAlgorithmException(algorithm);
		}

		try
		{
			spi = (KeyFactorySpi)Class.forName(
						className).newInstance();
		}
		catch (Exception e)
		{
			throw new NoSuchAlgorithmException(
						algorithm + ": " + e);
		}

		return new KeyFactory(spi, prov, algorithm);
	}

	/**
	 * Generates a KeyFactory object that implements the specified digest
	 * algorithm. If the default provider package provides an
	 * implementation of the requested digest algorithm, an instance of
	 * KeyFactory containing that implementation is returned. If the
	 * algorithm is not available in the default package, other packages
	 * are searched.
	 *
	 * @param algorithm the standard name of the requested key algorithm.
	 *    See Appendix A in the Java Cryptography Extension API
	 *    Specification & Reference for information about standard
	 *    algorithm names. 
	 * @return the new KeyFactory object.
	 * @exception NoSuchAlgorithmException if the requested algorithm is
	 * not available in the default provider package or any of the other
	 * provider packages that were searched.
	 */
	public static final KeyFactory getInstance(
		String algorithm)
		throws NoSuchAlgorithmException
	{
		Provider[] prov = Security.getProviders();

		for (int i = 0; i != prov.length; i++)
		{
			try
			{
				return getInstance(algorithm, prov[i]);
			}
			catch (Exception e)
			{
				// ignore
			}
		}

		throw new NoSuchAlgorithmException(
			"No KeyFactory could be found for algorithm "
				+ algorithm);
	}

	/**
	 * Generates a KeyFactory object for the specified algorithm from
	 * the specified provider.
	 *
         * @param algorithm the name of the requested key algorithm. See
         * Appendix A in the Java Cryptography Architecture API Specification
	 * & Reference for information about standard algorithm names. 
         * @param provider the name of the provider. 
	 * @return a KeyFactory object for the specified algorithm.
	 * @exception NoSuchAlgorithmException if the algorithm is not
	 *    available from the specified provider.
         * @exception NoSuchProviderException if the provider has not been
	 *    configured.
	 * @see Provider
	 */
	public static final KeyFactory getInstance(
		String algorithm,
		String provider)
		throws NoSuchAlgorithmException, NoSuchProviderException
	{
		Provider	prov;

		prov = Security.getProvider(provider);
		if (prov == null)
		{
			throw new NoSuchProviderException(
					"Provider " + provider + " not found");
		}

		return getInstance(algorithm, prov);
	}

	/**
	 * Returns the provider of this key factory object.
	 *
	 * @return the provider of this key factory object
	 */
	public final Provider getProvider()
	{
		return provider;
	}

	/**
	 * Returns the algorithm of this key factory object.
	 *
	 * @return the algorithm of this key factory object
	 */
	public final String getAlgorithm()
	{
		return algorithm;
	}

	/**
	 * Generates a private key object from the provided key specification
	 * (key material).
	 *
	 * @param keySpec The specification (key material of the private key.
	 * @returns the private key.
	 * @exception InvalidKeySpecException  if the given key specification
	 * 	is inappropriate for this key factory to produce a private key.
	 */
	public PrivateKey generatePrivate(
		KeySpec	keySpec)
		throws InvalidKeySpecException
	{
		return keyFacSpi.engineGeneratePrivate(keySpec);
	}

	/**
	 * Generates a public key object from the provided key
	 * specification (key material).
	 *
	 * @param keySpec - the specification (key material) of the
	 * 	public key.
	 * @returns the public key.
	 * @exception InvalidKeySpecException - if the given key
	 * 	specification is inappropriate for this key factory to
	 * 	produce a public key.
	 */
	public PublicKey generatePublic(
		KeySpec	keySpec)
		throws InvalidKeySpecException
	{
		return keyFacSpi.engineGeneratePublic(keySpec);
	}

	/**
	 * Returns a specification (key material) of the given key object.
	 * keySpec identifies the specification class in which the key material
	 * should be returned. It could, for example, be DSAPublicKeySpec.class,
	 * to indicate that the key material should be returned in an instance
	 * of the DSAPublicKeySpec class. 
	 *
	 * @param key	the key
	 * @param keySpec	the specification class in which the key material
	 * 	should be returned.
	 * @exception InvalidKeySpecException - if the requested key
	 * 	specification is inappropriate for the given key, or the given key
	 * 	cannot be processed (e.g., the given key has an unrecognized
	 * 	algorithm or format).
	 */
	public final KeySpec getKeySpec(
		Key	key,
		Class	keySpec)
		throws InvalidKeySpecException
	{
		return keyFacSpi.engineGetKeySpec(key, keySpec);
	}

	/**
	 * Translates a key object, whose provider may be unknown or
	 * potentially untrusted, into a corresponding key object of this
	 * key factory.
	 *
	 * @param key the key whose provider is unknown or untrusted.
	 * @return the translated key.
         * @exception InvalidKeyException if the given key cannot be processed
	 * by this key factory.
	 */
	public final Key translateKey(
		Key	key)
		throws InvalidKeyException
	{
		return keyFacSpi.engineTranslateKey(key);
	}
}
