/*
 * $Id: KeyFactorySpi.java,v 1.4 1998/10/26 01:46:52 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/KeyFactorySpi.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/26 01:46:52 $
 * $State: Exp $
 */
package au.net.aba.security;

import java.security.*;

import au.net.aba.security.spec.*;

/**
 *
 * This class defines the Service Provider Interface (SPI) for the KeyFactory
 * class. All the abstract methods in this class must be implemented by each
 * cryptographic service provider who wishes to supply the implementation of a
 * key factory for a particular algorithm. 
 *
 * Key factories are used to convert keys (opaque cryptographic keys of type
 * Key) into key specifications (transparent representations of the underlying
 * key material), and vice versa. 
 *
 * Key factories are bi-directional. That is, they allow you to build an
 * opaque key object from a given key specification (key material), or to
 * retrieve the underlying key material of a key object in a suitable format. 
 *
 * Multiple compatible key specifications may exist for the same key. For
 * example, a DSA public key may be specified using DSAPublicKeySpec or
 * X509EncodedKeySpec. A key factory can be used to translate between
 * compatible key specifications. 
 * 
 * A provider should document all the key specifications supported by its key
 * factory. 
 *
 * @see KeyFactory
 * @see Key
 * @see PublicKey
 * @see PrivateKey
 * @see KeySpec
 * @see DSAPublicKeySpec
 * @see X509EncodedKeySpec
 */
public abstract class KeyFactorySpi
{
	/**
	 * Generates a public key object from the provided key specification
	 * (key material).
	 *
	 * @param keySpec the specification (key material) of the public key. 
	 * @return the public key.
	 * @exception InvalidKeySpecException if the given key specification is
	 * inappropriate for this key factory to produce a public key.
	 */
	protected abstract PublicKey engineGeneratePublic(
		KeySpec keySpec)
		throws InvalidKeySpecException;

	/**
	 * Generates a private key object from the provided key specification
	 * (key * material).
	 * 
	 * @param keySpec the specification (key material) of the private key. 
	 * @return the private key.
	 * @exception InvalidKeySpecException if the given key specification is
	 * inappropriate for this key factory to produce a private key.
	 */
	protected abstract KeySpec engineGetKeySpec(
		Key key,
		Class keySpec)
		throws InvalidKeySpecException;

	/**
	 * Returns a specification (key material) of the given key object.
	 * keySpec identifies the specification class in which the key material
	 * should be returned. It could, for example, be DSAPublicKeySpec.class,
	 * to indicate that the key material should be returned in an instance
	 * of the DSAPublicKeySpec class.
	 *
	 * @param key the key. 
	 * @param keySpec the specification class in which the key material
	 * should be returned. 
	 * @return the underlying key specification (key material) in an
	 * instance of the requested specification class.
	 * @exception InvalidKeySpecException if the requested key
	 * specification is inappropriate for the given key, or the given key
	 * cannot be dealt with (e.g., the given key has an unrecognised
	 * format).
	 */
	protected abstract PrivateKey engineGeneratePrivate(
		KeySpec keySpec)
		throws InvalidKeySpecException;

	/**
	 * Translates a key object, whose provider may be unknown or potentially
	 * untrusted, into a corresponding key object of this key factory.
	 *
	 * @param key the key whose provider is unknown or untrusted. 
	 * @return the translated key.
	 * @exception InvalidKeyException if the given key cannot be processed
	 * by this key factory.
	 */
	protected abstract Key engineTranslateKey(
		Key key)
		throws InvalidKeyException;
}
