/*
 * $Id: KeyStore.java,v 1.7 1999/01/19 06:01:11 wslade Exp $
 * $Author: wslade $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/KeyStore.java,v $
 * $Revision: 1.7 $
 * $Date: 1999/01/19 06:01:11 $
 * $State: Exp $
 */

package au.net.aba.security;

import java.io.*;
import java.util.*;
import java.security.*;

import au.net.aba.security.cert.Certificate;
import au.net.aba.security.cert.CertificateException;

/**
 * This class represents an in-memory collection of keys and
 * certificates. It manages two types of entries: 
 * <ul>
 * <li>Key Entry 
 *
 *   <p>This type of keystore entry holds very sensitive cryptographic
 *   key information, which is stored in a protected format to
 *   prevent unauthorised access. 
 *
 *   <p>Typically, a key stored in this type of entry is a secret key, or a
 *   private key accompanied by the certificate chain for the
 *   corresponding public key. 
 *
 *   <p>Private keys and certificate chains are used by a given entity
 *   for self-authentication. Applications for this authentication
 *   include software distribution organisations which sign JAR
 *   files as part of releasing and/or licensing software.
 *
 * <li>Trusted Certificate Entry 
 *
 *   <p>This type of entry contains a single public key certificate
 *   belonging to another party. It is called a trusted certificate
 *   because the keystore owner trusts that the public key in the
 *   certificate indeed belongs to the identity identified by the
 *   subject (owner) of the certificate. 
 *
 *   <p>This type of entry can be used to authenticate other parties. 
 * </ul>
 * Each entry in a keystore is identified by an "alias" string. In the case
 * of private keys and their associated certificate chains, these strings
 * distinguish among the different ways in which the entity may
 * authenticate itself. For example, the entity may authenticate itself
 * using different certificate authorities, or using different public key
 * algorithms. 
 * <p>
 * Whether keystores are persistent, and the mechanisms used by the
 * keystore if it is persistent, are not specified here. This allows use of a
 * variety of techniques for protecting sensitive (e.g., private or secret)
 * keys. Smart cards or other integrated cryptographic engines
 * (SafeKeyper) are one option, and simpler mechanisms such as files
 * may also be used (in a variety of formats). 
 * <p>
 * There are two ways to request a KeyStore object: by specifying
 * either just a keystore type, or both a keystore type and a package
 * provider. 
 * 
 * <ul>
 *   <li>If just a keystore type is specified: 
 *
 *   <pre>      KeyStore ks = KeyStore.getInstance("JKS");</pre>
 *    
 *
 *   the system will determine if there is an implementation of the
 *   keystore type requested available in the environment, and if
 *   there is more than one, if there is a preferred one.
 *
 *   <li>If both a keystore type and a package provider are specified: 
 *
 *   <pre>      KeyStore ks = KeyStore.getInstance("JKS", "SUN");</pre>
 *    
 *
 *   the system will determine if there is an implementation of the
 *   keystore type in the package requested, and throw an exception
 *   if there is not. 
 * </ul>
 * <p>
 * Before a keystore can be accessed, it must be loaded. In order to
 * create an empty keystore, you pass null as the InputStream
 * argument to the load method. 
 * 
 * Since: JDK1.2 
 * @see PrivateKey
 * @see Certificate
 */
public class KeyStore
{
	private KeyStoreSpi	spi;
	private Provider	provider;
	private String		type;

	/**
	 * Creates a KeyStore object of the given type, and encapsulates
	 * the given provider implementation (SPI object) in it.
	 *
	 * @param keyStoreSpi the provider implementation.
	 * @param provider the provider.
	 * @param type the keystore type.
	 */
	protected KeyStore(
		KeyStoreSpi	spi,
		Provider	provider,
		String		type)
	{
		this.spi = spi;
		this.provider = provider;
		this.type = type;
	}

	/**
	 * create an instance of a key store object given the provider.
	 *
	 * @param prov the provider object.
	 * @param type the type we want.
	 */
	private static final KeyStore getInstance(
		String		type,
		Provider	prov)
		throws KeyStoreException
	{
		KeyStore	fact;
		KeyStoreSpi	spi;
		String		className;

		className = prov.getProperty("KeyStore." + type);
		if (className == null)
		{
			throw new KeyStoreException(type);
		}

		try
		{
			spi = (KeyStoreSpi)Class.forName(
						className).newInstance();
		}
		catch (Exception e)
		{
			throw new KeyStoreException(type + ": " + e);
		}

		return new KeyStore(spi, prov, type);
	}

	/**
	 * Generates a keystore object of the given type. 
	 * 
	 * If the default provider package provides a keystore
	 * implementation of the given type, an instance of KeyStore
	 * containing that implementation is returned. If the requested
	 * keystore type is not available in the default package, other
	 * packages are searched.
	 *
	 * @param type the type of keystore. See Appendix A in the Java
	 *     Cryptography Architecture API Specification &
	 *     Reference for information about standard keystore types.
	 * @return a keystore object of the specified type.
	 * @exception KeyStoreException if the requested keystore type is
	 *     not available in the default provider package or any of
	 *     the other provider packages that were searched.
	 */
	public static final KeyStore getInstance(
		String type)
		throws KeyStoreException
	{
		Provider[] prov = Security.getProviders();

		for (int i = 0; i != prov.length; i++)
		{
			try
			{
				return getInstance(type, prov[i]);
			}
			catch (Exception e)
			{
				// ignore
			}
		}

		throw new KeyStoreException(
			"No KeyStore of type " + type + " could be found.");
	}

	/**
	 * Generates a keystore object for the specified keystore type
	 * from the specified provider.
	 *
	 * @param type the type of keystore. See Appendix A in the Java
	 *     Cryptography Architecture API Specification &
	 *     Reference for information about standard keystore types.
	 * @param provider the name of the provider.
	 * @return a keystore object of the specified type, as supplied by the
	 *     specified provider.
	 * @exception KeyStoreException if the requested keystore type is
	 *     not available from the provider.
	 * @exception NoSuchProviderException if the provider has not been
	 *     configured.
	 * @see Provider
	 */
	public static final KeyStore getInstance(
		String type,
		String provider)
		throws KeyStoreException, NoSuchProviderException
	{
		Provider	prov;

		prov = Security.getProvider(provider);
		if (prov == null)
		{
			throw new NoSuchProviderException(
					"Provider " + provider + " not found");
		}

		return getInstance(type, prov);
	}

	/**
	 * Returns the provider of this keystore.
	 *
	 * @return the provider of this keystore.
	 */
	public final Provider getProvider()
	{
		return provider;
	}

	/**
	 * Returns the type of this keystore.
	 *
	 * @return the type of this keystore.
	 */
	public final String getType()
	{
		return type;
	}

	/**
	 * Returns the key associated with the given alias, using the given
	 * password to recover it.
	 *
	 * @param alias the alias name
	 * @param password the password for recovering the key
	 * @return the requested key, or null if the given alias does not exist
	 *     or does not identify a key entry.
	 * @exception KeyStoreException if the keystore has not been
	 *     initialised (loaded).
	 * @exception NoSuchAlgorithmException if the algorithm for
	 *     recovering the key cannot be found
	 * @exception UnrecoverableKeyException if the key cannot be
	 *     recovered (e.g., the given password is wrong).
	 */
	public final Key getKey(
		String	alias,
		char[]	password)
		throws KeyStoreException,
			NoSuchAlgorithmException, UnrecoverableKeyException
	{
		return spi.engineGetKey(alias, password);
	}

	/**
	 * Returns the certificate chain associated with the given alias.
	 *
	 * @param alias the alias name
	 * @return the certificate chain (ordered with the user's certificate
	 *    first and the root certificate authority last), or null if the
	 *    given alias does not exist or does not contain a certificate
	 *    chain (i.e., the given alias identifies either a trusted
	 *    certificate entry or a key entry without a certificate
	 *    chain).
	 * @exception KeyStoreException if the keystore has not been
	 *    initialised (loaded).
	 */
	public final Certificate[] getCertificateChain(
		String	alias)
		throws KeyStoreException
	{
		return spi.engineGetCertificateChain(alias);
	}

	/**
	 * Returns the certificate associated with the given alias. 
	 * <p>
	 * If the given alias name identifies a trusted certificate entry, the
	 * certificate associated with that entry is returned. If the given
	 * alias name identifies a key entry, the first element of the
	 * certificate chain of that entry is returned, or null if that entry
	 * does not have a certificate chain.
	 *
	 * @param alias - the alias name
	 * @return the certificate, or null if the given alias does not exist or
	 * does not contain a certificate.
	 * @exception KeyStoreException - if the keystore has not been
	 * initialised (loaded).
	 */
	public final Certificate getCertificate(
		String	alias)
		throws KeyStoreException
	{
		return spi.engineGetCertificate(alias);
	}

	/**
	 * Returns the creation date of the entry identified by the given
	 * alias.
	 * 
	 * @param alias the alias name
	 * @return the creation date of this entry, or null if the given alias
	 *    does not exist
         * @exception KeyStoreException - if the keystore has not been
	 *    initialised (loaded).
	 */
	public final Date getCreationDate(
		String	alias)
		throws KeyStoreException
	{
		return spi.engineGetCreationDate(alias);
	}

	/**
	 * Assigns the given key to the given alias, protecting it with the
	 * given password. 
	 * <p>
	 * If the given key is of type java.security.PrivateKey, it
	 * must be accompanied by a certificate chain certifying the
	 * corresponding public key. 
	 * <p>
	 * If the given alias already exists, the keystore information
	 * associated with it is overridden by the given key (and possibly
	 * certificate chain).
	 *
	 * @param alias the alias name
	 * @param key the key to be associated with the alias
	 * @param password the password to protect the key
	 * @param chain the certificate chain for the corresponding
	 *     public key (only required if the given key is of type
	 *     java.security.PrivateKey).
	 * @exception KeyStoreException if the keystore has not been
	 *     initialised (loaded), the given key cannot be protected, or
	 *     this operation fails for some other reason
	 */
	public final void setKeyEntry(
		String		alias,
		Key		key,
		char[]		password,
		Certificate[]	chain)
		throws KeyStoreException
	{
		spi.engineSetKeyEntry(alias, key, password, chain);
	}

	/**
	 * Assigns the given key (that has already been protected) to the
	 * given alias. 
	 * <p>
	 * If the protected key is of type java.security.PrivateKey,
	 * it must be accompanied by a certificate chain certifying the
	 * corresponding public key. 
	 * <p>
	 * If the given alias already exists, the keystore information
	 * associated with it is overridden by the given key (and possibly
	 * certificate chain).
	 * 
	 * @param alias - the alias name
	 * @param key - the key (in protected format) to be associated with
	 *     the alias
	 * @param chain - the certificate chain for the corresponding
	 *     public key (only useful if the protected key is of type
	 *     java.security.PrivateKey).
	 * @exception KeyStoreException if the keystore has not been
	 *    initialised (loaded), or if this operation fails for some
	 *    other reason.
	 */
	public final void setKeyEntry(
		String		alias,
		byte[]		key,
		Certificate[]	chain)
		throws KeyStoreException
	{
		spi.engineSetKeyEntry(alias, key, chain);
	}

	/**
	 * Assigns the given certificate to the given alias. 
	 * <p>
	 * If the given alias already exists in this keystore and identifies a
	 * trusted certificate entry, the certificate associated with it is
	 * overridden by the given certificate.
	 *
	 * @param alias the alias name
	 * @param cert the certificate
	 * @exception KeyStoreException - if the keystore has not been
	 * initialised, or the given alias already exists and does not
	 * identify a trusted certificate entry, or this operation fails
	 * for some other reason.
	 */
	public final void setCertificateEntry(
		String		alias,
		Certificate	cert)
		throws KeyStoreException
	{
		spi.engineSetCertificateEntry(alias, cert);
	}

	/**
	 * Deletes the entry identified by the given alias from this
	 * keystore.
	 *
	 * @param alias - the alias name
	 * @exception KeyStoreException - if the keystore has not been
	 *     initialised, or if the entry cannot be removed.
	 */
	public final void deleteEntry(
		String alias)
		throws KeyStoreException
	{
		spi.engineDeleteEntry(alias);
	}

	/**
	 * Lists all the alias names of this keystore.
	 *
	 * @return enumeration of the alias names
	 * @exception KeyStoreException if the keystore has not been
	 *     initialised (loaded).
	 */
	public final Enumeration aliases()
		throws KeyStoreException
	{
		return spi.engineAliases();
	}


	/**
	 * Checks if the given alias exists in this keystore.
	 *
	 * @param alias the alias name
	 * @return true if the alias exists, false otherwise
	 * @exception KeyStoreException if the keystore has not been
	 *     initialised (loaded).
	 */
	public final boolean containsAlias(
		String alias)
		throws KeyStoreException
	{
		return spi.engineContainsAlias(alias);
	}

	/**
	 * Retrieves the number of entries in this keystore.
	 *
	 * @return the number of entries in this keystore
	 * @exception KeyStoreException if the keystore has not been
	 *    initialised (loaded).
	 */
	public final int size()
		throws KeyStoreException
	{
		return spi.engineSize();
	}

	/**
	 * Returns true if the entry identified by the given alias is a key
	 * entry, and false otherwise.
	 *
	 * @return true if the entry identified by the given alias is a key
	 * entry, false otherwise.
	 * @exception KeyStoreException if the keystore has not been
	 * initialised (loaded).
	 */
	public final boolean isKeyEntry(
		String alias)
		throws KeyStoreException
	{
		return spi.engineIsKeyEntry(alias);
	}

	/**
	 * Returns true if the entry identified by the given alias is a
	 * trusted certificate entry, and false otherwise.
	 *
	 * @return true if the entry identified by the given alias is a trusted
	 * certificate entry, false otherwise.
	 * @exception KeyStoreException if the keystore has not been
	 * initialised (loaded).
	 */
	public final boolean isCertificateEntry(
		String	alias)
		throws KeyStoreException
	{
		return spi.engineIsCertificateEntry(alias);
	}

	/**
	 * Returns the (alias) name of the first keystore entry whose
	 * certificate matches the given certificate. 
	 * <p>
	 * This method attempts to match the given certificate with each
	 * keystore entry. If the entry being considered is a trusted
	 * certificate entry, the given certificate is compared to that
	 * entry's certificate. If the entry being considered is a key entry,
	 * the given certificate is compared to the first element of that
	 * entry's certificate chain (if a chain exists).
	 *
	 * @param cert the certificate to match with.
	 * @return the (alias) name of the first entry with matching
	 * certificate, or null if no such entry exists in this
	 * keystore.
	 * @exception KeyStoreException - if the keystore has not been
	 * initialised (loaded).
	 */
	public final String getCertificateAlias(
		Certificate	cert)
		throws KeyStoreException
	{
		return spi.engineGetCertificateAlias(cert);
	}


	/**
	 * Stores this keystore to the given output stream, and protects its
	 * integrity with the given password.
	 *
	 * @param stream - the output stream to which this keystore is
	 *     written.
	 * @param password - the password to generate the keystore
	 *     integrity check
	 *
	 * @exception KeyStoreException - if the keystore has not been
	 *     initialised (loaded).
	 * @exception IOException - if there was an I/O problem with data
	 * @exception NoSuchAlgorithmException - if the appropriate data
	 *     integrity algorithm could not be found
	 * @exception CertificateException - if any of the certificates included
	 *     in the keystore data could not be stored
	 */
	public final void store(
		OutputStream	stream,
		char[]		password)
		throws KeyStoreException, IOException,
                        NoSuchAlgorithmException, CertificateException
	{
		spi.engineStore(stream, password);
	}

	/**
	 * Loads the keystore from the given input stream. 
	 * <p>
	 * If a password is given, it is used to check the integrity of the
	 * keystore data. Otherwise, the integrity of the keystore is not
	 * checked. 
	 * <p>
	 * In order to create an empty keystore, you pass null as the
	 * stream argument.
	 *
	 * @param stream the input stream from which the keystore is
	 *    loaded, or null if an empty keystore is to be created.
	 * @param password the (optional) password used to check the
	 *    integrity of the keystore.
	 * @exception IOException if there is an I/O or format problem with
	 *    the keystore data
	 * @exception NoSuchAlgorithmException if the algorithm used to
	 *    check the integrity of the keystore cannot be found
	 * @exception CertificateException if any of the certificates in the
	 *    keystore could not be loaded
	 */
	public final void load(
		InputStream	stream,
		char[]		password)
		throws IOException, NoSuchAlgorithmException,
			CertificateException
	{
		spi.engineLoad(stream, password);
	}

	/**
	 * Returns the default keystore type as specified in the Java
	 * security properties file, or the string "jks" (acronym for "Java
	 * keystore") if no such property exists. The Java security
	 * properties file is located in the file named
	 * <JAVA_HOME>/lib/security/java.security, where
	 * <JAVA_HOME> refers to the directory where the JDK was
	 * installed. 
	 * <p>
	 * The default keystore type can be used by applications that do
	 * not want to use a hard-coded keystore type when calling one
	 * of the getInstance methods, and want to provide a default
	 * keystore type in case a user does not specify its own. 
	 * <p>
	 * The default keystore type can be changed by setting the value
	 * of the "keystore.type" security property (in the Java security
	 * properties file) to the desired keystore type.
	 */
	public static final String getDefaultType()
	{
		String	sType;

		sType = Security.getProperty("keystore.type");
		if (sType == null)
		{
			return "jks";
		}

		return sType;
	}
}
