/*
 * $Id: MessageDigest.java,v 1.8 1998/10/27 05:10:47 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.security/MessageDigest.java,v $
 * $Revision: 1.8 $
 * $Date: 1998/10/27 05:10:47 $
 * $State: Exp $
 */

package au.net.aba.security;

/**
 * The base class for a message digest.
 */
public abstract class MessageDigest
{
	public final static String ident = "$Id: MessageDigest.java,v 1.8 1998/10/27 05:10:47 leachbj Exp $";

	private String algorithm;

	/**
	 * Creates a message digest with the specified algorithm name. 
	 */
	public MessageDigest(
		String	name)
	{
		algorithm = name;
	}

	/**
	 * Generates a MessageDigest object that implements the specified
	 * digest algorithm. If the default provider package contains a 
	 * MessageDigest subclass implementing the algorithm, an instance of
	 * that subclass is returned. If the algorithm is not available in the
	 * default package, other packages are searched. 
	 *
	 * @param algorithm - the name of the algorithm requested.  See
	 * 	Appendix A in the Java Cryptography Architecture API Specification
	 * 	& Reference for information about standard algorithm names.
	 * @returns a Message Digest object implementing the specified algorithm.
	 * @exception NoSuchAlgorithmException - if the algorithm is not
	 * 	available in the caller's environment.
	 */
	public static MessageDigest getInstance(
		String algorithm)
	throws NoSuchAlgorithmException
	{
		Object	obj;
		String	name;

		if (algorithm.equals("SHA"))
		{
			algorithm = "SHA1";
		}

		name = "au.net.aba.crypto.provider." + algorithm;

		try
		{
			obj = Class.forName(name).newInstance();
		}
		catch (Exception e)
		{
			throw new NoSuchAlgorithmException(name + " : " + e);
		}

		return (MessageDigest)obj;
	}

	/**
	 * Generates a MessageDigest object implementing the specified
	 * algorithm, as supplied from the specified provider, if such an
	 * algorithm is available from the provider.  Only the ABA provider
	 * ("ABA") is supported.
	 *
	 * @param algorithm - the name of the algorithm requested.  See
	 * 	Appendix A in the Java Cryptography Architecture API Specification
	 * 	& Reference for information about standard algorithm names.
	 * @returns a Message Digest object implementing the specified algorithm.
	 * @exception NoSuchAlgorithmException - if the algorithm is not
	 * 	available in the caller's environment.
	 * @exception NoSuchProviderException - if the provider is not
	 * 	available in the caller's environment (only ABA is supported).
	 */
	public static MessageDigest getInstance(
		String	algorithm,
		String	provider)
	throws NoSuchAlgorithmException, NoSuchProviderException
	{
		if (!provider.equals("ABA"))
		{
			throw new NoSuchProviderException("ABA only provider");
		}

		return getInstance(algorithm);
	}

	/**
	 * Updates the digest using the specified byte.
	 */
	public void update(
		byte	input)
	{
		engineUpdate(input);
	}

	/**
	 * Updates the digest using the specified array of bytes, starting at
	 * the specified offset.
	 */
	public void update(
		byte[]	input,
		int	offset,
		int	len)
	{
		engineUpdate(input, offset, len);
	}

	/**
	 * Updates the digest using the specified array of bytes.
	 */
	public void update(
		byte[]	input)
	{
		engineUpdate(input, 0, input.length);
	}

	/**
	 * Completes the hash computation by performing final operations such
	 * as padding. The digest is reset after this call is made. 
	 */
	public byte[] digest()
	{
		return engineDigest();
	}

	/**
	 * Performs a final update on the digest using the specified array of
	 * bytes, then completes the digest computation. That is, this method
	 * first calls update on the array, then calls digest(). 
	 */
	public byte[] digest(
		byte[]	input)
	{
		update(input);
		return digest();
	}

	/**
	 * Compares two digests for equality. Does a simple byte compare.
	 */
	public static boolean isEqual(
		byte[]	digesta,
		byte[]	digestb)
	{
		if (digesta.length != digestb.length)
			return false;

		for (int i = 0; i < digesta.length; i++)
			if (digesta[i] != digestb[i])
				return false;

		return true;
	}

	/**
	 * Resets the digest for further use.
	 */
	public void reset()
	{
		engineReset();
	}

	/**
	 * Returns a string that identifies the algorithm, independent of
	 * implementation details. The name should be a standard Java Security
	 * name (such as "SHA", "MD5", and so on). See Appendix A in the Java
	 * Cryptography Architecture API Specification & Reference for
	 * information about standard algorithm names. 
	 */
	public final String getAlgorithm()
	{
		return algorithm;
	}

	/**
	 * Returns a clone if the implementation is cloneable.
	 * @exception CloneNotSupportedException - if this is called on an
	 * 	implementation that does not support Cloneable.
	 */
	public Object clone()
		throws CloneNotSupportedException
	{
		if (this instanceof Cloneable)
			return super.clone();
		else
			throw new CloneNotSupportedException();
	}

	/**
	 * SPI: Updates the digest using the specified byte.
	 */
	protected abstract void engineUpdate(byte input);

	/**
	 * SPI: Updates the digest using the specified array of bytes, starting
	 * at the specified offset.  This should be a no-op if the digest has
	 * been finalised. 
	 */
	protected abstract void engineUpdate(byte[] input, int off, int len);

	/**
	 * SPI: Completes the hash computation by performing final operations
	 * such as padding. Once engineDigest has been called, the engine
	 * should be reset (see reset). Resetting is the responsibility of the
	 * engine implementor. 
	 */
	protected abstract byte[] engineDigest();

	/**
	 * SPI: Resets the digest for further use.
	 */
	protected abstract void engineReset();
}
