/*
 * $Id: SecureRandom.java,v 1.11 1999/01/13 01:21:20 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba.security/SecureRandom.java,v $
 * $Revision: 1.11 $
 * $Date: 1999/01/13 01:21:20 $
 * $State: Exp $
 */

package au.net.aba.security;

import java.util.Random;

/**
 * This class provides a cryptographically strong pseudo-random number
 * generator based on the SHA-1 hash algorithm.
 *
 * The calls inherited from Random will be implemented in terms of the
 * strengthened functionality.
 *
 * NOTE: For seed generator (ie the default constructor) we only use
 * a very small number of random bytes using an entropy generator of
 * unknown quality.  For secure results further entropy should be
 * added to the generator.
 */
public final class SecureRandom extends Random
{
	public final static String ident = "$Id: SecureRandom.java,v 1.11 1999/01/13 01:21:20 leachbj Exp $";

	//==================================
	// Constructors
	//==================================

	/**
	 * This empty constructor automatically seeds the generator. We
	 * attempt to provide sufficient seed bytes to completely randomise
	 * the internal state of the generator (20 bytes). Note, however,
	 * that our seed generation algorithm has not been thoroughly
	 * studied or widely deployed. It relies on counting the number of
	 * times that the calling thread can yield while waiting for
	 * another thread to sleep for a specified interval.
	 */
	public SecureRandom()
	{
		this(nextSeed());
	}

	/**
	 * This constructor uses a user-provided seed in preference to the
	 * self-seeding algorithm referred to in the empty constructor
	 * description. It may be preferable to the empty constructor if
	 * the caller has access to high-quality random bytes from some
	 * physical device (for example, a radiation detector or a noisy
	 * diode).
	 *
	 * @param seed the seed for this generator
	 */
	public SecureRandom(
		byte seed[])
	{
		super(0);

		try
		{
			digest = MessageDigest.getInstance("SHA", "ABA");
		}
		catch (NoSuchAlgorithmException e)
		{
			throw new RuntimeException("SHA unavailable.");
		}
		catch (NoSuchProviderException e)
		{
			throw new RuntimeException("ABA's SHA unavailable.");
		}

		setSeed(seed);
	}

	//==================================
	// Public Interface
	//==================================

	/**
	 * Re-seeds this random object. The given seed supplements, rather *
	 * than replaces, the existing seed. Thus, repeated calls are
	 * guaranteed never to reduce randomness.
	 *
	 * @param seed		A series of bytes to seed with.
	 */
	public synchronized void setSeed(
		byte seed[])
	{
		// accumulate our state
		if (state != null)
		{
			digest.update(state);
		}

		// hash in the new seed information
		state = digest.digest(seed);
	}

	/**
	 * Re-seeds this random object, using the eight bytes contained in
	 * the given long seed. The given seed supplements, rather than
	 * replaces, the existing seed. Thus, repeated calls are guaranteed
	 * never to reduce randomness.
	 *
	 * This method is defined for compatibility with java.util.Random.
	 *
	 * @param l		the seed.
	 */
	public void setSeed(
		long l)
	{
		if (l != 0)
		{
			setSeed(longToByteArray(l));
		}
	}

	/**
	 * Generates a user-specified number of random bytes. This method
	 * is used as the basis of all random entities returned by this
	 * class (except seed bytes).  Thus, it may be overridden to change
	 * the behaviour of the class.
	 *
	 * @param bytes		the array to be filled in with random bytes.
	 */
	public synchronized void nextBytes(
		byte[] bytes)
	{
		for (int i=0; i < bytes.length; i++)
		{
			if (randBytes == null
				|| bytesUsed == randBytes.length)
			{
				digest.update(state);
				randBytes = digest.digest(longToByteArray(counter++));
				bytesUsed = 0;
			}

			bytes[i] = randBytes[bytesUsed++];
		}
	}

	/**
	 * Generates an integer containing the user-specified number of
	 * pseudo-random bits (right justified, with leading zeros). This
	 * method overrides a java.util.Random method, and serves to
	 * provide a source of random bits to all of the methods inherited
	 * from that class (for example, nextInt, nextLong, and
	 * nextFloat).
	 *
	 * @param numBits - number of pseudo-random bits to be generated,
	 * where 0 <= numBits <= 32.
	 */
	protected final int next(int numBits)
	{
		int nBytes = (numBits / 8) + 1;
		byte[] bytes = new byte[nBytes];

		nextBytes(bytes);

		// compress the bytes into an integer
		int result = 0;
		for (int i = 0; i < nBytes; i++)
		{
			result = (result << 8) + (bytes[i] & 255);
		}

		// strip off any extra round-off bits
		return result >> nBytes * 8 - numBits;
	}

	/**
	 * Generate random bytes. This is a poor mans version of what
	 * is available in the JDK 1.1 and should be chucked as soon as
	 * possible. We allocate a minimum of 1 second for each loop, adding
	 * some number between 0 and 127 milliseconds at each iteration.
	 * Note: this will give us much greater protection than we get from
	 * just using the date, don't rely on it as the only source of 
	 * randomness though.
	 */
	static public byte[] getSeed(
		int numBytes)
	{
		byte[] bytes = new byte[numBytes];
		int timerSleep = BASE_SLEEP;

		for (int i = 0; i != numBytes; i++)
		{
			Timer timer = new Timer();
			timer.start();

			try
			{
				Thread.sleep(timerSleep);
			}
			catch (InterruptedException e)
			{
				// ignored
			}

			timer.stopTimer();

			int count = timer.getCount();
			timerSleep = BASE_SLEEP + (count & 0x7f);

			bytes[i] = (byte)count;
		}

		return bytes;
	}

	//==================================
	// Private Implementation
	//==================================

	/*
	 * The internal digest buffer.
	 */
	private MessageDigest digest;

	/*
	 * The state of a the last digest.
	 */
	private byte[] state;

	/*
	 * Our random byte buffer.
	 */
	private byte[] randBytes;
	private int bytesUsed;

	/*
	 * Counter to update state hash with
	 */
	private long counter;

	/*
	 * time our random number generator sleeps
	 */
	private static final int BASE_SLEEP = 1000;

	/*
	 * Seed generator random number generator
	 */
	private static SecureRandom seedGenerator;

	/*
	 * The number of bytes we use for our seed generation.
	 * NOTE: We only use a very small number, so we assume
	 * that other entropy will be introduced.
	 */
	final private static int SEEDSIZE = 4;

	private static synchronized byte[] nextSeed()
	{
		if (seedGenerator == null)
		{
			seedGenerator = new SecureRandom(
				getSeed(SecureRandom.SEEDSIZE));
		}

		byte[] seed = new byte[SecureRandom.SEEDSIZE];
		seedGenerator.nextBytes(seed);

		return seed;
	}

	private byte[] longToByteArray(long l)
	{
		byte[] b = new byte[8];
		for (int i = 0; i < 8; i++)
		{
			b[i] = (byte)l;
			l >>= 8;
		}

		return b;
	}
}


/**
 * a thread for our random byte generator to yield to.
 */
class Timer extends Thread
{
	private int	counter;
	private boolean	running;

	/**
	 * basic constructor
	 */
	public Timer()
	{
		this.counter = 1;
		this.running = true;
	}

	/*
	 * all we do here is sleep for our time.
	 */
	public void run()
	{
		while (running)
		{
			counter += 1;
			Thread.yield();
		}
	}

	public void stopTimer()
	{
		running = false;
	}

	int getCount()
	{
		return counter;
	}
}

