/*
 * $Id: SignedObject.java,v 1.5 1998/10/26 01:46:53 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/SignedObject.java,v $
 * $Revision: 1.5 $
 * $Date: 1998/10/26 01:46:53 $
 * $State: Exp $
 */

package au.net.aba.security;

import java.io.*;
import java.security.*;

/**
 * <p>SignedObject is a class for the purpose of creating authentic
 * runtime objects whose integrity cannot be compromised without being
 * detected.
 *
 * <p>More specifically, a SignedObject contains another Serializable
 * object, the (to-be-)signed object and its signature.
 *
 * <p>The signed object is a "deep copy" (in serialised form) of an
 * original object. Once the copy is made, further manipulation of the
 * original object has no side effect on the copy.
 *
 * <p>The underlying signing algorithm is designated by the Signature
 * object passed to the constructor and the verify method. A typical
 * usage for signing is the following:
 *
 * <pre>
 * Signature signingEngine = Signature.getInstance(algorithm,
 *                                                 provider);
 * SignedObject so = new SignedObject(myobject, signingKey,
 *                                    signingEngine);
 * </pre>
 *
 * <p>A typical usage for verification is the following (having
 * received SignedObject so):
 *
 * <pre>
 * Signature verificationEngine =
 *     Signature.getInstance(algorithm, provider);
 * if (so.verify(publickey, verificationEngine))
 *     try {
 *         Object myobj = so.getObject();
 *     } catch (java.lang.ClassNotFoundException e) {};
 * </pre>
 *
 * <p>Several points are worth noting. First, there is no need to
 * initialise the signing or verification engine, as it will be
 * re-initialised inside the constructor and the verify method.
 * Secondly, for verification to succeed, the specified public key
 * must be the public key corresponding to the private key used to
 * generate the SignedObject.
 *
 * <p>More importantly, for flexibility reasons, the constructor and
 * verify method allow for customised signature engines, which can
 * implement signature algorithms that are not installed formally as
 * part of a crypto provider. However, it is crucial that the
 * programmer writing the verifier code be aware what Signature engine
 * is being used, as its own implementation of the verify method is
 * invoked to verify a signature. In other words, a malicious
 * Signature may choose to always return true on verification in an
 * attempt to bypass a security check.
 *
 * <p>The signature algorithm can be, among others, the NIST standard
 * DSA, using DSA and SHA-1.  The algorithm is specified using the
 * same convention as that for signatures. The DSA algorithm using the
 * SHA-1 message digest algorithm can be specified, for example, as
 * "SHA/DSA" or "SHA-1/DSA" (they are equivalent). In the case of RSA,
 * there are multiple choices for the message digest algorithm, so the
 * signing algorithm could be specified as, for example, "MD2/RSA",
 * "MD5/RSA" or "SHA-1/RSA". The algorithm name must be specified, as
 * there is no default.
 *
 * <p>The name of the Cryptography Package Provider is designated also
 * by the Signature parameter to the constructor and the verify
 * method. If the provider is not specified, the default provider is
 * used.  Each installation can be configured to use a particular
 * provider as default.
 *
 * <p>Potential applications of SignedObject include: 
 *
 * <ul>
 * <li>It can be used internally to any Java runtime as an unforgeable
 * authorisation token -- one that can be passed around without the
 * fear that the token can be maliciously modified without being
 * detected. 
 *
 * <li>It can be used to sign and serialise data/object for storage
 * outside the Java runtime (e.g., storing critical access control
 * data on disk). 
 *
 * <li>Nested SignedObjects can be used to construct a logical
 * sequence of signatures, resembling a chain of authorisation and
 * delegation. 
 * </ul>
 *
 * @see Signature
 */
public class SignedObject
	extends Object
	implements Serializable
{
	public final static String ident = "$Id: SignedObject.java,v 1.5 1998/10/26 01:46:53 leachbj Exp $";

	private byte[] object;
	private byte[] signature;
	private String algorithm;

	/**
	 * Constructs a SignedObject from any Serializable object.
	 * The given object is signed with the given signing key,
	 * using the designated signature engine. 
	 *
    * @param object - the object to be signed. 
	 * @param signingKey - the private key for signing. 
	 * @param signingEngine - the signature signing engine. 
	 * @exception IOException if an error occurs during serialisation
	 * @exception InvalidKeyException if the key is invalid.
	 * @exception SignatureException if signing fails.
	 */
	public SignedObject(Serializable object,
		PrivateKey signingKey,
		Signature signingEngine)
	throws IOException,
		InvalidKeyException,
		SignatureException
	{
		this.algorithm = signingEngine.getAlgorithm();

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		ObjectOutputStream os = new ObjectOutputStream(bos);
		os.writeObject(object);
		os.close();
		this.object = bos.toByteArray();

		signingEngine.initSign(signingKey);
		signingEngine.update(this.object, 0, this.object.length);
		this.signature = signingEngine.sign();
	}

	/**
	 * Retrieves the encapsulated object. The encapsulated
	 * object is de-serialised before it is returned. 
	 * 
	 * @return the encapsulated object.
	 * @exception IOException if an error occurs during de-serialisation
	 * @exception ClassNotFoundException if an error occurs
	 * during de-serialisation
	 */
	public Object getObject()
	throws IOException,
		ClassNotFoundException
	{
		ByteArrayInputStream bin = new ByteArrayInputStream(object);
		ObjectInputStream oin = new ObjectInputStream(bin);
		return oin.readObject();
	}

	/**
	 * Retrieves the signature on the signed object, in
	 * the form of a byte array. 
	 * 
	 * @return the signature.
	 */
	public byte[] getSignature()
	{
		return signature;
	}

	/** 
	 * Retrieves the name of the signature algorithm. 
	 * 
	 * @return the signature algorithm name.
	 */
	public String getAlgorithm()
	{
		return algorithm;
	}

	/**
	 * Verifies that the signature in this SignedObject
	 * is the valid signature for the object stored inside,
	 * with the given verification key, using the designated
	 * verification engine. 
	 *
	 * @param verificationKey - the public key for verification. 
	 * @param verificationEngine - the signature verification engine. 
	 * @exception SignatureException if signature verification failed.
	 * @exception InvalidKeyException if the verification key is invalid.
	 */
	public boolean verify(
		PublicKey verificationKey,
		Signature verificationEngine)
	throws InvalidKeyException,
		SignatureException
	{
		verificationEngine.initVerify(verificationKey);
		verificationEngine.update(this.object, 0, this.object.length);
		return verificationEngine.verify(signature);
	}
}

