/*
 * $Id: CertificateFactory.java,v 1.1 1999/01/19 05:15:00 wslade Exp $ 
 * $Author: wslade $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately.
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/cert/CertificateFactory.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/01/19 05:15:00 $
 * $State: Exp $
 */

package au.net.aba.security.cert;

import java.io.*;
import au.net.aba.util.Collection;
import java.security.*;

public class CertificateFactory 
{
	private final static String ident = "$Id"; 

    private String type;

    private Provider provider;

    private CertificateFactorySpi certFacSpi;

	/*
	 * Constructors
	 */

	/*
		Creates a CertificateFactory object of the given type, and 
		encapsulates the given provider implementation (SPI object) 
		in it.
	*/
	protected CertificateFactory(
		CertificateFactorySpi certFacSpi,
		Provider provider, 
		String type
		)
	{
		this.certFacSpi = certFacSpi;
		this.provider = provider;
		this.type = type;
	}

	/*
	 * Methods
	 */

	public final Certificate generateCertificate(InputStream inStream)
	throws CertificateException
	{
		return certFacSpi.engineGenerateCertificate(inStream);
	}

    public final Collection generateCertificates(InputStream inStream)
	throws CertificateException
    {
		return certFacSpi.engineGenerateCertificates(inStream);
    }

    public final CRL generateCRL(InputStream inStream)
	throws CRLException
    {
		return certFacSpi.engineGenerateCRL(inStream);
    }

    public final Collection generateCRLs(InputStream inStream)
	throws CRLException
    {
		return certFacSpi.engineGenerateCRLs(inStream);
    }

	/*
		Generates a certificate factory object that implements
		the specified certificate type.
	 */
    public static final CertificateFactory getInstance(String type)
	throws CertificateException
    {
		Provider[] prov = Security.getProviders();

		for (int i = 0; i != prov.length; i++)
		{
			try
			{
				return getInstance(type, prov[i].getName());
			}
			catch (Exception e)
			{
				// ignore
			}
		}
		throw new CertificateException("No provider for " + type);
    }

	/*
		Generates a certificate factory object for the specified
		certificate type from the specified provider.
	 */
    public static final CertificateFactory getInstance(
		String type,
	    String provider
		)
	throws CertificateException, NoSuchProviderException
    {
		CertificateFactory	fact;
		CertificateFactorySpi	spi;
		String		className;

		Provider	prov;

		prov = Security.getProvider(provider);
		if (prov == null)
		{
			throw new NoSuchProviderException(
					"Provider " + provider + " not found");
		}

		className = prov.getProperty("CertificateFactory." + type);
		if (className == null)
		{
			throw new CertificateException(type);
		}

		try
		{
			spi = (CertificateFactorySpi)Class.forName(
						className).newInstance();
		}
		catch (Exception e)
		{
			throw new CertificateException(
						type + ": " + e);
		}

		return new CertificateFactory(spi, prov, type);
    }

	// Returns the provider of this certificate factory.
	public final Provider getProvider() 
	{
		return provider;
	}

	/*
		Returns the name of the certificate type associated with
		this certificate factory.
	 */
	public final String getType() 
	{
		return this.type;
	}

}
