/*
 * $Id: RSAPrivateCrtKeySpec.java,v 1.2 1998/10/26 01:46:54 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.security/spec/RSAPrivateCrtKeySpec.java,v $
 * $Revision: 1.2 $
 * $Date: 1998/10/26 01:46:54 $
 * $State: Exp $
 */

package au.net.aba.security.spec;

import java.math.BigInteger;

/**
 * A class that provides a specification for a RSA Private Key with 
 * components for using Chinese remainder theorem.
 *
 * @see Key
 * @see KeyFactory
 * @see KeySpec
 * @see PKCS8EncodedKeySpec
 * @see RSAPrivateKeySpec
 * @see RSAPublicKeySpec
 */
public class RSAPrivateCrtKeySpec
	implements KeySpec
{
	public final static String ident = "$Id: RSAPrivateCrtKeySpec.java,v 1.2 1998/10/26 01:46:54 leachbj Exp $";

	private BigInteger	modulus;	 // the modulus.
	private BigInteger	publicExponent;	 // the public exponent.
	private BigInteger	privateExponent; // the private exponent.
	private BigInteger	primeP;		 // the prime P.
	private BigInteger	primeQ;		 // the prime Q.
	private BigInteger	primeExponentP;	 // the prime exponent for P.
	private BigInteger	primeExponentQ;	 // the prime exponent for Q.
	private BigInteger	crtCoefficient;	 // the CRT coefficient.

	/**
	 * basic constructor
	 *
	 * @param modulus the modulus.
	 * @param publicExponent the public exponent.
	 * @param privateExponent the private exponent.
	 * @param primeP the prime P.
	 * @param primeQ the prime Q.
	 * @param primeExponentP the prime exponent for P.
	 * @param primeExponentQ the prime exponent for Q.
	 * @param crtCoefficient the CRT coefficient.
	 */
	public RSAPrivateCrtKeySpec(
		BigInteger	modulus,
		BigInteger	publicExponent,
		BigInteger	privateExponent,
		BigInteger	primeP,
		BigInteger	primeQ,
		BigInteger	primeExponentP,
		BigInteger	primeExponentQ,
		BigInteger	crtCoefficient)
	{
		this.modulus = modulus;
		this.publicExponent = publicExponent;
		this.privateExponent = privateExponent;
		this.primeP = primeP;
		this.primeQ = primeQ;
		this.primeExponentP = primeExponentP;
		this.primeExponentQ = primeExponentQ;
		this.crtCoefficient = crtCoefficient;
	}

	/**
	 * Returns the modulus.
	 *
	 * @return the modulus.
	 */
	public BigInteger getModulus()
	{
		return modulus;
	}

	/**
	 * Returns the public exponent.
	 *
	 * @return the public exponent.
	 */
	public BigInteger getPublicExponent()
	{
		return publicExponent;
	}

	/**
	 * Returns the private exponent.
	 *
	 * @return the private exponent.
	 */
	public BigInteger getPrivateExponent()
	{
		return privateExponent;
	}

	/**
	 * Returns the primeP.
	 *
	 * @return the primeP.
	 */
	public BigInteger getPrimeP()
	{
		return primeP;
	}

	/**
	 * Returns the primeQ.
	 *
	 * @return the primeQ.
	 */
	public BigInteger getPrimeQ()
	{
		return primeQ;
	}

	/**
	 * Returns the primeExponentP.
	 *
	 * @return the primeExponentP.
	 */
	public BigInteger getPrimeExponentP()
	{
		return primeExponentP;
	}

	/**
	 * Returns the primeExponentQ.
	 *
	 * @return the primeExponentQ.
	 */
	public BigInteger getPrimeExponentQ()
	{
		return primeExponentQ;
	}

	/**
	 * Returns the crtCoefficient.
	 *
	 * @return the crtCoefficient.
	 */
	public BigInteger getCrtCoefficient()
	{
		return crtCoefficient;
	}
}
