/*
 * $Id: SimpleSet.java,v 1.1 1999/01/18 23:35:59 wslade Exp $
 * $Author: wslade $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba.util/SimpleSet.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/01/18 23:35:59 $
 * $State: Exp $
 */

/*
 * Note: This code was implemented to allow Java 1.2 functionality to be back
 *       ported to previous verions of Java. Hence only minimal effort has 
 *       been made to implement this Set and leverages of the existing Vector class.
 *       This class is not part of the 1.2 collection classes, however the
 *       interface Set is.
 */
package au.net.aba.util;

import java.util.Vector;

/**
 * A collection that contains no duplicate elements.  More formally, sets
 * contain no pair of elements <code>e1</code> and <code>e2</code> such that
 * <code>e1.equals(e2)</code>, and at most one null element.  As implied by
 * its name, this interface models the mathematical <i>set</i> abstraction.<p>
 *
 * The <tt>Set</tt> interface places additional stipulations, beyond those
 * inherited from the <tt>Collection</tt> interface, on the contracts of all
 * constructors and on the contracts of the <tt>add</tt>, <tt>equals</tt> and
 * <tt>hashCode</tt> methods.  Declarations for other inherited methods are
 * also included here for convenience.  (The specifications accompanying these
 * declarations have been tailored to the <tt>Set</tt> interface, but they do
 * not contain any additional stipulations.)<p>
 *
 * The additional stipulation on constructors is, not surprisingly,
 * that all constructors must create a set that contains no duplicate elements
 * (as defined above).<p>
 *
 * Note: Great care must be exercised if mutable objects are used as set
 * elements.  The behavior of a set is not specified if the value of an object
 * is changed in a manner that affects equals comparisons while the object is
 * an element in the set.  A special case of this prohibition is that it is
 * not permissible for a set to contain itself as an element.
 *
 */
public class SimpleSet implements Set
{
    // Query Operations
	private Vector mVector = new Vector();
	
    /**
     * Returns the number of elements in this set (its cardinality).  If this
     * set contains more than <tt>Integer.MAX_VALUE</tt> elements, returns
     * <tt>Integer.MAX_VALUE</tt>.
     *
     * @return the number of elements in this set (its cardinality).
     */
    public int size()
	{
		return mVector.size();
	}

    /**
     * Returns <tt>true</tt> if this set contains no elements.
     *
     * @return <tt>true</tt> if this set contains no elements.
     */
    public boolean isEmpty()
	{
		return mVector.isEmpty();
	}

    /**
     * Returns <tt>true</tt> if this set contains the specified element.  More
     * formally, returns <tt>true</tt> if and only if this set contains an
     * element <code>e</code> such that <code>(o==null ? e==null :
     * o.equals(e))</code>.
     *
     * @return <tt>true</tt> if this set contains the specified element.
     */
    public boolean contains(Object o)
	{
		return mVector.contains(o);
	}

    /**
     * Returns an iterator over the elements in this set.  The elements are
     * returned in no particular order (unless this set is an instance of some
     * class that provides a guarantee).
     *
     * @return an iterator over the elements in this set.
     */
    public Iterator iterator()
	{
		return new Iterator()
		{
			int index = 0;
 			public boolean hasNext()
			{
				return index != size();
			}

		    public Object next()
			{
				if (!hasNext())
					return null;
				return mVector.elementAt(index++);
			}

		    /**
		     * 
		     * Removes from the underlying collection the last element returned by the
		     * iterator (optional operation).  This method can be called only once per
		     * call to <tt>next</tt>.  The behavior of an iterator is unspecified if
		     * the underlying collection is modified while the iteration is in
		     * progress in any way other than by calling this method.
		     *
		     * @exception UnsupportedOperationException if the <tt>remove</tt>
		     *		  operation is not supported by this Iterator.
		     
		     * @exception IllegalStateException if the <tt>next</tt> method has not
		     *		  yet been called, or the <tt>remove</tt> method has already
		     *		  been called after the last call to the <tt>next</tt>
		     *		  method.
		     */
		    public void remove()
			{
				mVector.removeElementAt(index);
			}
		};
	}

    /**
     * Returns an array containing all of the elements in this set.
     * Obeys the general contract of the <tt>Collection.toArray</tt> method.
     *
     * @return an array containing all of the elements in this set.
     */
    public Object[] toArray()
	{
		synchronized(this)
		{
			Object[] objects = new Object[size()];
			for (int i = 0; i < size(); i++)
				objects[i] = mVector.elementAt(i);
			return objects;
		}
	}

    /**
     * Returns an array containing all of the elements in this set whose
     * runtime type is that of the specified array.  Obeys the general
     * contract of the <tt>Collection.toArray(Object[])</tt> method.
     *
     * @param a the array into which the elements of this set are to
     *		be stored, if it is big enough; otherwise, a new array of the
     * 		same runtime type is allocated for this purpose.
     * @return an array containing the elements of this set.
     * @throws    ArrayStoreException the runtime type of a is not a supertype
     * of the runtime type of every element in this set.
     */
    public Object[] toArray(Object a[])
	{
		synchronized(this)
		{
			if (size() < a.length)
			{
				for (int i = 0; i < size(); i++)
					a[i] = mVector.elementAt(i);
				return a;
			}
			else
				return toArray();
		}
	}


    // Modification Operations

    /**
     * Adds the specified element to this set if it is not already present
     * (optional operation).  More formally, adds the specified element,
     * <code>o</code>, to this set if this set contains no element
     * <code>e</code> such that <code>(o==null ? e==null :
     * o.equals(e))</code>.  If this set already contains the specified
     * element, the call leaves this set unchanged and returns <tt>false</tt>.
     * In combination with the restriction on constructors, this ensures that
     * sets never contain duplicate elements.<p>
     *
     * The stipulation above does not imply that sets must accept all
     * elements; sets may refuse to add any particular element, including
     * <tt>null</tt>, and throwing an exception, as described in the
     * specification for <tt>Collection.add</tt>.  Individual set
     * implementations should clearly document any restrictions on the the
     * elements that they may contain.
     *
     * @param o element to be added to this set.
     * @return <tt>true</tt> if this set did not already contain the specified
     *         element.
     * 
     * @throws UnsupportedOperationException if the <tt>add</tt> method is not
     * 	       supported by this set.
     * @throws ClassCastException if the class of the specified element
     * 	       prevents it from being added to this set.
     * @throws IllegalArgumentException if some aspect of this element
     *         prevents it from being added to this set.
     */
	public boolean add(Object o)
	{
		if (mVector.contains(o))
			return false;
		mVector.addElement(o);
		return true;
	}


    /**
     * Removes the specified element from this set if it is present (optional
     * operation).  More formally, removes an element <code>e</code> such that
     * <code>(o==null ?  e==null : o.equals(e))</code>, if the set contains
     * such an element.  Returns <tt>true</tt> if the set contained the
     * specified element (or equivalently, if the set changed as a result of
     * the call).  (The set will not contain the specified element once the
     * call returns.)
     *
     * @param o object to be removed from this set, if present.
     * @return true if the set contained the specified element.
     * @throws UnsupportedOperationException if the <tt>remove</tt> method is
     *         not supported by this set.
     */
    public boolean remove(Object o)
	{
		return mVector.removeElement(o);
	}


    // Bulk Operations

    /**
     * Returns <tt>true</tt> if this set contains all of the elements of the
     * specified collection.  If the specified collection is also a set, this
     * method returns <tt>true</tt> if it is a <i>subset</i> of this set.
     *
     * @param c collection to be checked for containment in this set.
     * @return <tt>true</tt> if this set contains all of the elements of the
     * 	       specified collection.
     */
    public boolean containsAll(Collection c)
	{
		for (Iterator itr = c.iterator(); itr.hasNext();)
		{
			if (!contains(itr.next()))
				return false;
		}
		return true;
	}

    /**
     * Adds all of the elements in the specified collection to this set if
     * they're not already present (optional operation).  If the specified
     * collection is also a set, the <tt>addAll</tt> operation effectively
     * modifies this set so that its value is the <i>union</i> of the two
     * sets.  The behavior of this operation is unspecified if the specified
     * collection is modified while the operation is in progress.
     *
     * @param c collection whose elements are to be added to this set.
     * @return <tt>true</tt> if this set changed as a result of the call.
     * 
     * @throws UnsupportedOperationException if the <tt>addAll</tt> method is
     * 		  not supported by this set.
     * @throws ClassCastException if the class of some element of the
     * 		  specified collection prevents it from being added to this
     * 		  set.
     * @throws IllegalArgumentException if some aspect of some element of the
     *		  specified collection prevents it from being added to this
     *		  set.
     * @see #add(Object)
     */
    public boolean addAll(Collection c)
	{
		boolean ret = false;
		for (Iterator itr = c.iterator(); itr.hasNext();)
		{
			if (add(itr.next()))
				ret = true;
		}
		return ret;
	}

    /**
     * Retains only the elements in this set that are contained in the
     * specified collection (optional operation).  In other words, removes
     * from this set all of its elements that are not contained in the
     * specified collection.  If the specified collection is also a set, this
     * operation effectively modifies this set so that its value is the
     * <i>intersection</i> of the two sets.
     *
     * @param c collection that defines which elements this set will retain.
     * @return <tt>true</tt> if this collection changed as a result of the
     *         call.
     * @throws UnsupportedOperationException if the <tt>retainAll</tt> method
     * 		  is not supported by this Collection.
     * 
     * @see #remove(Object)
     */
    public boolean retainAll(Collection c)
	{
		boolean ret = false;
		for (int i = size() - 1; i >= 0; i--)
		{
			Object element = mVector.elementAt(i);
			if (!c.contains(element))
			{
				remove(element);
				ret = true;
			}
		}
				
		return ret;
	}


    /**
     * Removes from this set all of its elements that are contained in the
     * specified collection (optional operation).  If the specified
     * collection is also a set, this operation effectively modifies this
     * set so that its value is the <i>asymmetric set difference</i> of
     * the two sets.
     *
     * @param c collection that defines which elements will be removed from
     *          this set.
     * @return <tt>true</tt> if this set changed as a result of the call.
     * 
     * @throws UnsupportedOperationException if the <tt>removeAll</tt>
     * 		  method is not supported by this Collection.
     * 
     * @see #remove(Object) */
    public boolean removeAll(Collection c)
	{
		boolean ret = false;
		for (int i = size() - 1; i >= 0; i--)
		{
			Object element = mVector.elementAt(i);
			if (c.contains(element))
			{
				remove(element);
				ret = true;
			}
		}
				
		return ret;
	}

    /**
     * Removes all of the elements from this set (optional operation).
     * This set will be empty after this call returns (unless it throws an
     * exception).
     *
     * @throws UnsupportedOperationException if the <tt>clear</tt> method
     * 		  is not supported by this set.
     */
    public void clear()
	{
		mVector.removeAllElements();
	}


    // Comparison and hashing

    /**
     * Compares the specified object with this set for equality.  Returns
     * <tt>true</tt> if the specified object is also a set, the two sets
     * have the same size, and every member of the specified set is
     * contained in this set (or equivalently, every member of this set is
     * contained in the specified set).  This definition ensures that the
     * equals method works properly across different implementations of the
     * set interface.
     *
     * @param o Object to be compared for equality with this set.
     * @return <tt>true</tt> if the specified Object is equal to this set.
     */
    public boolean equals(Object o)
	{
		if (!(o instanceof Set))
			return false;
		
		Set set = (Set)o;
		
		if (size() != set.size())
			return false;
		
		return containsAll(set);
	}

    /**
     * 
     * Returns the hash code value for this set.  The hash code of a set is
     * defined to be the sum of the hash codes of the elements in the set,
     * where the hashcode of a <tt>null</tt> element is defined to be zero.
     * This ensures that <code>s1.equals(s2)</code> implies that
     * <code>s1.hashCode()==s2.hashCode()</code> for any two sets
     * <code>s1</code> and <code>s2</code>, as required by the general
     * contract of the <tt>Object.hashCode</tt> method.
     *
     * @return the hash code value for this set.
     * @see Object#hashCode()
     * @see Object#equals(Object)
     * @see Set#equals(Object)
     */
    public int hashCode()
	{
		int hash = 0;
		for (int i = 0; i < size(); i++)
		{
			hash += mVector.elementAt(i).hashCode();
		}
		return hash;
	}
}
