/*
 * $Id: BlockCipher.java,v 1.25 1999/02/11 04:28:32 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/provider/BlockCipher.java,v $
 * $Revision: 1.25 $
 * $Date: 1999/02/11 04:28:32 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.IvParameterSpec;
import au.net.aba.crypto.spec.InlineIvParameterSpec;
import au.net.aba.security.*;
import au.net.aba.security.InvalidAlgorithmParameterException;
import au.net.aba.security.spec.AlgorithmParameterSpec;

/**
 * This abstract class is the base class for all Block Ciphers.
 * It will perform the necessary block buffering and then
 * pass the data on through the protected interface.
 * <p>
 * This class supports PKCS#5 and NoPadding, as well
 * as supporting ECB and CBC modes.  The standard block size
 * is 8.
 * <p>
 * To implement a new block cipher it is necessary to implement
 * the three abstract methods that re-key the cipher, do encryption
 * and decryption using the current.  It may also be necessary
 * to override the <code>engineSetMode</code> method if CBC
 * or ECB are not supported.  If different padding mechanisms are
 * used <code>engineSetPadding</code> may be overridden (in
 * which case it may also be necessary to override
 * <code>engineGetOutputSize</code>).  For block sizes other than 8
 * <code>engineGetBlockSize</code> should be overridden.  Finally
 * if the algorithm supports AlgorithmParameterSpec just override
 * the <code>engineInit</code> method that accepts those parameters.
 */
public abstract class BlockCipher extends CipherSpi
{
	public final static String ident = "$Id: BlockCipher.java,v 1.25 1999/02/11 04:28:32 leachbj Exp $";

	protected final static int BLOCK_SIZE = 8;

	protected static final int ECB = 0;
	protected static final int CBC = 1;

	protected byte[] buffer;
	protected int bufferPos;

	protected int mode;
	protected boolean paddedStream;
	protected int streamMode;

	protected byte[] ivec;
	protected byte[] cbcV;

	protected boolean ivInline;
	protected boolean ivEncrypted;
	protected boolean firstBlock;

	protected Key key;

	protected SecureRandom random;

	public BlockCipher()
	{
		paddedStream = false;
		streamMode = ECB;
	}

	/**
	 * Sets the mode of this cipher.  Valid modes for are;
	 * ECB or CBC.
	 *
	 * @param mode the cipher mode 
	 * @exception NoSuchAlgorithmException	if the requested cipher mode
	 * 	does not exist 
	 */
	protected void engineSetMode(String mode)
	throws NoSuchAlgorithmException
	{
/*
		System.out.println("engineSetMode("+mode+")");
*/
		if (mode.equals("ECB"))
		{
			streamMode = ECB;
		}
		else if (mode.equals("CBC"))
		{
			streamMode = CBC;
		}
		else
		{
			throw new NoSuchAlgorithmException(
				"Mode " + mode + " not supported.");
		}
	}

	/**
	 * Sets the padding mechanism of this cipher.  Valid padding mechanisms
	 * are; "PKCS5Padding" and "NoPadding".
	 *
	 * @param padding the padding mechanism
	 * @exception NoSuchPaddingException if the requested padding 
	 * 	mechanism does not exist.
	 */
	protected void engineSetPadding(String padding)
	throws NoSuchPaddingException
	{
/*
		System.out.println("engineSetPadding("+padding+")");
*/
		if (padding.equals("PKCS5Padding"))
		{
			paddedStream = true;
		}
		else if (padding.equals("NoPadding"))
		{
			paddedStream = false;
		}
		else
		{
			throw new NoSuchPaddingException(
				"Unsupported padding " + padding);
		}
	}

	/**
	 * Returns the block size (in bytes).  In this case 8.  This
	 * method should be overridden if the given algorithm has
	 * a different block size.
	 *
	 * @return the block size (in bytes), or 0 if the underlying
	 * 	algorithm is not a block cipher 
	 */
	protected int engineGetBlockSize()
	{
		return BLOCK_SIZE;
	}

	/**
	 * Returns the length in bytes that an output buffer would need to be
	 * in order to hold the result of the next update or doFinal operation,
	 * given the input length inputLen (in bytes). 
	 * <p>
	 * This call takes into account any unprocessed (buffered) data from a
	 * previous update call, and padding. 
	 * <p>
	 * The actual output length of the next update or doFinal call may be
	 * smaller than the length returned by this method. 
	 *
	 * @param inputLen the input length (in bytes) 
	 * @return the required output buffer size (in bytes)
	 */
	protected int engineGetOutputSize(int inputLen)
	{
		int blockSize = engineGetBlockSize();

		if (firstBlock && ivInline)
		{
			if (mode == Cipher.ENCRYPT_MODE)
			{
				inputLen += blockSize;
			}
			else if (mode == Cipher.DECRYPT_MODE)
			{
				inputLen -= blockSize;
			}
		}

		inputLen += bufferPos;

		if (paddedStream)
		{
			if ((inputLen % blockSize) == 0)
			{
				inputLen += blockSize;
			}

			return ((inputLen + blockSize - 1) / blockSize)
				* blockSize;
		}

		return (inputLen / blockSize) * blockSize;
	}

	/**
	 * Returns the initialisation vector (IV) in a new buffer. 
	 * <p>
	 * This is useful in the context of password-based encryption or
	 * decryption, where the IV is derived from a user-provided passphrase. 
	 *
	 * @return the initialisation vector in a new buffer, or null if the
	 * 	underlying algorithm does not use an IV, or if the IV has
	 * 	not yet been set. 
	 */
	protected byte[] engineGetIV()
	{
		if (ivec != null)
		{
			int blockSize = engineGetBlockSize();
			byte[] result = new byte[blockSize];
			System.arraycopy(ivec, 0, result, 0, blockSize);

			return result;
		}

		return null;
	}

	/**
	 * Returns the parameters used with this cipher. 
	 * <p>
	 * The returned parameters may be the same that were used to initialise
	 * this cipher, or may contain the default set of parameters or a set of
	 * randomly generated parameters used by the underlying cipher
	 * implementation (provided that the underlying cipher implementation
	 * uses a default set of parameters or creates new parameters if it
	 * needs parameters but was not initialised with any).
	 *
	 * @return the parameters used with this cipher, or null if this cipher
	 *     does not use any parameters.
	 */
	protected AlgorithmParameters engineGetParameters()
	{
		return null;
	}

	/**
	 * Initialises this cipher with a key and a source of randomness.  
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending
	 * on the value of opmode.
	 * <p>
	 * If this cipher requires any algorithm parameters that cannot be
	 * derived from the given key, the underlying cipher implementation
	 * is supposed to generate the required parameters itself (using
	 * provider-specific default or random values) if it is being
	 * initialised for encryption, and raise an InvalidKeyException if it
	 * is being initialised for decryption. The generated parameters can
	 * be retrieved using engineGetParameters or engineGetIV (if the
	 * parameter is an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them from random.
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher and
	 * initialising it

	 * @param opmode the operation mode of this cipher (this is either
	 *			ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the encryption key 
	 * @param random the source of randomness 
	 * @exception InvalidKeyException if the given key is inappropriate
	 *			for initialising this cipher 
	 */
	protected void engineInit(
		int opmode,
		Key key,
		SecureRandom random)
	throws InvalidKeyException
	{
		try
		{
			engineInit(opmode, key, (AlgorithmParameterSpec)null,
				random);
		}
		catch (InvalidAlgorithmParameterException e)
		{
			// ouch this should never happen!
			throw new InvalidKeyException(
				"InvalidAlgorithmParameterException: "
				+ e.getMessage());
		}
	}

	/**
	 * Initialises this cipher with a key, a set of algorithm parameters,
	 * and a source of randomness.
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending
	 * on the value of opmode.
	 * <p>
	 * If this cipher requires any algorithm parameters and params is
	 * null, the underlying cipher implementation is supposed to generate
	 * the required parameters itself (using provider-specific default or
	 * random values) if it is being initialised for encryption, and
	 * raise an InvalidAlgorithmParameterException if it is being
	 * initialised for decryption. The generated parameters can be
	 * retrieved using engineGetParameters or engineGetIV (if the
	 * parameter is an IV). 
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them from random. 
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher and
	 * initialising it.
	 * <p>
	 * @param opmode the operation mode of this cipher (this is either
	 *    ENCRYPT_MODE or DECRYPT_MODE)
	 * @param key the encryption key
	 * @param params the algorithm parameters
	 * @param random the source of randomness
	 * @exception InvalidKeyException if the given key is
	 *    inappropriate for initialising this cipher
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 * 	parameters are inappropriate for this cipher, or if this
	 * 	cipher is being initialised fro decryption and requires
	 * 	algorithm parameters and params is null
	 */
	protected void engineInit(
		int opmode,
		Key key,
		AlgorithmParameters params,
		SecureRandom random)
	throws InvalidKeyException, InvalidAlgorithmParameterException
	{
		AlgorithmParameterSpec spec = null;
		if (params != null)
		{
			try
			{
				spec = params.getParameterSpec(IvParameterSpec.class);
			}
			catch (Exception e)
			{
				try
				{
					spec = params.getParameterSpec(InlineIvParameterSpec.class);
				}
				catch (Exception e2)
				{
					throw new InvalidAlgorithmParameterException(
						"Processing error: " + e2);
				
				}
			}
		}

		engineInit(opmode, key, spec, random);
	}

	/**
	 * Initialises this cipher with a key, a set of algorithm parameters,
	 * and a source of randomness.
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending
	 * on the value of opmode.
	 * <p>
	 * If this cipher requires any algorithm parameters and params is
	 * null, the underlying cipher implementation is supposed to generate
	 * the required parameters itself (using provider-specific default or
	 * random values) if it is being initialised for encryption, and
	 * raise an InvalidAlgorithmParameterException if it is being
	 * initialised for decryption. The generated parameters can be
	 * retrieved using engineGetParameters or engineGetIV (if the
	 * parameter is an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them from random.
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher and
	 * initialising it.
	 * <p>
	 * @param opmode the operation mode of this cipher (this is either
	 *	ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the encryption key 
	 * @param params the algorithm parameters
	 * @param random the source of randomness 
	 *
	 * @exception InvalidKeyException if the given key is inappropriate
	 * 	for initialising this cipher 
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 * 	parameters are inappropriate for this cipher, or if this
	 * 	cipher is being initialised fro decryption and requires
	 * 	algorithm parameters and params is null
	 */
	protected void engineInit(
		int opmode,
		Key key,
		AlgorithmParameterSpec params,
		SecureRandom random)
	throws InvalidKeyException, InvalidAlgorithmParameterException
	{
		reset();

		this.mode = opmode;
		this.random = random;
		this.key = key;

		if (streamMode == CBC)
		{
			int blockSize = engineGetBlockSize();

			/*
			 * create two buffers, one for the original
			 * iv, the other for the "current" iv
			 */
			ivec = new byte[blockSize];
			cbcV = new byte[blockSize];

			if (params instanceof IvParameterSpec)
			{
				/*
				 * copy the supplied iv
				 */
				System.arraycopy(
					((IvParameterSpec)params).getIV(),
					0, ivec, 0, blockSize);
			}
			else
			{
				/*
				 * create a random one, it will be
				 * added to the output stream
				 */
				if (params instanceof InlineIvParameterSpec)
				{
					ivInline = true;
					firstBlock = true;
					ivEncrypted = ((InlineIvParameterSpec)params).isEncryptedIv();
				}

				random.nextBytes(ivec);
			}

			/*
			 * copy the iv for use
			 */
			System.arraycopy(ivec, 0, cbcV, 0, blockSize);
		}

		setKey(key);
	}

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, are processed, and the result is stored in a new buffer.
	 *
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @return the new buffer with the result, or null if the underlying
	 *    cipher is a block cipher and the input data is too short to
	 *    result in a new block. 
	 */
	protected byte[] engineUpdate(
		byte[] input,
		int inputOff,
		int inputLen)
	{
		byte[] output = null;
		int len = engineGetOutputSize(inputLen);
		if (len != 0)
		{
			output = new byte[len];
		}

/*
		System.out.println("len = " + len);
*/

		try
		{
			len = engineUpdate(input, inputOff, inputLen,
				output, 0);
		}
		catch (ShortBufferException e)
		{
			// this shouldnt happen
			e.printStackTrace();
			throw new RuntimeException("ShortBufferException: "
				+ e.getMessage());
		}

/*
		System.out.println("len = " + len);
*/

		/*
		 * our initial buffer was too big!
		 */
		if (output != null && len != output.length)
		{
/*
			System.out.println("len = " + len + " output.length = " + output.length);
*/
			byte[] buf = new byte[len];
			System.arraycopy(output, 0, buf, 0, len);

			output = buf;
		}


		return output;
	}

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, are processed, and the result is stored in the output
	 * buffer, starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown. In this case, repeat this call with
	 * a larger output buffer. Use getOutputSize to determine how big the
	 * output buffer should be. 
	 * 
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * 
	 * @exception ShortBufferException if the given output buffer is too
	 *     small to hold the result 
	 *
	 * @return the number of bytes stored in output 
	 */
	protected int engineUpdate(
		byte[] input,
		int inputOff,
		int inputLen,
		byte[] output,
		int outputOff)
	throws ShortBufferException
	{
		try
		{
			return processAllBlocks(input, inputOff, inputLen,
				output, outputOff);
		}
		catch (BadPaddingException bpe)
		{
			// this shouldnt happen
			bpe.printStackTrace();
		}
		catch (IllegalBlockSizeException ibse)
		{
			// this shouldnt happen
			ibse.printStackTrace();
		}

		throw new ShortBufferException(
			"Internal error, see stacktrace on console.");
	}


	/**
	 * Encrypts or decrypts data in a single-part operation, or
	 * finishes a multiple-part operation. The data is encrypted or
	 * decrypted, depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in a new buffer. 
	 * <p>
	 * The cipher is reset to its initial state (uninitialised) after
	 * this call. 
	 *
	 * @param input the input buffer 
	 * @param inputOff the offset in input where the input starts 
	 * @param inputLen the input length 
	 *
	 * @exception IllegalBlockSizeException if this cipher is a block
	 * 	cipher, no padding has been requested (only in encryption
	 * 	mode), and the total input length of the data processed by
	 * 	this cipher is not a multiple of block size 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 * 	and (un)padding has been requested, but the decrypted data is
	 * 	not bounded by the appropriate padding bytes.
	 *
	 * @return the new buffer with the result 
	 */
	protected byte[] engineDoFinal(
		byte input[],
		int inputOff,
		int inputLen)
	throws IllegalBlockSizeException, BadPaddingException
	{
		/*
		 * create result array for the entire result
		 */
		byte[] output = new byte[engineGetOutputSize(inputLen)];

		try
		{
			int len = engineDoFinal(input, inputOff,
				inputLen, output, 0);
			if (len != output.length)
			{
				byte[] tmp = new byte[len];

/*
				System.out.println(
					"output " + output
					+ " output.length "
					+ (output != null ? output.length : -1)
					+ " tmp " + tmp
					+ " tmp.length "
					+ (tmp != null ? tmp.length : -1)
					+ " len " + len);
*/

				System.arraycopy(output, 0, tmp, 0, len);
				output = tmp;
			}
		}
		catch (ShortBufferException e)
		{
			// ouch this should never happen!
			throw new BadPaddingException("ShortBufferException: "
				+ e.getMessage());
		}

		return output;
	}

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes
	 * a multiple-part operation. The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in the output buffer,
	 * starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown.  In this case, repeat this call
	 * with a larger output buffer. Use getOutputSize to determine how
	 * big the output buffer should be. 
	 * 
	 * @param input the input buffer 
	 * @param inputOffset - the offset in input where the input starts 
	 * @param inputLen - the input length 
	 * @param output - the buffer for the result 
	 * @param outputOffset - the offset in output where the result is stored
	 * 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 * 	cipher, no padding has been requested (only in encryption mode),
	 * 	and the total input length of the data processed by this cipher
	 * 	is not a multiple of block size 
	 * @exception ShortBufferException if the given output buffer is too
	 * 	small to hold the result 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 * 	and (un)padding has been requested, but the decrypted data is
	 * 	not bounded by the appropriate padding bytes 
	 * @returns the number of bytes stored in output 
	 */
	protected int engineDoFinal(
		byte[] input,
		int inputOff,
		int inputLen,
		byte[] output,
		int outputOff)
	throws ShortBufferException, IllegalBlockSizeException,
		BadPaddingException
	{
		int blockSize = engineGetBlockSize();
		int resultLen;

/*
		System.out.println("engineDoFinal: "
			+ " input " + input
			+ " inputOff " + inputOff
			+ " inputLen " + inputLen
			+ " output " + output
			+ " outputOff " + outputOff);
*/

		/*
		 * process any full blocks
		 */
		resultLen = processAllBlocks(input, inputOff, inputLen,
			output, outputOff);
		outputOff += resultLen;

/*
		System.out.println("engineDoFinal: bufferPos " + bufferPos);
*/

		/*
		 * process any final partial block
		 */
		if (mode == Cipher.ENCRYPT_MODE)
		{
			if (bufferPos == blockSize)
			{
				int n = processBlock(buffer, 0,
					blockSize, output, outputOff);
				outputOff += n;
				resultLen += n;
				bufferPos = 0;
			}

			if (paddedStream)
			{
				/*
				 * add PKCS7 padding
				 */
				int code = blockSize - bufferPos;
				for (; bufferPos < blockSize;
					bufferPos++)
				{
					buffer[bufferPos] = (byte)code;
				}

/*
				System.out.println("padding with code: " + code);
*/
			}

			if (bufferPos > 0)
			{
				resultLen += processBlock(buffer, 0, bufferPos,
					output, outputOff);
				bufferPos = 0;
			}
		}
		else if (mode == Cipher.DECRYPT_MODE)
		{
			if (bufferPos > 0)
			{
				if (bufferPos == blockSize)
				{
					resultLen += processBlock(buffer, 0,
						blockSize, output, outputOff);
					bufferPos = 0;
				}
				else
				{
					throw new IllegalBlockSizeException(
						"input truncated.");
				}
			}

			if (paddedStream)
			{
/*
System.out.println(outputOff + " " + test.ByteUtil.bytesToHexStr(output));
*/

				/*
				 * remove PKCS7 padding
				 */
				int count = output[outputOff
					+ blockSize - 1] & 0xff;

/*
				System.out.println("pad count = " + count);
*/
				resultLen -= count;
			}
		}

		reset();
		return resultLen;
	}

	/**
	 * Reset the Cipher to uninitialised.
	 */
	protected void reset()
	{
		/*
		 * zero out our internal buffer
		 */
		if (buffer != null)
		{
			for (int i = 0; i < buffer.length; i++)
			{
				buffer[0] = 0;
			}
			buffer = null;
		}

		/*
		 * reset the cbcV
		 */
		if (ivec != null)
		{
			System.arraycopy(ivec, 0, cbcV, 0, engineGetBlockSize());
		}

		/*
		 * rekey the cipher
		 */
		if (key != null)
		{
			try
			{
				setKey(key);
			}
			catch (InvalidKeyException e)
			{
				// ignore it
			}
		}

		buffer = new byte[engineGetBlockSize()];
		bufferPos = 0;
		firstBlock = ivInline;
	}

	/**
	 * Re-key the cipher.  If the provided Key is not compatible
	 * with this cipher the exception should throw an InvalidKeyException.
	 */
	protected abstract void setKey(Key key) throws InvalidKeyException;

	/**
	 * Encrypt the given block starting at the given offset and place
	 * the result in the provided buffer starting at the given offset.
	 * The input will be an exact multiple of our blocksize.
	 */
	protected abstract int encryptBlock(byte[] in, int inoff, int len,
		byte[] out, int outOff) throws IllegalBlockSizeException;

	/**
	 * Decrypt the given block starting at the given offset and place
	 * the result in the provided buffer starting at the given offset.
	 * The input will be an exact multiple of our blocksize.
	 */
	protected abstract int decryptBlock(byte[] in, int inoff, int len,
		byte[] out, int outOff) throws BadPaddingException;

	/**
	 * This method is used to process in block sized chunks all the
	 * input data.
	 */
	private int processAllBlocks(
		byte[] input,
		int inputOff,
		int inputLen,
		byte[] output,
		int outputOff)
	throws BadPaddingException, IllegalBlockSizeException
	{
		int blockSize = engineGetBlockSize();
		int resultBytes = 0;

/*
		System.out.println (
			"processAllBlocks: input " + input
			+ " inputOff " + inputOff
			+ " inputLen " + inputLen
			+ " output " + output
			+ " outputOff " + outputOff
			+ " bufferPos " + bufferPos);
*/

		if (inputLen < (blockSize - bufferPos))
		{
			if (inputLen > 0)
			{
				System.arraycopy(input, inputOff, buffer,
					bufferPos, inputLen);
				bufferPos += inputLen;
			}
			return 0;
		}
		else if (inputLen == (blockSize - bufferPos))
		{
			if (inputLen > 0)
			{
				System.arraycopy(input, inputOff, buffer,
					bufferPos, inputLen);
			}

			if (paddedStream)
			{
				bufferPos += inputLen;
				return 0;
			}
			else
			{
				resultBytes = processBlock(buffer, 0,
					blockSize, output, outputOff);
				bufferPos = 0;
				return resultBytes;
			}
		}
		else
		{
			int n = blockSize - bufferPos;
			System.arraycopy(input, inputOff, buffer, bufferPos, n);
			resultBytes = processBlock(buffer, 0, blockSize,
				output, outputOff);

			inputLen -= n;
			inputOff += n;
			outputOff += resultBytes;
			bufferPos = 0;
		}

		int blockCount = (inputLen + blockSize - 1) / blockSize;
		int lastBlockSize = inputLen % blockSize;

		/*
		 * decode each element of the array, chopping the array
		 * into blocks to match the current key.
		 */
		for (int i = 0; i < blockCount - 1; i++)
		{
			int n = processBlock(input, inputOff, blockSize,
				output, outputOff);

			inputOff += blockSize;
			outputOff += n;
			resultBytes += n;
		}

		if (lastBlockSize == 0)    /* exact multiple */
		{
			/*
			 * if we are padded we need to hold the last
			 * block unless we are an RSA cipher.
			 */
			if (paddedStream)
			{
				System.arraycopy(input, inputOff,
					buffer, 0, blockSize);
				bufferPos = blockSize;
			}
			else
			{
				int n = processBlock(input, inputOff,
					blockSize, output, outputOff);
				outputOff += n;
				resultBytes += n;
				bufferPos = 0;
			}
		}
		else
		{
			System.arraycopy(input, inputOff,
				buffer, 0, lastBlockSize);
			bufferPos = lastBlockSize;
		}

		return resultBytes;
	}

	/**
	 * This method is used to process complete blocks.  The cipher
	 * dependent encrypt or decrypt function will be called depending
	 * on how this cipher is currently initialised.
	 */
	private int processBlock(
		byte[] input,
		int inputOff,
		int len,
		byte[] output,
		int outputOff)
	throws BadPaddingException, IllegalBlockSizeException
	{
		int blockSize = engineGetBlockSize();
		int resultLen = 0;

/*
		System.out.println (
			"processBlock: input " + input
			+ " input.length " + input.length
			+ " inputOff " + inputOff
			+ " len " + len
			+ " output " + output
			+ " output.length " + output.length
			+ " outputOff " + outputOff);
*/

		/*
		 * if we are doing the first block and have an inline IV
		 * copy/encrypt the IV to the output or copy/decrypt the
		 * IV from the input
		 */
		if (firstBlock && ivInline)
		{
			firstBlock = false;
			if (mode == Cipher.ENCRYPT_MODE)
			{
				int n;
				if (ivEncrypted)
				{
					/*
					 * encrypt the ivec to the output
					 */
					n = encryptBlock(ivec, 0, blockSize,
						output, outputOff);
				}
				else
				{
					/*
					 * copy the ivec to the output
					 */
					System.arraycopy(ivec, 0, output,
						outputOff, blockSize);
					n = blockSize;
				}

				/*
				 * just produced a block of output
				 */
				outputOff += n;
				resultLen += n;
			}
			else // Cipher.DECRYPT_MODE
			{
				if (ivEncrypted)
				{
					/*
					 * decrypt input to the ivec
					 */
					decryptBlock(input, inputOff, len,
						ivec, 0);
				}
				else
				{
					/*
					 * copy input to ivec
					 */
					System.arraycopy(input, inputOff,
						ivec, 0, blockSize);
				}


				/*
				 * copy recovered ivec to cbcV
				 */
				System.arraycopy(ivec, 0, cbcV, 0, blockSize);

				/*
				 * we have consumed a block of input so we return,
				 * however there is no output
				 */
				return 0;
			}
		}

		/*
		 * process the next block
		 */
		if (mode == Cipher.ENCRYPT_MODE)
		{
			if (streamMode == CBC)
			{
				/*
				 * XOR the cbcV and the input,
				 * then encrypt the cbcV
				 */
				for (int i = 0; i < blockSize; i++)
				{
					cbcV[i] ^= input[inputOff + i];
				}

				input = cbcV;
				inputOff = 0;
			}

			resultLen += encryptBlock(input, inputOff, len,
				output, outputOff);

			if (streamMode == CBC)
			{
				/*
				 * copy ciphertext to cbcV
				 */
				System.arraycopy(output, outputOff, cbcV, 0,
					blockSize);
			}
		}
		else
		{
			resultLen += decryptBlock(input, inputOff, len,
				output, outputOff);

			if (streamMode == CBC)
			{
				/*
				 * XOR the cbcV and the output
				 */
				for (int i = 0; i < blockSize; i++)
				{
					output[outputOff + i] ^= cbcV[i];
				}

				/*
				 * copy ciphertext to cbcV
				 */
				System.arraycopy(input, inputOff, cbcV, 0,
					blockSize);
			}
		}

		return resultLen;
	}
}
