/*
 * $Id: DESKeyFactory.java,v 1.12 1999/02/18 06:28:48 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/provider/DESKeyFactory.java,v $
 * $Revision: 1.12 $
 * $Date: 1999/02/18 06:28:48 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import au.net.aba.security.spec.InvalidKeySpecException;
import au.net.aba.security.spec.KeySpec;
import java.security.InvalidKeyException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactorySpi;
import javax.crypto.spec.DESKeySpec;
import javax.crypto.spec.SecretKeySpec;

/**
 * This class is used to convert DES keys into a format usable by the
 * ABA provider.  The supported KeySpec classes are DESKeySpec and
 * SecretKeySpec.  In the case of the SecretKeySpec the key data must
 * be at least 8 bytes long and there must be no encoding on the key
 * data.  The supported Key class is DESKey.
 * <p>
 * This class should not be instantiated directly, instead use the
 * java.security.KeyFactory interface.
 *
 * @see java.security.KeyFactory
 */
public class DESKeyFactory extends SecretKeyFactorySpi
{
	public final static String ident = "$Id: DESKeyFactory.java,v 1.12 1999/02/18 06:28:48 leachbj Exp $";

	/**
	 * Generates a DES SecretKey object from the provided key specification
	 * (key material).  This class supports the DESKeySpec KeySpec and
	 * SecretKeySpec classes.
	 *
	 * @returns The secret key.
	 * @exception InvalidKeySpecException  The provided KeySpec was not
	 * 	a DESKeySpec or SecretKeySpec instance.
	 */
	protected SecretKey engineGenerateSecret(
		KeySpec keySpec)
		throws InvalidKeySpecException
	{
		if (keySpec instanceof DESKeySpec)
		{
			return new DESKey(((DESKeySpec)keySpec).getKey());
		}
		else if (keySpec instanceof SecretKeySpec)
		{
			SecretKeySpec skspec = (SecretKeySpec)keySpec;

			if (!"DES".equals(skspec.getAlgorithm()))
			{
				throw new InvalidKeySpecException(
					"SecretKeySpec not for DES");
			}
			if (!"RAW".equals(skspec.getFormat()))
			{
				throw new InvalidKeySpecException(
					"Unknown encoding format in KeySpec");
			}

			return new DESKey(skspec.getEncoded());
		}

		throw new InvalidKeySpecException(
			keySpec.getClass().getName()
			+ " not valid for DES.");
	}

	/**
	 * Returns a specification (key material) of the given key object in
	 * the requested format. 
	 * <p>
	 * Currently supports conversion from ABA's DES keys to SecretKeySpec
	 * or DESKeySpec instances.
	 *
	 * @param key the key 
	 * @param keySpec the requested format in which the key material shall
	 *		be returned 
	 * @returns the underlying key specification (key material) in the
	 *		requested format
	 * @exception InvalidKeySpecException if the requested key
	 *	specification is inappropriate for the given key, or the given
	 *	key cannot be dealt with (e.g., the given key has an
	 *	unrecognised format). 
	 */
	protected KeySpec engineGetKeySpec(
		SecretKey key,
		Class keySpec)
		throws InvalidKeySpecException
	{
		if (key instanceof DESKey)
		{
			if (SecretKeySpec.class.isAssignableFrom(keySpec))
			{
				return new SecretKeySpec(key.getEncoded(),
					"DES");
			}
			else if (DESKeySpec.class.isAssignableFrom(keySpec))
			{
				try
				{
					return new DESKeySpec(key.getEncoded());
				}
				catch (InvalidKeyException e)
				{
					throw new InvalidKeySpecException(
						e.getMessage());
				}
			}
		}

		throw new InvalidKeySpecException("not implemented");
	}

	/**
	 * Translates a DES key object, whose provider may be unknown or
	 * potentially untrusted, into a corresponding key object of this key
	 * factory. 
	 *
	 * @param key - the key whose provider is unknown or untrusted 
	 * @returns the translated key 
	 * @exception InvalidKeyException if the given key cannot be processed
	 * 	by this key factory. 
	 */
	protected SecretKey engineTranslateKey(
		SecretKey key)
		throws InvalidKeyException
	{
		if (!key.getAlgorithm().equals("DES"))
		{
			throw new InvalidKeyException("not a DES key");
		}

		return new DESKey(key.getEncoded());
	}
}
