/*
 * $Id: DESKeyGenerator.java,v 1.7 1999/01/13 07:32:22 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/provider/DESKeyGenerator.java,v $
 * $Revision: 1.7 $
 * $Date: 1999/01/13 07:32:22 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import javax.crypto.*;
import javax.crypto.spec.*;

import au.net.aba.security.*;
import au.net.aba.security.spec.*;

/**
 * This class is used for generating random DES keys.  This class
 * should not be instantiated directly, instead use the
 * javax.crypto.KeyGenerator interface.
 * <p>
 * There is no AlgorithmParameterSpec class defined for DES so this
 * generator can only be initialised using the keysize,random
 * initialisation.
 * <p>
 * The returned key will be a non-weak key with odd parity.
 */
public class DESKeyGenerator extends KeyGeneratorSpi
{
	public final static String ident = "$Id: DESKeyGenerator.java,v 1.7 1999/01/13 07:32:22 leachbj Exp $";

	protected SecureRandom rand;

	/**
	 * Initialises the key generator with the specified parameter set and
	 * a user-provided source of randomness.
	 */
	protected void engineInit(
		AlgorithmParameterSpec	params,
		SecureRandom		random)
	{
		rand = random;
	}

	/**
	 * Initialises the key generator.
	 */
	protected void engineInit(
		SecureRandom	random)
	{
		engineInit(null, random);
	}

	/**
	 * Generates a secret key, setting odd parity and confirming that the
	 * key is not a weak key.
	 *
	 * @return a secret key representing a DES key.
	 */
	protected SecretKey engineGenerateKey()
	{
		// generate random non-weak key
		byte[] bytes = new byte[DESKeySpec.DES_KEY_LEN];

		DESKey key = null;
		do {
			// get 64 bits of random data, 8 bits are wasted :(
			rand.nextBytes(bytes);
			// set the parity to odd
			setOddParity(bytes);

			// check for weak keys
			if (!isWeakKey(bytes))
			{
				key = new DESKey(bytes);

				// clear the key from the byte buffer
				for (int i = 0; i < DESKeySpec.DES_KEY_LEN; i++)
				{
					bytes[i] = 0;
				}
			}
		} while (key == null);

		return key;
	}

	/**
	 * DES Keys use the LSB as the odd parity bit.  This can
	 * be used to check for corrupt keys.
	 *
	 * @param bytes the byte array to set the parity on.
	 */
	protected void setOddParity(byte[] bytes)
	{
		for (int i = 0; i < bytes.length; i++)
		{
			int b = bytes[i];
			bytes[i] = (byte)((b & 0xfe) |
				(((b >> 1) ^
				(b >> 2) ^
				(b >> 3) ^
				(b >> 4) ^
				(b >> 5) ^
				(b >> 6) ^
				(b >> 7)) ^ 0x01));
		}
	}

	/**
 	 * DES has a number of weak keys.  This method will check
	 * the given key against the table of weak keys.
	 *
	 * @param bytes the key to check.
	 * @return true if the key is a known weak key false otherwise.
	 * @see "Applied Cryptography" by Bruce Schneier.
	 */
	protected boolean isWeakKey(byte[] bytes)
	{
		nextkey: for (int i = 0; i < N_WEAK_KEYS; i++)
		{
			boolean weak = false;
			for (int j = 0; j < DESKeySpec.DES_KEY_LEN; j++)
			{
				if (bytes[j] != (byte)weak_keys[i*8 + j])
				{
					continue nextkey;
				}
			}
			return true;
		}
		return false;
	}

	/* Table of weak and semi-weak keys taken from Schneier pp281 */
	static int N_WEAK_KEYS = 16;
	static short[] weak_keys =
	{
		/* weak keys */
		0x01,0x01,0x01,0x01, 0x01,0x01,0x01,0x01,
		0x1f,0x1f,0x1f,0x1f, 0x0e,0x0e,0x0e,0x0e,
		0x0e,0xe0,0xe0,0xe0, 0xf1,0xf1,0xf1,0xf1,
		0xfe,0xfe,0xfe,0xfe, 0xfe,0xfe,0xfe,0xfe,
		/* semi-weak keys */
		0x01,0xfe,0x01,0xfe, 0x01,0xfe,0x01,0xfe,
		0x1f,0xe0,0x1f,0xe0, 0x0e,0xf1,0x0e,0xf1,
		0x01,0xe0,0x01,0xf1, 0x01,0xf1,0x01,0xf1,
		0x1f,0xfe,0x1f,0xfe, 0x0e,0xfe,0xef,0xfe,
		0x01,0x1f,0x01,0x1f, 0x01,0x0e,0x10,0x0e,
		0xe0,0xfe,0xe0,0xfe, 0xf1,0xfe,0xf1,0xfe,
		0xfe,0x01,0xfe,0x01, 0xfe,0x01,0xfe,0x01,
		0xe0,0x1f,0xe0,0x1f, 0xf1,0x0e,0xf1,0x0e,
		0xe0,0x01,0xe0,0x01, 0xf1,0x01,0xf1,0x01,
		0xfe,0x1f,0xfe,0x1f, 0xfe,0x0e,0xfe,0x0e,
		0x1f,0x01,0x1f,0x01, 0x0e,0x01,0x0e,0x01,
		0xfe,0xe0,0xfe,0xe0, 0xfe,0xf1,0xfe,0xf1
	};
}
