/*
 * $Id: KeyStore.java,v 1.1 1999/01/25 00:46:16 wslade Exp $
 * $Author: wslade $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/provider/KeyStore.java,v $
 * $Revision: 1.1 $
 * $Date: 1999/01/25 00:46:16 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import java.io.*;
import java.util.*;
import java.security.*;

import java.security.cert.Certificate;
import java.security.*;
import java.security.cert.*;
import java.security.spec.*;

import au.net.aba.crypto.spec.AsciiEncodedKeySpec;

import javax.crypto.*;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;
import javax.crypto.spec.SecretKeySpec;

/**
 * This class defines the <i>Service Provider Interface</i> (<b>SPI</b>)
 * for the <code>KeyStore</code> class.
 * All the methods in this class must be implemented by each
 * cryptographic service provider who wishes to supply the implementation
 * of a keystore for a particular keystore type.
 *
 *
 * @since JDK1.2
 */
public class KeyStore extends KeyStoreSpi 
{
	// Defines the cipher to use to encrypt the store.
	// private static final String CIPHER = "PBEWithMD5AndDES";
	private static final String CIPHER = "PBEWithSHA1And128BitRC4";
	
	private class PrivateKeyEntry implements Serializable
	{
		Key mKey;
		char[] mPassword;
		Certificate mCertificates[];
		
		public boolean validate(char[] password)
		{
			// Validate the password
			if (mPassword.length != password.length)
				return false;
				
			for (int i = 0; i < mPassword.length; i++)
				if (mPassword[i] != password[i])
					return false;
			return true;
		}
	}


	// Storage for all Trusted Certificates;
	private Hashtable mTrustedCerts = new Hashtable();

	// Storage for all Private Key Entries
	private Hashtable mPrivateKeys = new Hashtable();
	
    /**
     * Returns the key associated with the given alias, using the given
     * password to recover it.
     *
     * @param alias the alias name
     * @param password the password for recovering the key
     *
     * @return the requested key, or null if the given alias does not exist
     * or does not identify a <i>key entry</i>.
     *
     * @exception NoSuchAlgorithmException if the algorithm for recovering the
     * key cannot be found
     * @exception UnrecoverableKeyException if the key cannot be recovered
     * (e.g., the given password is wrong).
     */
    public Key engineGetKey(String alias, char[] password)
	throws NoSuchAlgorithmException, UnrecoverableKeyException
	{
		PrivateKeyEntry entry = (PrivateKeyEntry)mPrivateKeys.get(alias);
		
		if (entry == null)
			return null;

		if (!entry.validate(password))
			throw new UnrecoverableKeyException();

				
		// OK they passed, give them what they want
		return entry.mKey;
	}

    /**
     * Returns the certificate chain associated with the given alias.
     *
     * @param alias the alias name
     *
     * @return the certificate chain (ordered with the user's certificate first
     * and the root certificate authority last), or null if the given alias
     * does not exist or does not contain a certificate chain (i.e., the given
     * alias identifies either a <i>trusted certificate entry</i> or a
     * <i>key entry</i> without a certificate chain).
     */
    public Certificate[] engineGetCertificateChain(String alias)
	{
		PrivateKeyEntry entry = (PrivateKeyEntry)mPrivateKeys.get(alias);
		if (entry != null)
		{
			// Return all the certificates
			return entry.mCertificates;
		}
		return null;
	}

    /**
     * Returns the certificate associated with the given alias.
     *
     * <p>If the given alias name identifies a
     * <i>trusted certificate entry</i>, the certificate associated with that
     * entry is returned. If the given alias name identifies a
     * <i>key entry</i>, the first element of the certificate chain of that
     * entry is returned, or null if that entry does not have a certificate
     * chain.
     *
     * @param alias the alias name
     *
     * @return the certificate, or null if the given alias does not exist or
     * does not contain a certificate.
     */
    public Certificate engineGetCertificate(String alias)
	{
		// First check the Trusted Certificates for the given alias
		Object cert = mTrustedCerts.get(alias);
		if (cert != null)
			return (Certificate)cert;
		
		// Now try the private keys
		Object pke = mPrivateKeys.get(alias);
		if (pke != null)
		{
			// Return the first certificate if there is one
			PrivateKeyEntry entry = (PrivateKeyEntry)pke;
			return entry.mCertificates != null ? entry.mCertificates[0] : null;
		}
		return null;
	}

    /**
     * Returns the creation date of the entry identified by the given alias.
     *
     * @param alias the alias name
     *
     * @return the creation date of this entry, or null if the given alias does
     * not exist
     */
    public Date engineGetCreationDate(String alias)
	{
		throw new RuntimeException("not implemented");
	}

    /**
     * Assigns the given key to the given alias, protecting it with the given
     * password.
     *
     * <p>If the given key is of type <code>java.security.PrivateKey</code>,
     * it must be accompanied by a certificate chain certifying the
     * corresponding public key.
     *
     * <p>If the given alias already exists, the keystore information
     * associated with it is overridden by the given key (and possibly
     * certificate chain).
     *
     * @param alias the alias name
     * @param key the key to be associated with the alias
     * @param password the password to protect the key
     * @param chain the certificate chain for the corresponding public
     * key (only required if the given key is of type
     * <code>java.security.PrivateKey</code>).
     *
     * @exception KeyStoreException if the given key cannot be protected, or
     * this operation fails for some other reason
     */
    public void engineSetKeyEntry(String alias, Key key,
					   char[] password,
					   Certificate[] chain)
		throws KeyStoreException
	{
		if (mTrustedCerts.containsKey(alias))
			throw new KeyStoreException("Trusted Cert exists with given alias");
			
		PrivateKeyEntry entry = (PrivateKeyEntry)mPrivateKeys.get(alias);

		if (entry != null)
		{
			// Already exists, validate password
			if (!entry.validate(password))
				throw new KeyStoreException("Invalid password");
		}
		else
		{
			// Never seen this alias before, so create an entry and add to the store
			entry = new PrivateKeyEntry();
			mPrivateKeys.put(alias, entry);
		}

		// Fill out the entry, note we copy both arrays.
		entry.mKey = key;
		entry.mPassword = new char[password.length];
		System.arraycopy(password, 0, entry.mPassword, 0, password.length);
		
		if (key instanceof PrivateKey)
		{
			entry.mCertificates = new Certificate[chain.length];
			System.arraycopy(chain, 0, entry.mCertificates, 0, chain.length);
		}
	}

    /**
     * Assigns the given key (that has already been protected) to the given
     * alias.
     *
     * <p>If the protected key is of type
     * <code>java.security.PrivateKey</code>,
     * it must be accompanied by a certificate chain certifying the
     * corresponding public key.
     *
     * <p>If the given alias already exists, the keystore information
     * associated with it is overridden by the given key (and possibly
     * certificate chain).
     *
     * @param alias the alias name
     * @param key the key (in protected format) to be associated with the alias
     * @param chain the certificate chain for the corresponding public
     * key (only useful if the protected key is of type
     * <code>java.security.PrivateKey</code>).
     *
     * @exception KeyStoreException if this operation fails.
     */
    public void engineSetKeyEntry(String alias, byte[] key,
					   Certificate[] chain)
		throws KeyStoreException
	{
		throw new RuntimeException("not implemented");
	}

    /**
     * Assigns the given certificate to the given alias.
     *
     * <p>If the given alias already exists in this keystore and identifies a
     * <i>trusted certificate entry</i>, the certificate associated with it is
     * overridden by the given certificate.
     *
     * @param alias the alias name
     * @param cert the certificate
     *
     * @exception KeyStoreException if the given alias already exists and does
     * not identify a <i>trusted certificate entry</i>, or this operation
     * fails for some other reason.
     */
    public void engineSetCertificateEntry(String alias,
						   Certificate cert)
		throws KeyStoreException
	{
		if (mPrivateKeys.get(alias) != null)
			throw new KeyStoreException("Alias exists as a Private Key Entry");
			
		mTrustedCerts.put(alias, cert);
	}

    /**
     * Deletes the entry identified by the given alias from this keystore.
     *
     * @param alias the alias name
     *
     * @exception KeyStoreException if the entry cannot be removed.
     */
    public void engineDeleteEntry(String alias)
	throws KeyStoreException
	{
		Object value;
		
		value = mPrivateKeys.remove(alias);
		
		if (mPrivateKeys.remove(alias) != null)
			return;
			
		if (mTrustedCerts.remove(alias) != null)
			return;
			
		throw new KeyStoreException("Alias not found - " + alias);
	}

    /**
     * Lists all the alias names of this keystore.
     *
     * @return enumeration of the alias names
     */
    public Enumeration	engineAliases()
	{
		return new Enumeration()
		{
			boolean first = true;
			Enumeration enum = mTrustedCerts.keys();
			
			public boolean hasMoreElements()
			{
				if (!swap())
					return false;
				return enum.hasMoreElements();
			}
			
			public Object nextElement()
			{
				if (!swap())
					return null;
				return enum.nextElement();
			}
			
			private boolean swap()
			{
				if (!enum.hasMoreElements())
				{
					if (!first)
						return false;
					first = false;
					enum = mPrivateKeys.keys();
				}
				return true;
			}
		};
	}

    /**
     * Checks if the given alias exists in this keystore.
     *
     * @param alias the alias name
     *
     * @return true if the alias exists, false otherwise
     */
    public boolean engineContainsAlias(String alias)
	{
		if (mPrivateKeys.containsKey(alias))
			return true;
			
		return mTrustedCerts.containsKey(alias);
	}
	
    /**
     * Retrieves the number of entries in this keystore.
     *
     * @return the number of entries in this keystore
     */
    public int engineSize()
	{
		return mPrivateKeys.size() + mTrustedCerts.size();
	}

    /**
     * Returns true if the entry identified by the given alias is a
     * <i>key entry</i>, and false otherwise.
     *
     * @return true if the entry identified by the given alias is a
     * <i>key entry</i>, false otherwise.
     */
    public boolean engineIsKeyEntry(String alias)
	{
		return mPrivateKeys.containsKey(alias);
	}

    /**
     * Returns true if the entry identified by the given alias is a
     * <i>trusted certificate entry</i>, and false otherwise.
     *
     * @return true if the entry identified by the given alias is a
     * <i>trusted certificate entry</i>, false otherwise.
     */
    public boolean engineIsCertificateEntry(String alias)
	{		
		return mTrustedCerts.containsKey(alias);
	}

    /**
     * Returns the (alias) name of the first keystore entry whose certificate
     * matches the given certificate.
     *
     * <p>This method attempts to match the given certificate with each
     * keystore entry. If the entry being considered
     * is a <i>trusted certificate entry</i>, the given certificate is
     * compared to that entry's certificate. If the entry being considered is
     * a <i>key entry</i>, the given certificate is compared to the first
     * element of that entry's certificate chain (if a chain exists).
     *
     * @param cert the certificate to match with.
     *
     * @return the (alias) name of the first entry with matching certificate,
     * or null if no such entry exists in this keystore.
     */
    public String engineGetCertificateAlias(Certificate cert)
	{
		if (mTrustedCerts.contains(cert))
		{
			for (Enumeration enum = mTrustedCerts.keys(); 
				 enum.hasMoreElements();)
			{
				String alias = (String)enum.nextElement();
				Certificate acert = (Certificate)mTrustedCerts.get(alias);
				if (acert.equals(cert))
					return alias;
			}
			
			// At this point we have a fundamental problem, the hashtable
			// thinks there is a matching value but the equals() never equated.
			return null;
		}			 
		
		for (Enumeration enum = mPrivateKeys.keys(); 
			 enum.hasMoreElements();)
		{
			String alias = (String)enum.nextElement();
			PrivateKeyEntry entry = (PrivateKeyEntry)mPrivateKeys.get(alias);
			if (entry.mCertificates != null && entry.mCertificates[0].equals(cert))
				return alias;
		}

		return null;
	}

    /**
     * Stores this keystore to the given output stream, and protects its
     * integrity with the given password.
     *
     * @param stream the output stream to which this keystore is written.
     * @param password the password to generate the keystore integrity check
     *
     * @exception IOException if there was an I/O problem with data
     * @exception NoSuchAlgorithmException if the appropriate data integrity
     * algorithm could not be found
     * @exception CertificateException if any of the certificates included in
     * the keystore data could not be stored
     */
    public synchronized void engineStore(OutputStream stream, char[] password)
	throws IOException, NoSuchAlgorithmException, CertificateException
	{
		// Create a nice little object stream to save our stuff.
		ByteArrayOutputStream outs = new ByteArrayOutputStream();
		SecurityOutputStream out = new SecurityOutputStream(outs);

		store(out);
		out.flush();
		
		byte[] input = outs.toByteArray();
		
		// Convert from char[] to byte[]
		byte[] pwd = new byte[password.length];
		for (int i = 0; i < password.length; i++)
			pwd[i] = (byte)password[i];
		
		// Create some salt
		MessageDigest md = MessageDigest.getInstance("MD5");
		md.update(pwd);
		md.update(input);
		byte[] digest = md.digest();
		
		// Move digest into the salt
		byte[] salt = new byte[8];
		System.arraycopy(digest, 0, salt, 0, salt.length);
		
		byte[] output = encrypt(Cipher.ENCRYPT_MODE, salt, password, input);
			
		// Finally get the data out the stream
		stream.write(salt);
		stream.write(output);
		
		out.close();
	}

    /**
     * Loads the keystore from the given input stream.
     *
     * <p>If a password is given, it is used to check the integrity of the
     * keystore data. Otherwise, the integrity of the keystore is not checked.
     *
     * @param stream the input stream from which the keystore is loaded
     * @param password the (optional) password used to check the integrity of
     * the keystore.
     *
     * @exception IOException if there is an I/O or format problem with the
     * keystore data
     * @exception NoSuchAlgorithmException if the algorithm used to check
     * the integrity of the keystore cannot be found
     * @exception CertificateException if any of the certificates in the
     * keystore could not be loaded
     */
    public void engineLoad(InputStream stream, char[] password)
	throws IOException, NoSuchAlgorithmException, CertificateException
	{
		byte[] salt = new byte[8];
		stream.read(salt);

		// Read remaining data from the stream
		ByteArrayOutputStream store = new ByteArrayOutputStream();
		int b;
		while((b = stream.read()) != -1)
		{
			store.write(b);
		}
		
		byte[] input = store.toByteArray();
		byte[] origData = encrypt(Cipher.DECRYPT_MODE, salt, password, input);

		ByteArrayInputStream ins = new ByteArrayInputStream(origData);
		SecurityInputStream in = new SecurityInputStream(ins);
		
		try
		{
			load(in);
		}
		catch(InvalidKeySpecException e)
		{e.printStackTrace();
			// It was not possible to re-create a key that was stored.
			// +++ Is this the correct handling of this exception
			throw new CertificateException(e.getMessage());
		}
		catch(ClassNotFoundException e)
		{
			// All the classes instantiated, are known and must be present
			// so this just can't happen
			throw new ExceptionInInitializerError(e);
		}
	}

	/* Saves the current Key store to the stream
	 * Version (float)
	 * Count of Trusted Certs (int)
	 * [0..*]
	 *     alias (string)
	 *     Cert
	 * Count of Private Keys (int)
	 * [0..*]
	 *     alias (string)
	 *     Password
	 *     Key
	 *     Count of Certs (int)
	 *     [0..*]
	 *         Cert
	 */
	private void store(SecurityOutputStream out)
		throws IOException, CertificateException
	{
		// Write The trusted certs.
		out.writeInt(mTrustedCerts.size());
		for (Enumeration enum = mTrustedCerts.keys(); enum.hasMoreElements();)
		{
			String alias = (String)enum.nextElement();
			out.writeObject(alias);
			out.writeCert((Certificate)mTrustedCerts.get(alias));
		}
			
		// Write The trusted certs.
		out.writeInt(mPrivateKeys.size());
		for (Enumeration enum = mPrivateKeys.keys(); enum.hasMoreElements();)
		{
			String alias = (String)enum.nextElement();
			out.writeObject(alias);
			
			PrivateKeyEntry pke = (PrivateKeyEntry)mPrivateKeys.get(alias);

			// Write the password
			out.writePassword(pke.mPassword);
			
			// Write the key
			out.writeKey(pke.mKey);
			
			// Write the cert chain
			int certCount = pke.mCertificates != null ? pke.mCertificates.length : 0;
			out.writeInt(certCount);
			for (int i = 0; i < certCount; i++)
				out.writeCert(pke.mCertificates[i]);
		}
	}
	
	/* Loads the current Key store to the stream
	 * Version (float)
	 * Count of Trusted Certs (int)
	 * [0..*]
	 *     alias (string)
	 *     Cert
	 * Count of Private Keys (int)
	 * [0..*]
	 *     alias (string)
	 *     Password
	 *     Key
	 *     Count of Certs (int)
	 *     [0..*]
	 *         Cert
	 */
	private void load(SecurityInputStream in)
		throws IOException, CertificateException, ClassNotFoundException,
		InvalidKeySpecException, NoSuchAlgorithmException
	{
		mTrustedCerts.clear();
		
		// Read The trusted certs.
		int trustedCertCount = in.readInt();
		for (int i = 0; i < trustedCertCount; i++)
		{
			String alias = (String)in.readObject();
			Certificate cert = in.readCert();
			mTrustedCerts.put(alias, cert);
		}
			
		// Read The private keys.
		mPrivateKeys.clear();
		int privateKeysCount = in.readInt();
		for (int i = 0; i < privateKeysCount; i++)
		{
			String alias = (String)in.readObject();
			
			PrivateKeyEntry pke = new PrivateKeyEntry();

			mPrivateKeys.put(alias, pke);
			
			// Write the password
			pke.mPassword = in.readPassword();
			
			// Write the key
			pke.mKey = in.readKey();
			
			// Write the cert chain
			int certCount = in.readInt();
			if (certCount != 0)
				pke.mCertificates = new Certificate[certCount];
				
			for (int j = 0; j < certCount; j++)
				pke.mCertificates[j] = in.readCert();
		}
	}
	
	private static final float VERSION = 1.0f;
	private class SecurityOutputStream extends ObjectOutputStream
	{
	
		public SecurityOutputStream(OutputStream out)
			throws IOException
		{
			super(out);
		}
		
		protected void writeStreamHeader()
			throws IOException
		{
			// Write a header
			writeFloat(VERSION);
		}
		
		/**
		 * Writes a Key to the stream:<ol>
		 *   Type (int 1..3)</li>
		 *   Algorithm (String)</li>
		 *   Format (String)</li>
		 *   Encoded (byte [])</li></ol>
		 */
		public void writeKey(Key key)
			throws IOException
		{
			if (key instanceof PrivateKey)
				writeInt(1);
			else if (key instanceof PublicKey)
				writeInt(2);
			else if (key instanceof SecretKey)
				writeInt(3);
			else
				throw new IOException("Unknown Key - " + key);
				
			writeObject(key.getAlgorithm());
			writeObject(key.getFormat());
			writeArray(key.getEncoded());
		}
		
		/**
		 * Writes a Certificate to the stream<ol>
		 *   Type (String)</li>
		 *   Encoded (byte[])</li></ol>
		 */
		public void writeCert(Certificate cert)
			throws IOException, CertificateEncodingException
		{
			writeObject(cert.getType());
			writeArray(cert.getEncoded());
		}
		
		/**
		 * Writes a password to the stream<ol>
		 *   Length (String)</li>
		 *   bytes (char 0..*)</li></ol>
		 */
		public void writePassword(char[] pwd)
			throws IOException
		{
			writeInt(pwd.length);
			for (int i = 0; i < pwd.length; i++)
				writeChar(pwd[i]);
		}
		
		/**
		 * Writes an array of bytes, first storing the size and then the array
		 */
		public void writeArray(byte[] array)
			throws IOException
		{
			writeInt(array.length);
			write(array);
		}
	}
	
	private class SecurityInputStream extends ObjectInputStream
	{
		public SecurityInputStream(InputStream in)
			throws IOException
		{
			super(in);
		}
		
		protected void readStreamHeader()
			throws IOException
		{
			// read a header
			float version = readFloat();
			if (version != VERSION)
				throw new IOException("Invalid stream version:" + version);
		}
		
		/**
		 * Writes a Key to the stream:<ol>
		 *   Type (int 1..3)</li>
		 *   Algorithm (String)</li>
		 *   Format (String)</li>
		 *   Encoded (byte [])</li></ol>
		 */
		public Key readKey()
			throws IOException, ClassNotFoundException, 
			NoSuchAlgorithmException, InvalidKeySpecException
		{
			int type = readInt();
			String algorithm = (String)readObject();
			String format = (String)readObject();
			byte[] encode = readArray();
			
			KeySpec keySpec;
			if (format.equals("PKCS#8"))
				keySpec = new PKCS8EncodedKeySpec(encode);
			else if (format.equals("X.509"))
				keySpec = new X509EncodedKeySpec(encode);
			else if (format.equals("ASCII"))
				keySpec = new AsciiEncodedKeySpec(new String(encode));
			else
				throw new IOException("Unknown Key format: " + format);
				
			switch(type)
			{
			case 1:	// PrivateKey
				KeyFactory kf1 = KeyFactory.getInstance(algorithm);
				return kf1.generatePrivate(keySpec);
			case 2: // PublicKey
				KeyFactory kf2 = KeyFactory.getInstance(algorithm);
				return kf2.generatePublic(keySpec);
			case 3: // SecretKey
				SecretKeyFactory kf3 = SecretKeyFactory.getInstance(algorithm);
				return kf3.generateSecret(keySpec);
			default:
				throw new IOException("Unknown KeyType");
			}
		}
		
		/**
		 * Reads a Certificate from the stream<ol>
		 *   Type (String)</li>
		 *   Encoded (byte[])</li></ol>
		 */
		public Certificate readCert()
			throws IOException, ClassNotFoundException, CertificateException
		{
			String type = (String)readObject();
			byte[] encoded = readArray();
			
			CertificateFactory cf = CertificateFactory.getInstance(type);
			
			ByteArrayInputStream in = new ByteArrayInputStream(encoded);
			return cf.generateCertificate(in);
		}
		
		/**
		 * Reads a password from the stream<ol>
		 *   Length (String)</li>
		 *   bytes (char 0..*)</li></ol>
		 */
		public char[] readPassword()
			throws IOException
		{
			int length = readInt();
			char pwd[] = new char[length];
			for (int i = 0; i < pwd.length; i++)
				pwd[i] = readChar();
			return pwd;
		}
		
		/**
		 * Reads an array of bytes, first storing the size and then the array
		 */
		public byte[] readArray()
			throws IOException
		{
			int length = readInt();
			byte[] array = new byte[length];
			read(array, 0, array.length);
			return array;
		}
	}
	
	private byte[] encrypt(int code, byte[] salt, char[] password, byte[] input)
		throws NoSuchAlgorithmException
	{
		// Algorithm parameters
		AlgorithmParameterSpec aps = new PBEParameterSpec(salt, 20); // 20 iterations
		
		// Encrypt the input now
		try
		{
			Cipher cipher = Cipher.getInstance(CIPHER, "ABA");
			Key key = createKey(password);
			cipher.init(code, key, aps);
			return cipher.doFinal(input);
		}
		catch(java.security.NoSuchProviderException e)
		{
			throw new ExceptionInInitializerError(e);
		}
		catch(InvalidAlgorithmParameterException e)
		{
			// We are smart enough not to get this wrong
			throw new ExceptionInInitializerError(e);
		}
		catch(NoSuchPaddingException e)
		{
			// This too.
			throw new ExceptionInInitializerError(e);
		}
		catch(BadPaddingException e)
		{
			throw new ExceptionInInitializerError(e);
		}
		catch(javax.crypto.IllegalBlockSizeException e)
		{
			// Most likely this as well.
			throw new ExceptionInInitializerError(e);
		}
		catch(InvalidKeyException e)
		{
			// Most likely this as well.
			throw new ExceptionInInitializerError(e);
		}
	}
	
	private Key createKey(char[] password)
		throws NoSuchAlgorithmException
	{
		KeySpec keySpec = new PBEKeySpec(password);
		SecretKeyFactory skf = SecretKeyFactory.getInstance(CIPHER);
		try
		{
			return skf.generateSecret(keySpec);
		}
		catch(InvalidKeySpecException e)
		{
			// We won't get this wrong
			throw new ExceptionInInitializerError(e);
		}
	}
}
