/*
 * $Id: RSA.java,v 1.12 1999/01/15 00:35:26 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/provider/RSA.java,v $
 * $Revision: 1.12 $
 * $Date: 1999/01/15 00:35:26 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import au.net.aba.security.*;
import au.net.aba.security.*;
import au.net.aba.security.InvalidAlgorithmParameterException;
import au.net.aba.security.spec.AlgorithmParameterSpec;
import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;

/**
 * A class that provides RSA public/private key encryption and
 * decryption as specified in PKCS#1.
 * <p>
 * By default this cipher will use PKCS#1 padding however it
 * also supports NoPadding.
 */
public class RSA extends BlockCipher
{
	public final static String ident = "$Id: RSA.java,v 1.12 1999/01/15 00:35:26 leachbj Exp $";

	private static final int HEADER_SIZE = 11;  // PKCS1 - with padding

	private RSAPublicKey	rsakey;	// the key we use
	private boolean padded;		// PKCS1 padding?
	
	private int significantBytes = HEADER_SIZE;

	public RSA()
	{
		padded = true;
		paddedStream = false;
	}

	protected void setKey(Key key)
	throws InvalidKeyException
	{
		if (!(key instanceof RSAPrivateKey)
			&& !(key instanceof RSAPublicKey))
		{
			throw new InvalidKeyException(
				"expecting RSAPrivateKey/RSAPublicKey");
		}

		/*
		 * the RSA private keys extend RSAPublicKey
		 */
		rsakey = (RSAPublicKey)key;
		significantBytes = rsakey.significantBytes();

		/*
		 * reset our buffer size, we need to do this because out
		 * block size just changed.
		 */
		buffer = new byte[engineGetBlockSize()];
	}

	/**
	 * Sets the mode of this cipher.  Only ECB is supported.
	 *
	 * @param mode  the mode of the cipher (ECB, CBC).  Currently only
	 *    ECB is supported.
	 * @exception NoSuchAlgorithmException if the mode is not supported.
	 */
	protected void engineSetMode(String mode)
	throws NoSuchAlgorithmException
	{
		if (!mode.equals("ECB"))
		{
			throw new NoSuchAlgorithmException(
				"RSA only supports ECB.");
		}
	}

	/**
	 * Sets the padding mechanism of this cipher. 
	 *
	 * @param padding the name of the type of padding to be applied,
	 * 	currently "PKCS1Padding", and "NoPadding" will
	 * 	be accepted.
	 * @exception NoSuchPaddingException if the padding type is unknown.
	 */
	protected void engineSetPadding(
		String padding)
	throws NoSuchPaddingException
	{
		if (padding.equals("PKCS1Padding"))
		{
			padded = true;
		}
		else if (padding.equals("NoPadding"))
		{
			padded = false;
		}
		else
		{
			throw new NoSuchPaddingException(
				"RSA only supports PKCS1.");
		}
	}

	/**
	 * Returns the block size.  In encryption mode this will be
	 * the size of the key less the header bytes, in decryption
	 * mode it will be the size of the key.
	 */
	protected int engineGetBlockSize()
	{
		if (mode == Cipher.DECRYPT_MODE)
		{
			return significantBytes;
		}

		return significantBytes - HEADER_SIZE;
	}

	/**
	 * Returns the length in bytes that an output buffer would need to be
	 * in order to hold the result of the next update or doFinal operation,
	 * given the input length inputLen (in bytes).
	 */
	protected int engineGetOutputSize(int inputLen)
	{
		int blockSize = engineGetBlockSize();

		inputLen += bufferPos;

		if (mode == Cipher.DECRYPT_MODE)
		{
			return inputLen / blockSize * (blockSize - HEADER_SIZE);
		}

		return (inputLen + blockSize - 1) / blockSize
			* significantBytes;
	}

	/**
	 * This method encrypts the specified array, placing the
	 * ciphered data into the destination array.
	 *
	 * @param src		The plain text.
	 * @param srcIndex	The index from which to read.
	 * @param len		The length of the plain text.
	 * @param dst		The cipher text.
	 * @param dstIndex	The index at which to write.
	 * @return		The number of bytes processed.
	 */
	protected final int encryptBlock(
		byte[]	src,
		int	srcIndex,
		int	len,
		byte[]	dst,
		int	dstIndex)
	{
		/*
		 * create a PKCS1 block (from the PKCS-1 standard);
		 *
		 * EB = 00 || BT || PS || 00 || D
		 *
		 * BT is
		 * 	00 for private key encryption with no padding
		 * 	01 for private key padding (each byte is 0xFF)
		 * 	02 for public key padding (each byte is rand none zero)
		 *
		 * PS is the padding string of length k-3-||D||.  That
		 * is (significantBytes - 3 - data.length).
		 *
		 * D is the data, maximum length k - 11
		 */

		int blockSize = engineGetBlockSize();

		/*
		 * create the new block (all zeros)
		 */
		byte[] block = new byte[significantBytes];

		/*
		 * add the correct padding
		 */
		if (padded)
		{
			if (rsakey instanceof RSAPrivateKey)
			{
				block[0] = 0x00;	// so we are < the key!
				block[1] = 0x01;

				for (int i = 2; i != block.length; i++)
				{
					block[i] = (byte)0xFF;
				}
			}
			else
			{
				random.nextBytes(block); // random fill
				
				block[0] = 0x00;	// so we are < the key!
				block[1] = 0x02;

				/*
				 * all our random bytes must be <> 0
				 */
				for (int i = 2; i != block.length; i++)
				{
					if (block[i] == 0)
					{
						block[i] = (byte)(0xFF & ~i);
					}
				}
			}

			/*
			 * mark the end of the padding
			 */
			block[block.length - len - 1] = 0x00;
		}

		/*
		 * copy in the data
		 */
		System.arraycopy(src, srcIndex, block,
			block.length - len, len);

		/*
		 * encrypt the block
		 */
		block = rsakey.encrypt(block);

		/*
		 * as the byte format returned is in twos complement we
		 * sometimes get a byte representation that is one byte longer
		 * than we want. We need to ignore this byte (it is always
		 * zero anyway).
		 */
		if (block.length >= significantBytes)
		{
			System.arraycopy(block, block.length - significantBytes,
				dst, dstIndex, significantBytes);
		}
		else
		{
			int off = significantBytes - block.length;

			/*
			 * if the number is smaller...
			 */
			for (int i = 0; i <= off; i++)
			{
				dst[dstIndex + i] = 0;
			}

			System.arraycopy(block, 0, dst, dstIndex + off,
				block.length);
		}

		return significantBytes;
	}

	/**
	 * This method decrypts the specified array, placing the plain text
	 * data into the destination array.
	 *
	 * @param src		The cipher text.
	 * @param srcIndex	The index from which to read.
	 * @param len		The lenght of the cipher text.
	 * @param dst		The plain text.
	 * @param dstIndex	The index at which to write.
	 * @return		The number of bytes of plaintext.
	 */
	protected final int decryptBlock(
		byte[]	src,
		int	srcIndex,
		int	len,
		byte[]	dst,
		int	dstIndex)
	{
		/*
		 * copy the source data into an array of correct length
		 */
		byte[] enc = new byte[significantBytes];
		System.arraycopy(src, srcIndex, enc, 0, enc.length);

		/*
		 * decrypt the block
		 */
		byte[] block = rsakey.decrypt(enc);
		int offset = 0;

		/*
		 * determine the start of the data portion of
		 * the packet according to the padding type
		 */
		if (padded)
		{
			if (block[0] != 0x01 && block[0] != 0x02)
			{
				throw new RuntimeException("Bad block type");
			}

			for (offset = 1; offset != block.length; offset++)
			{
				if (block[offset] == 0)
				{
					break;
				}
			}

			offset++;	// data starts in the next byte
		}

		/*
		 * there can be at most (significantBytes - HEADER_SIZE)
		 * bytes of data in the packet
		 */
		int resultLen = block.length - offset;
		if (resultLen <= 0
			|| resultLen > significantBytes - HEADER_SIZE)
		{
			// throw new IllegalBlockSizeException("Invalid PKCS1 block");
			throw new RuntimeException("Invalid PKCS1 block");
		}

		System.arraycopy(block, offset, dst, dstIndex, resultLen);

		return resultLen;
	}
}

