/*
 * $Id: RSAKeyFactory.java,v 1.7 1998/10/27 05:10:40 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/provider/RSAKeyFactory.java,v $
 * $Revision: 1.7 $
 * $Date: 1998/10/27 05:10:40 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import au.net.aba.security.*;
import au.net.aba.security.spec.InvalidKeySpecException;
import au.net.aba.security.spec.KeySpec;
import au.net.aba.crypto.spec.AsciiEncodedKeySpec;
import au.net.aba.math.BigInteger;
import au.net.aba.security.spec.RSAPrivateCrtKeySpec;
import au.net.aba.security.spec.RSAPublicKeySpec;
import java.util.StringTokenizer;

/**
 * This class is used to convert RSA keys into a format usable by the
 * ABA provider.  Currently this class can only convert from a KeySpec
 * into a Key.  The supported KeySpec classes are AsciiEncodedKeySpec,
 * RSAPrivateCrtKeySpec and RSAPublicKeySpec.
 * <p>
 * This class should not be instantiated directly, instead use the
 * java.security.KeyFactory interface.
 *
 * @see KeyFactory
 */
public class RSAKeyFactory extends KeyFactory
{
	public final static String ident = "$Id: RSAKeyFactory.java,v 1.7 1998/10/27 05:10:40 leachbj Exp $";

	/**
	 * Generates a private key object from the provided key specification
	 * (key material).  This class supports the AsciiEncodedKeySpec and
	 * RSAPrivateCrtKeySpec KeySpec classes.
	 *
	 * @returns	The private key.
	 * @exception InvalidKeySpecException	The provided KeySpec was not
	 * 	an AsciiEncodedKeySpec or RSAPrivateCrtKeySpec KeySpec.
	 */
	protected PrivateKey engineGeneratePrivate(
		KeySpec	keySpec)
	throws InvalidKeySpecException
	{
		if (keySpec instanceof AsciiEncodedKeySpec)
		{
			/*
			 * Convert the AsciiEncodedKeySpec into a RSAPrivateCrtKeySpec
			 */
			AsciiEncodedKeySpec aKeySpec =
				(AsciiEncodedKeySpec)keySpec;

			BigInteger bigInts[] = parseKey(aKeySpec.getEncoded());
			if (bigInts.length == 8)
			{
				keySpec = new RSAPrivateCrtKeySpec(
					bigInts[0],	// modulus
					bigInts[1],	// publicExponent
					bigInts[2],	// privateExponent
					bigInts[3],	// primeP
					bigInts[4],	// primeQ
					bigInts[5],	// dP
					bigInts[6],	// dQ
					bigInts[7]);	// qInv
			}
			else
			{
				throw new InvalidKeySpecException(
					"Incomplete AsciiEncodedKeySpec.");
			}
		}
		else if (!(keySpec instanceof RSAPrivateCrtKeySpec))
		{
			throw new InvalidKeySpecException(
				"Unknown KeySpec type.");
		}

		return new RSAPrivateKey((RSAPrivateCrtKeySpec)keySpec);
	}

	/**
	 * Generates a public key object from the provided key specification
	 * (key material).  This class supports the AsciiEncodedKeySpec and
	 * RSAPublicKeySpec KeySpec classes.
	 *
	 * @returns	The public key.
	 * @exception InvalidKeySpecException	The provided KeySpec was not
	 * 	an AsciiEncodedKeySpec or RSAPublicKeySpec KeySpec.
	 */
	protected PublicKey engineGeneratePublic(
		KeySpec	keySpec)
	throws InvalidKeySpecException
	{
		if (keySpec instanceof AsciiEncodedKeySpec)
		{
			/*
			 * Convert the AsciiKeySpec directly into a RSA
			 * Public key.
			 */
			AsciiEncodedKeySpec aKeySpec =
				(AsciiEncodedKeySpec)keySpec;

			BigInteger bigInts[] = parseKey(aKeySpec.getEncoded());
			if (bigInts.length >= 2)
			{
				return new RSAPublicKey(
					bigInts[0],	// modulus
					bigInts[1]);	// publicExponent
			}
			else
			{
				throw new InvalidKeySpecException(
					"Incomplete AsciiEncodedKeySpec.");
			}
		}
		else if (keySpec instanceof RSAPublicKeySpec)
		{
			RSAPublicKeySpec spec = (RSAPublicKeySpec)keySpec;

			return new RSAPublicKey(
				spec.getModulus(),
				spec.getPublicExponent());
		}
		else
		{
			throw new InvalidKeySpecException(
				"Unknown KeySpec type.");
		}
	}

	/**
	 * Returns a specification (key material) of the given key object.
	 * keySpec identifies the specification class in which the key material
	 * should be returned. It could, for example, be DSAPublicKeySpec.class,
	 * to indicate that the key material should be returned in an instance
	 * of the DSAPublicKeySpec class.
	 * <p>
	 * Not currently implemented.
	 */
	protected KeySpec engineGetKeySpec(
		Key	key,
		Class	spec)
	{
		System.err.println(
			"RSAKeyFactory: engineGetKeySpec not implemented.");

		return null;
	}

	/**
	 * Translates a key object, whose provider may be unknown or
	 * potentially untrusted, into a corresponding key object of this key
	 * factory.
	 * <p>
	 * Not currently implemented.
	 */
	protected Key engineTranslateKey(
		Key	key)
	{
		System.err.println(
			"RSAKeyFactory: engineTranslateKey not implemented.");

		return null;
	}

	/**
	 * Parse a key from the ABA/PGP format into an array of
	 * BigIntegers.  The format is:
	 * modulus.publicExp.privateExp.p.q
	 */
	protected static BigInteger[] parseKey(
		byte[] keyBytes)
	{
		String s = new String(keyBytes, 0);
		StringTokenizer st = new StringTokenizer(s, ".");

		BigInteger[]	bigInts;
		int size = st.countTokens();
		if (size > 2)
		{
			bigInts = new BigInteger[8];
		}
		else
		{
			bigInts = new BigInteger[2];
		}

		for (int i = 0; i != size; i++)
		{
			bigInts[i] = new BigInteger(st.nextToken(), 16);
		}

		if (size > 2)
		{
			BigInteger	privateExponent;
			BigInteger	primeP, primeQ, pMinus1, qMinus1;

			privateExponent = bigInts[2];
			primeP = bigInts[3];
			primeQ = bigInts[4];

			/*
			 *  ensure that p is the larger of the two primes
			 */
			if (primeP.compareTo(primeQ) < 0)
			{       
				BigInteger temp;

				temp = primeP; 
				bigInts[3] = primeP = primeQ;
				bigInts[4] = primeQ = temp;
			}

			pMinus1 = primeP.subtract(BigInteger.valueOf(1));
			qMinus1 = primeQ.subtract(BigInteger.valueOf(1));

			bigInts[5] = privateExponent.remainder(pMinus1);
			bigInts[6] = privateExponent.remainder(qMinus1);
			bigInts[7] = primeQ.modInverse(primeP);
		}

		return bigInts;
	}
}
