/*
 * $Id: RSAPrivateKey.java,v 1.6 1998/10/26 01:45:58 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/provider/RSAPrivateKey.java,v $
 * $Revision: 1.6 $
 * $Date: 1998/10/26 01:45:58 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import java.io.*;

import javax.crypto.*;

import au.net.aba.security.*;
import au.net.aba.math.BigInteger;
import au.net.aba.security.spec.KeySpec;
import au.net.aba.security.spec.RSAPrivateCrtKeySpec;

/**
 * A class for ABA RSA private keys.
 */
public class RSAPrivateKey extends RSAPublicKey implements PrivateKey
{
	public final static String ident = "$Id: RSAPrivateKey.java,v 1.6 1998/10/26 01:45:58 leachbj Exp $";

	//==================================
	// Constructors
	//==================================

	/**
	 * Construct an empty RSAPrivateKey.
	 */
	public RSAPrivateKey()
	{
	}

	/**
	 * Construct an empty RSAPrivateKey.
	 */
	public RSAPrivateKey(
		KeySpec	keySpec)
	{
		RSAPrivateCrtKeySpec spec = (RSAPrivateCrtKeySpec)keySpec;

		this.exponent = spec.getPublicExponent();
		this.modulus = spec.getModulus();
		this.d = spec.getPrivateExponent();
		this.p = spec.getPrimeP();
		this.q = spec.getPrimeQ();
		this.dP = spec.getPrimeExponentP();
		this.dQ = spec.getPrimeExponentQ();
		this.qInv = spec.getCrtCoefficient();
	}

	//==================================
	// Public Interface
	//==================================

	/**
	 * Generate a String representation of this key.
	 *
	 * @return		The key as a string.
	 */
	public String toString()
	{
		return modulus.toString(16) + "."
			+ exponent.toString(16) + "."
			+ d.toString(16) + "."
			+ p.toString(16) + "."
			+ q.toString(16);
	}

	//==================================
	// Protected Interface
	//==================================

	/**
	 * The private exponent of this key.
	 */
	protected BigInteger d;

	/**
	 * The larger of the two prime factors.
	 */
	protected BigInteger p;

	/**
	 * The smaller of the two prime factors.
	 */
	protected BigInteger q;

	/**
	 * The multiplicative inverse of q % p [calculated].
	 */
	protected BigInteger qInv;

	/**
	 * p-1 [calculated].
	 */
	protected BigInteger pMinus1;

	/**
	 * q-1 [calculated].
	 */
	protected BigInteger qMinus1;

	protected BigInteger dP;
	protected BigInteger dQ;

	/**
	 * Encrypt a data block with this key.
	 *
	 * @param plainText	The plain text block to encrypt.
	 * @return		The encrypted data.
	 */
	protected BigInteger encrypt(
		BigInteger plainText)
	{
		return process(plainText);
	}
 
	/**
	 * Decrypt a data block with this key.
	 *
	 * @param cipherText	The cipher text block to decrypt.
	 * @return		The decrypted data.
	 */
	protected BigInteger decrypt(
		BigInteger cipherText)
	{
		return process(cipherText);
	}

	//==================================
	// Private Implementation
	//==================================

	/*
	 * Perform RSA encipherment utilising private key data and the
	 * Chinese remainder theorem. This method should be noticeably
	 * faster than the standard public key form.
	 *
	 * @param data		The plain text or cipher text to cipher.
	 * @return		The ciphered data.
	 */
	private BigInteger process(
		BigInteger data)
	{
		BigInteger mP, mQ, tmp;

		// mP = ((data mod p) ^ dP)) mod p
		mP = (data.remainder(p)).modPow(dP, p);

		// mQ = ((data mod q) ^ dQ)) mod q
		mQ = (data.remainder(q)).modPow(dQ, q);

		if (mP.compareTo(mQ) >= 0)
		{
			tmp = mP.subtract(mQ);
		}
		else
		{
			tmp = p.subtract(mQ.subtract(mP));
		}

		tmp = tmp.multiply(qInv);
		tmp = tmp.remainder(p);
		tmp = tmp.multiply(q);
		tmp = tmp.add(mQ);

		return tmp;
	}
}
