/*
 * $Id: RSAPublicKey.java,v 1.7 1999/01/15 00:34:44 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/au.net.aba/crypto/provider/RSAPublicKey.java,v $
 * $Revision: 1.7 $
 * $Date: 1999/01/15 00:34:44 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import java.io.*;
import java.util.StringTokenizer;

import javax.crypto.*;

import au.net.aba.math.BigInteger;
import au.net.aba.security.*;
import au.net.aba.security.spec.*;

/**
 * A class for ABA RSA public keys.
 */
public class RSAPublicKey implements Key, PublicKey
{
	public final static String ident = "$Id: RSAPublicKey.java,v 1.7 1999/01/15 00:34:44 leachbj Exp $";

	//==================================
	// Constructors
	//==================================

	/**
	 * Construct an empty RSAPublicKey.
	 */
	public RSAPublicKey()
	{
	}

	public RSAPublicKey(
		KeySpec	keySpec)
	{
		RSAPublicKeySpec spec = (RSAPublicKeySpec)keySpec;

		setFields(spec.getModulus(), spec.getPublicExponent());
	}

	/**
	 * Construct an RSAPublicKey from two integral components.
	 *
	 * @param modulus	The modulus value.
	 * @param exponent	The exponent value.
	 */
	public RSAPublicKey(
		BigInteger modulus,
		BigInteger exponent)
	{
		setFields(modulus, exponent);
	}

	/**
	 * This should have been the constructor above, but as we can't
	 *	chain constructors other than as the first call (and we
	 *	can't do that at line 105), we're reduced to this.
	 *
	 * @param modulus	The modulus value.
	 * @param exponent	The exponent value.
	 */
	protected void setFields(
		BigInteger modulus,
		BigInteger exponent)
	{
		this.modulus = modulus;
		this.exponent = exponent;
	}

	//==================================
	// Public Interface
	//==================================

	/**
	 * return the algorithm for this key.
	 */
	public String getAlgorithm()
	{
		return "RSA";
	}

	/**
	 * return the format this key is in.
	 */
	public String getFormat()
	{
		return "ABA";
	}

	/**
	 * Return an encoded representation for this key.  Returns a byte
	 * array containing the string as a dotted hex string.
	 *
	 * @see #toString
	 */
	public byte[] getEncoded()
	{
		String str = toString();
		byte[] enc = new byte[str.length()];
		str.getBytes(0, str.length(), enc, 0);
		return enc;
	}

	/**
	 * Determine the number of significant bytes in the key.
	 * Note: this assumes our keys will always be positive numbers.
	 *
	 * @return		The number of bytes.
	 */
	public int significantBytes()
	{
		return ((modulus.bitLength() + BITS_IN_BYTE - 1)
                                        / BITS_IN_BYTE);
	}

	/**
	 * Generate a String representation of this key.
	 *
	 * @return		The key as a string.
	 */
	public String toString()
	{
		return modulus.toString(16) + "."
			+ exponent.toString(16);
	}

	//==================================
	// Protected Interface
	//==================================

	static final int	BITS_IN_BYTE = 8;

	/**
	 * The exponent component of the two part key that is
	 * required by the RSA algorithm.
	 */
	protected BigInteger	exponent = null;

	/**
	 * The modulus component of the two part key that is
	 * required by the RSA algorithm.
	 */
	protected BigInteger	modulus = null;

	/**
	 * Encrypt a data block with this key.
	 *
	 * @param plainText	The plain text block to encrypt.
	 * @return		The encrypted data.
	 */
	protected BigInteger encrypt(
		BigInteger plainText)
	{
		return plainText.modPow(exponent, modulus);
	}

	/**
	 * Encrypt a data block with this key.
	 *
	 * @param plainText	The plain text block to encrypt. On return,
	 *			this references a new array that contains
	 *			the encrypted data.
	 */
	final byte[] encrypt(
		byte[] plainText)
	{
		BigInteger value = new BigInteger(1, plainText);

		// do the cipher
		value = encrypt(value);

		return value.toByteArray();
	}

	/**
	 * Decrypt a data block with this key.
	 *
	 * @param data		The cipher text block to decrypt.
	 * @return		The decrypted data.
	 */
	protected BigInteger decrypt(
		BigInteger cipherText)
	{
		return cipherText.modPow(exponent, modulus);
	}

	/**
	 * Decrypt a data block with this key.
	 *
	 * @param cipherText	The cipher text block to decrypt. On return,
	 *			this references a new array that contains
	 *			the plain text.
	 */
	final byte[] decrypt(
		byte[] cipherText)
	{
		BigInteger value = new BigInteger(1, cipherText);

		// do the decipher
		value = decrypt(value);

		return value.toByteArray();
	}
}

