/*
 * $Id: Twofish.java,v 1.6 1999/01/18 05:01:43 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.         
 *       
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/au.net.aba/crypto/provider/Twofish.java,v $
 * $Revision: 1.6 $
 * $Date: 1999/01/18 05:01:43 $
 * $State: Exp $
 */

package au.net.aba.crypto.provider;

import javax.crypto.*;
import javax.crypto.spec.*;

import java.security.*;

import java.security.*;
import java.security.spec.*;

import au.net.aba.crypto.spec.*;

/**
 */
public final class Twofish extends BlockCipher
{
	public final static String ident = "$Id: Twofish.java,v 1.6 1999/01/18 05:01:43 leachbj Exp $";

	//====================================
	// Useful constants
	//====================================

	protected static final int	ROUNDS = 16;

	protected static final int	TF_BLOCK_SIZE = 16; // 16bytes = 128bits

	private byte[] buf;		// general purpose block size buffer

	public final static int MAX_KEY_BITS = 256;

	// for computing subkeys
	private final static int SK_STEP = 0x02020202;
	private final static int SK_BUMP = 0x01010101;
	private final static int SK_ROTL = 9;

	// field generator
	private final static int RS_GF_FDBK = 0x14D;

	// Primitive polynomial for GF(256)
	private static final int GF256_FDBK =   0x169;
	private static final int GF256_FDBK_2 = 0x169 / 2;
	private static final int GF256_FDBK_4 = 0x169 / 4;

	// offsets for the various subkeys
	private static final int INPUT_WHITEN = 0;
	private static final int OUTPUT_WHITEN = INPUT_WHITEN+TF_BLOCK_SIZE / 4;
	private static final int ROUND_SUBKEYS = OUTPUT_WHITEN+TF_BLOCK_SIZE/ 4;

	private static final int TOTAL_SUBKEYS = 40;

	/*
	 * SBox data and round subkeys
	 */
	private int[] sBoxKeys;
	private int[] subKeys;
	private int k64Cnt;

	/**
	 * Define the fixed p0/p1 permutations used in keyed S-box lookup.
	 * By changing the following constant definitions, the S-boxes will
	 * automatically get changed in the Twofish engine.
	 */
	private static final int P_00 = 1;
	private static final int P_01 = 0;
	private static final int P_02 = 0;
	private static final int P_03 = P_01 ^ 1;
	private static final int P_04 = 1;

	private static final int P_10 = 0;
	private static final int P_11 = 0;
	private static final int P_12 = 1;
	private static final int P_13 = P_11 ^ 1;
	private static final int P_14 = 0;

	private static final int P_20 = 1;
	private static final int P_21 = 1;
	private static final int P_22 = 0;
	private static final int P_23 = P_21 ^ 1;
	private static final int P_24 = 0;

	private static final int P_30 = 0;
	private static final int P_31 = 1;
	private static final int P_32 = 1;
	private static final int P_33 = P_31 ^ 1;
	private static final int P_34 = 1;

	/**
	 * Re-key the cipher.  If the provided Key is not compatible
	 * with this cipher the exception should throw an InvalidKeyException.
	 *
	 * @param inKey      the key to be used.
	 * @exception InvalidKeyException if the key is of the wrong type.
	 */
	protected void setKey(Key inKey)
	throws InvalidKeyException
	{
		if (!((inKey instanceof TwofishKey)
				|| (inKey instanceof SecretKeySpec)))
		{
			throw new InvalidKeyException("not a Twofish Key");
		}

		prepareKey(inKey.getEncoded());
	}

	/**
	 * Sets the padding mechanism of this cipher. 
	 *
	 * @param padding	the name of the type of padding to be applied,
	 * 	currently "PKCS7Padding", and "NoPadding" will
	 * 	be accepted.
	 * @exception NoSuchPaddingException if the padding type is unknown.
	 */
	protected void engineSetPadding(
		String padding)
	throws NoSuchPaddingException
	{
		if (padding.equals("PKCS7Padding"))
		{
			paddedStream = true;
		}
		else if (padding.equals("NoPadding"))
		{
			paddedStream = false;
		}
		else
		{
			throw new NoSuchPaddingException(
				"Twofish only supports PKCS7 padding.");
		}
	}

	/**
	 * Returns the block size (in bytes).
	 *
	 * @return the block size (in bytes) of the cipher.
	 */
	protected int engineGetBlockSize()
	{
		return TF_BLOCK_SIZE;
	}

	/**
	 * Encrypt the given input starting at the given offset and place
	 * the result in the provided buffer starting at the given offset.
	 * The input will be an exact multiple of our blocksize.
	 */
	protected int encryptBlock(
		byte[] src,
		int srcIndex,
		int len,
		byte[] dst,
		int dstIndex)
	throws IllegalBlockSizeException
	{
		if (len != TF_BLOCK_SIZE)
		{
			throw new IllegalBlockSizeException("Datasize less than block size.");
		}

		int[] x = new int[TF_BLOCK_SIZE / 4];

		/*
		 * copy in the block, add whitening
		 */
		for (int i = 0; i < TF_BLOCK_SIZE / 4; i++)
		{
			x[i] = src[srcIndex++] & 0xFF
				| (src[srcIndex++] & 0xFF) << 8
				| (src[srcIndex++] & 0xFF) << 16
				| (src[srcIndex++] & 0xFF) << 24;

			x[i] ^= subKeys[INPUT_WHITEN + i];
		}

		/*
		 * main Twofish encryption loop
		 */
		for (int i = 0; i < ROUNDS; i++)
		{
			int t0, t1;

			t0 = f32(x[0], sBoxKeys, k64Cnt);
			t1 = f32(x[1] << 8 | x[1] >>> 24, sBoxKeys, k64Cnt);

			// System.out.println("t0=" + Integer.toHexString(t0) + ".\tt1=" + Integer.toHexString(t1));

			x[3] = x[3] << 1 | x[3] >>> 31;

			// PHT, round keys
			x[2] ^= t0 + t1 + subKeys[ROUND_SUBKEYS+2*i];
			x[3] ^= t0 + 2*t1 + subKeys[ROUND_SUBKEYS+2*i+1];

			x[2] = x[2] << 31 | x[2] >>> 1;

			/* swap for next round */
			if (i < ROUNDS - 1)
			{
				int tmp;
				tmp = x[0]; x[0] = x[2]; x[2] = tmp;
				tmp = x[1]; x[1] = x[3]; x[3] = tmp;
			}
		}

		/*
		 * copy out the block, with whitening
		 */
		for (int i = 0; i < TF_BLOCK_SIZE / 4; i++)
		{
			x[i] ^= subKeys[OUTPUT_WHITEN + i];
			dst[dstIndex++] = (byte)b0(x[i]);
			dst[dstIndex++] = (byte)b1(x[i]);
			dst[dstIndex++] = (byte)b2(x[i]);
			dst[dstIndex++] = (byte)b3(x[i]);
		}

		return TF_BLOCK_SIZE;
	}

	/**
	 * Decrypt the given input starting at the given offset and place
	 * the result in the provided buffer starting at the given offset.
	 * The input will be an exact multiple of our blocksize.
	 */
	protected int decryptBlock(
		byte[] src,
		int srcIndex,
		int len,
		byte[] dst,
		int dstIndex)
	throws BadPaddingException
	{
		if (len != TF_BLOCK_SIZE)
		{
			throw new BadPaddingException("Datasize less than block size.");
		}

		int[] x = new int[TF_BLOCK_SIZE / 4];

		/*
		 * copy in the block, add whitening
		 */
		for (int i = 0; i < TF_BLOCK_SIZE / 4; i++)
		{
			x[i] = src[srcIndex++] & 0xFF
				| (src[srcIndex++] & 0xFF) << 8
				| (src[srcIndex++] & 0xFF) << 16
				| (src[srcIndex++] & 0xFF) << 24;

			x[i] ^= subKeys[OUTPUT_WHITEN + i];
		}

		/*
		 * main Twofish decryption loop
		 */
		for (int i = ROUNDS - 1; i >= 0 ; i--)
		{
			int t0, t1;

			t0 = f32(x[0], sBoxKeys, k64Cnt);
			t1 = f32(x[1] << 8 | x[1] >>> 24, sBoxKeys, k64Cnt);

			x[2] = x[2] << 1 | x[2] >>> 31;

			// PHT, round keys
			x[2] ^= t0 + t1 + subKeys[ROUND_SUBKEYS+2*i];
			x[3] ^= t0 + 2*t1 + subKeys[ROUND_SUBKEYS+2*i+1];

			x[3] = x[3] << 31 | x[3] >>> 1;

			/* swap for next round */
			if (i > 0)
			{
				int tmp;
				tmp = x[0]; x[0] = x[2]; x[2] = tmp;
				tmp = x[1]; x[1] = x[3]; x[3] = tmp;
			}
		}

		/*
		 * copy out the block, with whitening
		 */
		for (int i = 0; i < TF_BLOCK_SIZE / 4; i++)
		{
			x[i] ^= subKeys[INPUT_WHITEN + i];
			dst[dstIndex++] = (byte)b0(x[i]);
			dst[dstIndex++] = (byte)b1(x[i]);
			dst[dstIndex++] = (byte)b2(x[i]);
			dst[dstIndex++] = (byte)b3(x[i]);
		}

		return TF_BLOCK_SIZE;
	}

	//==================================
	// Private Implementation
	//==================================

	/**
	 * Initialise the key schedule from the given key material.
	 *
	 * <p>Here we precompute all the round subkeys, although that is
	 * not actually required.
	 */
	private void prepareKey(byte[] key)
	{
		int[] k32e = new int[MAX_KEY_BITS/64];
		int[] k32o = new int[MAX_KEY_BITS/64];

		/*
		 * key schedule data
		 */
		sBoxKeys = new int[MAX_KEY_BITS/64];
		subKeys = new int[TOTAL_SUBKEYS];

		/*
		 * Round up to next multiple of 64bits
		 *
		 * This assumes the key has been padded out with zeros to
		 * a valid key length
		 */
		k64Cnt = (key.length + 7) / 8;

		// System.out.println("S-box key");
		int offset = 0;
		for (int i = 0; i < k64Cnt; i++)
		{
			k32e[i] = (key[offset++] & 0xFF)
				| (key[offset++] & 0xFF) << 8
				| (key[offset++] & 0xFF) << 16
				| (key[offset++] & 0xFF) << 24;

			k32o[i] = (key[offset++] & 0xFF)
				| (key[offset++] & 0xFF) << 8
				| (key[offset++] & 0xFF) << 16
				| (key[offset++] & 0xFF) << 24;

			// the sbox keys are stored in reverse order
			sBoxKeys[k64Cnt - 1 - i] = RS_MDS_Encode(k32e[i], k32o[i]);

			// System.out.println(Integer.toHexString(sBoxKeys[k64Cnt - 1 - i]));
		}

		// System.out.println("Subkeys");
		/*
		 * compute round subkeys for PHT
		 */
		for (int i = 0; i < TOTAL_SUBKEYS / 2; i++)
		{
			int A = f32(i * SK_STEP, k32e, k64Cnt);
			int B = f32(i * SK_STEP + SK_BUMP, k32o, k64Cnt);

			// System.out.print("A = " + Integer.toHexString(A));
			// System.out.println(" B = " + Integer.toHexString(B));

			B = B << 8 | B >>> 24;

			A += B;
			subKeys[2 * i] = A;
			A += B;
			subKeys[2 * i + 1] = A << SK_ROTL | A >>> (32-SK_ROTL);

			// System.out.println(Integer.toHexString(subKeys[2 * i]) + "\t" + Integer.toHexString(subKeys[2 * i + 1]));
		}
	}

	/**
	 * Use (12,8) Reed-Solomon code over GF(256) to produce a key
	 * S-box dword from two key material integers.
	 *
	 * <p>Since this computation is done only once per reKey per 64
	 * bits of key, the performance impact of this routine is
	 * imperceptible. The RS code chosen has "simple" coefficients
	 * to allow smartcard/hardware implementation without lookup
	 * tables.
	 * @return Remainder polynomial generated using RS code
	 */
	final static int RS_MDS_Encode(int k0, int k1)
	{
		int r = 0;

		for (int i = 0; i < 2; i++)
		{
			// merge in 32 more key bits
			r ^= (i == 0) ? k1 : k0;

			// shift one byte at a time
			for (int j = 0; j < 4; j++)
			{
				/*
				 * Reed-Solomon code parameters: (12,8)
				 * reversible code
				 * g(x) = x**4 + (a + 1/a) x**3
				 *        + a x**2 + (a + 1/a) x + 1
				 * where a = primitive root of field
				 * generator 0x14D
				 */
				int b = b3(r);
				int g2 = ((b << 1) ^ ((b & 0x80) != 0 ? RS_GF_FDBK : 0 )) & 0xFF;
				int g3 = ((b >>> 1) & 0x7F) ^ ((b & 0x01) != 0 ? (RS_GF_FDBK >>> 1) : 0 ) ^ g2;
				r = (r << 8) ^ (g3 << 24) ^ (g2 << 16) ^ (g3 << 8) ^ b;
			}
		}

		return r;
	}

	/**
	 * Run four bytes through keyed S-boxes and apply MDS matrix.
	 *
	 * <p>This function is a keyed 32-bit permutation.  It is the
	 * major building block for the Twofish round function,
	 * including the four keyed 8x8 permutations and the 4x4 MDS
	 * matrix multiply.  This function is used both for generating
	 * round subkeys and within the round function on the block
	 * being encrypted.
	 *
	 * @param x input to f function
	 * @param k32 pointer to key values
	 * @returns The output of the keyed permutation applied to x.
	 */
	final static int f32(
		int x,
		int[] k32,
		int k64len)
	{
		int[] b = new int[4];

		/*
		 * Run each byte thru 8x8 S-boxes, xoring with key byte
		 * at each stage.  Note that each byte goes through a
		 * different combination of S-boxes.
		 */

		/*
		 * make b[0] = LSB, b[3] = MSB
		 */
		b[0] = b0(x);
		b[1] = b1(x);
		b[2] = b2(x);
		b[3] = b3(x);

		// System.out.println(Integer.toHexString(b[0]) + " " + Integer.toHexString(b[1]) + " " + Integer.toHexString(b[2]) + " " + Integer.toHexString(b[3]) + " ");

		switch (k64len & 3)
		{
		case 0:     /* 256 bits of key */
			b[0] = (P[P_04][b[0]] & 0xFF) ^ b0(k32[3]);
			b[1] = (P[P_14][b[1]] & 0xFF) ^ b1(k32[3]);
			b[2] = (P[P_24][b[2]] & 0xFF) ^ b2(k32[3]);
			b[3] = (P[P_34][b[3]] & 0xFF) ^ b3(k32[3]);
			/* fall thru, having pre-processed b[0]..b[3] with k32[3] */
		case 3:     /* 192 bits of key */
			b[0] = (P[P_03][b[0]] & 0xFF) ^ b0(k32[2]);
			b[1] = (P[P_13][b[1]] & 0xFF) ^ b1(k32[2]);
			b[2] = (P[P_23][b[2]] & 0xFF) ^ b2(k32[2]);
			b[3] = (P[P_33][b[3]] & 0xFF) ^ b3(k32[2]);
			/* fall thru, having pre-processed b[0]..b[3] with k32[2] */
		case 2:     /* 128 bits of key */
			b[0] = P[P_00][(P[P_01][(P[P_02][b[0]] ^ b0(k32[1])) & 0xFF] ^ b0(k32[0])) & 0xFF];
			b[1] = P[P_10][(P[P_11][(P[P_12][b[1]] ^ b1(k32[1])) & 0xFF] ^ b1(k32[0])) & 0xFF];
			b[2] = P[P_20][(P[P_21][(P[P_22][b[2]] ^ b2(k32[1])) & 0xFF] ^ b2(k32[0])) & 0xFF];
			b[3] = P[P_30][(P[P_31][(P[P_32][b[3]] ^ b3(k32[1])) & 0xFF] ^ b3(k32[0])) & 0xFF];
		}

		b[0] &= 0xFF;
		b[1] &= 0xFF;
		b[2] &= 0xFF;
		b[3] &= 0xFF;

		// System.out.println(Integer.toHexString(b[0]) + " " + Integer.toHexString(b[1]) + " " + Integer.toHexString(b[2]) + " " + Integer.toHexString(b[3]) + " ");

		/* Now perform the MDS matrix multiply inline. */
		return ((Mx_1(b[0]) ^ Mx_Y(b[1]) ^ Mx_X(b[2]) ^ Mx_X(b[3]))) ^
		((Mx_X(b[0]) ^ Mx_Y(b[1]) ^ Mx_Y(b[2]) ^ Mx_1(b[3])) << 8) ^
		((Mx_Y(b[0]) ^ Mx_X(b[1]) ^ Mx_1(b[2]) ^ Mx_Y(b[3])) <<16) ^
		((Mx_Y(b[0]) ^ Mx_1(b[1]) ^ Mx_Y(b[2]) ^ Mx_X(b[3])) <<24);
	}

	final static int b0(int x)
	{
		return x & 0xFF;
	}

	final static int b1(int x)
	{
		return (x >>> 8) & 0xFF;
	}

	final static int b2(int x)
	{
		return (x >>> 16) & 0xFF;
	}

	final static int b3(int x)
	{
		return (x >>> 24) & 0xFF;
	}

	final static int LFSR1(int x)
	{
		return (x >>> 1) ^ ((x & 0x01) != 0 ? GF256_FDBK_2 : 0);
	}

	final static int LFSR2(int x)
	{
		return (x >>> 2) ^
			((x & 0x02) != 0 ? GF256_FDBK_2 : 0) ^
			((x & 0x01) != 0 ? GF256_FDBK_4 : 0);
	}

	final static int Mx_1(int x)
	{
		return x;
	}

	final static int Mx_X(int x)
	{
		return x ^ LFSR2(x);
	} // 5B

	final static int Mx_Y(int x)
	{
		return x ^ LFSR1(x) ^ LFSR2(x);
	} // EF


	/** Fixed 8x8 permutation S-boxes */
	private final static byte[][] P = new byte[][] {
	{	// p0
		(byte) 0xA9, (byte) 0x67, (byte) 0xB3, (byte) 0xE8,
		(byte) 0x04, (byte) 0xFD, (byte) 0xA3, (byte) 0x76,
		(byte) 0x9A, (byte) 0x92, (byte) 0x80, (byte) 0x78,
		(byte) 0xE4, (byte) 0xDD, (byte) 0xD1, (byte) 0x38,
		(byte) 0x0D, (byte) 0xC6, (byte) 0x35, (byte) 0x98,
		(byte) 0x18, (byte) 0xF7, (byte) 0xEC, (byte) 0x6C,
		(byte) 0x43, (byte) 0x75, (byte) 0x37, (byte) 0x26,
		(byte) 0xFA, (byte) 0x13, (byte) 0x94, (byte) 0x48,
		(byte) 0xF2, (byte) 0xD0, (byte) 0x8B, (byte) 0x30,
		(byte) 0x84, (byte) 0x54, (byte) 0xDF, (byte) 0x23,
		(byte) 0x19, (byte) 0x5B, (byte) 0x3D, (byte) 0x59,
		(byte) 0xF3, (byte) 0xAE, (byte) 0xA2, (byte) 0x82,
		(byte) 0x63, (byte) 0x01, (byte) 0x83, (byte) 0x2E,
		(byte) 0xD9, (byte) 0x51, (byte) 0x9B, (byte) 0x7C,
		(byte) 0xA6, (byte) 0xEB, (byte) 0xA5, (byte) 0xBE,
		(byte) 0x16, (byte) 0x0C, (byte) 0xE3, (byte) 0x61,
		(byte) 0xC0, (byte) 0x8C, (byte) 0x3A, (byte) 0xF5,
		(byte) 0x73, (byte) 0x2C, (byte) 0x25, (byte) 0x0B,
		(byte) 0xBB, (byte) 0x4E, (byte) 0x89, (byte) 0x6B,
		(byte) 0x53, (byte) 0x6A, (byte) 0xB4, (byte) 0xF1,
		(byte) 0xE1, (byte) 0xE6, (byte) 0xBD, (byte) 0x45,
		(byte) 0xE2, (byte) 0xF4, (byte) 0xB6, (byte) 0x66,
		(byte) 0xCC, (byte) 0x95, (byte) 0x03, (byte) 0x56,
		(byte) 0xD4, (byte) 0x1C, (byte) 0x1E, (byte) 0xD7,
		(byte) 0xFB, (byte) 0xC3, (byte) 0x8E, (byte) 0xB5,
		(byte) 0xE9, (byte) 0xCF, (byte) 0xBF, (byte) 0xBA,
		(byte) 0xEA, (byte) 0x77, (byte) 0x39, (byte) 0xAF,
		(byte) 0x33, (byte) 0xC9, (byte) 0x62, (byte) 0x71,
		(byte) 0x81, (byte) 0x79, (byte) 0x09, (byte) 0xAD,
		(byte) 0x24, (byte) 0xCD, (byte) 0xF9, (byte) 0xD8,
		(byte) 0xE5, (byte) 0xC5, (byte) 0xB9, (byte) 0x4D,
		(byte) 0x44, (byte) 0x08, (byte) 0x86, (byte) 0xE7,
		(byte) 0xA1, (byte) 0x1D, (byte) 0xAA, (byte) 0xED,
		(byte) 0x06, (byte) 0x70, (byte) 0xB2, (byte) 0xD2,
		(byte) 0x41, (byte) 0x7B, (byte) 0xA0, (byte) 0x11,
		(byte) 0x31, (byte) 0xC2, (byte) 0x27, (byte) 0x90,
		(byte) 0x20, (byte) 0xF6, (byte) 0x60, (byte) 0xFF,
		(byte) 0x96, (byte) 0x5C, (byte) 0xB1, (byte) 0xAB,
		(byte) 0x9E, (byte) 0x9C, (byte) 0x52, (byte) 0x1B,
		(byte) 0x5F, (byte) 0x93, (byte) 0x0A, (byte) 0xEF,
		(byte) 0x91, (byte) 0x85, (byte) 0x49, (byte) 0xEE,
		(byte) 0x2D, (byte) 0x4F, (byte) 0x8F, (byte) 0x3B,
		(byte) 0x47, (byte) 0x87, (byte) 0x6D, (byte) 0x46,
		(byte) 0xD6, (byte) 0x3E, (byte) 0x69, (byte) 0x64,
		(byte) 0x2A, (byte) 0xCE, (byte) 0xCB, (byte) 0x2F,
		(byte) 0xFC, (byte) 0x97, (byte) 0x05, (byte) 0x7A,
		(byte) 0xAC, (byte) 0x7F, (byte) 0xD5, (byte) 0x1A,
		(byte) 0x4B, (byte) 0x0E, (byte) 0xA7, (byte) 0x5A,
		(byte) 0x28, (byte) 0x14, (byte) 0x3F, (byte) 0x29,
		(byte) 0x88, (byte) 0x3C, (byte) 0x4C, (byte) 0x02,
		(byte) 0xB8, (byte) 0xDA, (byte) 0xB0, (byte) 0x17,
		(byte) 0x55, (byte) 0x1F, (byte) 0x8A, (byte) 0x7D,
		(byte) 0x57, (byte) 0xC7, (byte) 0x8D, (byte) 0x74,
		(byte) 0xB7, (byte) 0xC4, (byte) 0x9F, (byte) 0x72,
		(byte) 0x7E, (byte) 0x15, (byte) 0x22, (byte) 0x12,
		(byte) 0x58, (byte) 0x07, (byte) 0x99, (byte) 0x34,
		(byte) 0x6E, (byte) 0x50, (byte) 0xDE, (byte) 0x68,
		(byte) 0x65, (byte) 0xBC, (byte) 0xDB, (byte) 0xF8,
		(byte) 0xC8, (byte) 0xA8, (byte) 0x2B, (byte) 0x40,
		(byte) 0xDC, (byte) 0xFE, (byte) 0x32, (byte) 0xA4,
		(byte) 0xCA, (byte) 0x10, (byte) 0x21, (byte) 0xF0,
		(byte) 0xD3, (byte) 0x5D, (byte) 0x0F, (byte) 0x00,
		(byte) 0x6F, (byte) 0x9D, (byte) 0x36, (byte) 0x42,
		(byte) 0x4A, (byte) 0x5E, (byte) 0xC1, (byte) 0xE0
	},
	{	// p1
		(byte) 0x75, (byte) 0xF3, (byte) 0xC6, (byte) 0xF4,
		(byte) 0xDB, (byte) 0x7B, (byte) 0xFB, (byte) 0xC8,
		(byte) 0x4A, (byte) 0xD3, (byte) 0xE6, (byte) 0x6B,
		(byte) 0x45, (byte) 0x7D, (byte) 0xE8, (byte) 0x4B,
		(byte) 0xD6, (byte) 0x32, (byte) 0xD8, (byte) 0xFD,
		(byte) 0x37, (byte) 0x71, (byte) 0xF1, (byte) 0xE1,
		(byte) 0x30, (byte) 0x0F, (byte) 0xF8, (byte) 0x1B,
		(byte) 0x87, (byte) 0xFA, (byte) 0x06, (byte) 0x3F,
		(byte) 0x5E, (byte) 0xBA, (byte) 0xAE, (byte) 0x5B,
		(byte) 0x8A, (byte) 0x00, (byte) 0xBC, (byte) 0x9D,
		(byte) 0x6D, (byte) 0xC1, (byte) 0xB1, (byte) 0x0E,
		(byte) 0x80, (byte) 0x5D, (byte) 0xD2, (byte) 0xD5,
		(byte) 0xA0, (byte) 0x84, (byte) 0x07, (byte) 0x14,
		(byte) 0xB5, (byte) 0x90, (byte) 0x2C, (byte) 0xA3,
		(byte) 0xB2, (byte) 0x73, (byte) 0x4C, (byte) 0x54,
		(byte) 0x92, (byte) 0x74, (byte) 0x36, (byte) 0x51,
		(byte) 0x38, (byte) 0xB0, (byte) 0xBD, (byte) 0x5A,
		(byte) 0xFC, (byte) 0x60, (byte) 0x62, (byte) 0x96,
		(byte) 0x6C, (byte) 0x42, (byte) 0xF7, (byte) 0x10,
		(byte) 0x7C, (byte) 0x28, (byte) 0x27, (byte) 0x8C,
		(byte) 0x13, (byte) 0x95, (byte) 0x9C, (byte) 0xC7,
		(byte) 0x24, (byte) 0x46, (byte) 0x3B, (byte) 0x70,
		(byte) 0xCA, (byte) 0xE3, (byte) 0x85, (byte) 0xCB,
		(byte) 0x11, (byte) 0xD0, (byte) 0x93, (byte) 0xB8,
		(byte) 0xA6, (byte) 0x83, (byte) 0x20, (byte) 0xFF,
		(byte) 0x9F, (byte) 0x77, (byte) 0xC3, (byte) 0xCC,
		(byte) 0x03, (byte) 0x6F, (byte) 0x08, (byte) 0xBF,
		(byte) 0x40, (byte) 0xE7, (byte) 0x2B, (byte) 0xE2,
		(byte) 0x79, (byte) 0x0C, (byte) 0xAA, (byte) 0x82,
		(byte) 0x41, (byte) 0x3A, (byte) 0xEA, (byte) 0xB9,
		(byte) 0xE4, (byte) 0x9A, (byte) 0xA4, (byte) 0x97,
		(byte) 0x7E, (byte) 0xDA, (byte) 0x7A, (byte) 0x17,
		(byte) 0x66, (byte) 0x94, (byte) 0xA1, (byte) 0x1D,
		(byte) 0x3D, (byte) 0xF0, (byte) 0xDE, (byte) 0xB3,
		(byte) 0x0B, (byte) 0x72, (byte) 0xA7, (byte) 0x1C,
		(byte) 0xEF, (byte) 0xD1, (byte) 0x53, (byte) 0x3E,
		(byte) 0x8F, (byte) 0x33, (byte) 0x26, (byte) 0x5F,
		(byte) 0xEC, (byte) 0x76, (byte) 0x2A, (byte) 0x49,
		(byte) 0x81, (byte) 0x88, (byte) 0xEE, (byte) 0x21,
		(byte) 0xC4, (byte) 0x1A, (byte) 0xEB, (byte) 0xD9,
		(byte) 0xC5, (byte) 0x39, (byte) 0x99, (byte) 0xCD,
		(byte) 0xAD, (byte) 0x31, (byte) 0x8B, (byte) 0x01,
		(byte) 0x18, (byte) 0x23, (byte) 0xDD, (byte) 0x1F,
		(byte) 0x4E, (byte) 0x2D, (byte) 0xF9, (byte) 0x48,
		(byte) 0x4F, (byte) 0xF2, (byte) 0x65, (byte) 0x8E,
		(byte) 0x78, (byte) 0x5C, (byte) 0x58, (byte) 0x19,
		(byte) 0x8D, (byte) 0xE5, (byte) 0x98, (byte) 0x57,
		(byte) 0x67, (byte) 0x7F, (byte) 0x05, (byte) 0x64,
		(byte) 0xAF, (byte) 0x63, (byte) 0xB6, (byte) 0xFE,
		(byte) 0xF5, (byte) 0xB7, (byte) 0x3C, (byte) 0xA5,
		(byte) 0xCE, (byte) 0xE9, (byte) 0x68, (byte) 0x44,
		(byte) 0xE0, (byte) 0x4D, (byte) 0x43, (byte) 0x69,
		(byte) 0x29, (byte) 0x2E, (byte) 0xAC, (byte) 0x15,
		(byte) 0x59, (byte) 0xA8, (byte) 0x0A, (byte) 0x9E,
		(byte) 0x6E, (byte) 0x47, (byte) 0xDF, (byte) 0x34,
		(byte) 0x35, (byte) 0x6A, (byte) 0xCF, (byte) 0xDC,
		(byte) 0x22, (byte) 0xC9, (byte) 0xC0, (byte) 0x9B,
		(byte) 0x89, (byte) 0xD4, (byte) 0xED, (byte) 0xAB,
		(byte) 0x12, (byte) 0xA2, (byte) 0x0D, (byte) 0x52,
		(byte) 0xBB, (byte) 0x02, (byte) 0x2F, (byte) 0xA9,
		(byte) 0xD7, (byte) 0x61, (byte) 0x1E, (byte) 0xB4,
		(byte) 0x50, (byte) 0x04, (byte) 0xF6, (byte) 0xC2,
		(byte) 0x16, (byte) 0x25, (byte) 0x86, (byte) 0x56,
		(byte) 0x55, (byte) 0x09, (byte) 0xBE, (byte) 0x91
	}
	};
}

