/*
 * $Id: Cipher.java,v 1.21 1999/02/11 03:56:07 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/Cipher.java,v $
 * $Revision: 1.21 $
 * $Date: 1999/02/11 03:56:07 $
 * $State: Exp $
 */

package javax.crypto;

import java.security.*;
import java.util.*;

import java.security.*;
import java.security.spec.*;

/**
 * This class provides the functionality of a cryptographic cipher for
 * encryption and decryption. It forms the core of the Java Cryptographic
 * Extension (JCE) framework. 
 * <p>
 * In order to create a Cipher object, the application calls the Cipher's
 * getInstance method, and passes the name of the requested transformation
 * to it. Optionally, the name of a provider may be specified. 
 * <p>
 * A transformation is a string that describes the operation (or set of
 * operations) to be performed on the given input, to produce some output.
 * A transformation always includes the name of a cryptographic algorithm
 * (e.g., DES), and may be followed by a feedback mode and padding scheme.
 * <p>
 * A * transformation is of the form:
 * <ul>
 * <li>"algorithm/mode/padding" or
 * <li>"algorithm"
 * </ul>
 * (in the latter case provider-specific defaults values for mode and padding scheme are used).  For example, the following is a valid transformation:
 * <pre>
 * Cipher c = Cipher.getInstance("DES/CBC/PKCS5Padding");
 * </pre>
 * <p>
 * When requesting a block cipher in stream cipher mode (e.g., DES in CFB
 * or OFB mode), the user may optionally specify the number of bits to be
 * processed at a time, by appending this number to the mode name as shown
 * in the "DES/CFB8/NoPadding" and "DES/OFB32/PKCS5Padding" transformations.
 * If no such number is specified, a provider-specific default is used. (For
 * example, the SunJCE provider uses a default of 64 bits.) 
 *
 * @see KeyGenerator
 * @see SecretKey
 */
public class Cipher
{
	public final static String ident = "$Id: Cipher.java,v 1.21 1999/02/11 03:56:07 leachbj Exp $";

	public static final int ENCRYPT_MODE = 1;
	public static final int DECRYPT_MODE = 2;

	private boolean		initialised;

	private CipherSpi	cipherSpi;
	private Provider	provider;
	private String		transformation;

	/**
	 * Creates a Cipher object. 
	 * 
	 * @param cipherSpi the delegate 
	 * @param provider the provider 
	 * @param transformation the transformation 
	 */
	protected Cipher(
		CipherSpi	cipherSpi,
		Provider	provider,
		String		transformation)
	{
		this.initialised = false;
		this.cipherSpi = cipherSpi;
		this.provider = provider;
		this.transformation = transformation;
	}

	/**
	 * create an instance of a Cipher object given the provider.
	 *
	 * @param prov the provider object.
	 * @param transformation the transformation we want.
	 */
	private static final Cipher getInstance(
		String		transformation,
		Provider	prov)
		throws NoSuchAlgorithmException, NoSuchPaddingException
	{
		Cipher		cipher;
		CipherSpi	spi;

		try
		{
			spi = (CipherSpi)Class.forName(
					prov.getProperty(
						"Cipher."
					    + transformation)).newInstance();
		}
		catch (Exception e)
		{
			StringTokenizer	st;
			String		algorithm;

			st = new StringTokenizer(transformation, "/");

			algorithm = st.nextToken();

			try
			{
				spi = (CipherSpi)Class.forName(
					prov.getProperty(
						"Cipher."
					    + algorithm)).newInstance();
			}
			catch (Exception ex)
			{
				throw new NoSuchAlgorithmException(
					algorithm + ": " + ex);
			}

			if (st.countTokens() != 0)
			{
				try
				{
					spi.engineSetMode(st.nextToken());
					spi.engineSetPadding(st.nextToken());
				}
				catch (Exception ex)
				{
					throw new NoSuchAlgorithmException(
							transformation);
				}
			}
		}

		cipher = new Cipher(spi, prov, transformation);

		return cipher;
	}

	/**
	 * Generates a Cipher object that implements the specified
	 * transformation.
	 * <p>
	 * If the default provider package provides an implementation of the
	 * requested transformation, an instance of Cipher containing that
	 * implementation is returned. If the transformation is not available
	 * in the default provider package, other provider packages are
	 * searched.
	 *
	 * @param transformation the name of the transformation, e.g.,
	 * 	DES/CBC/PKCS5Padding.  See Appendix A in the Java Cryptography
	 * 	Extension API Specification & Reference for information about
	 * 	standard transformation names.
	 * @return a cipher that implements the requested transformation
	 * @exception NoSuchAlgorithmException if the requested algorithm is
	 *    not available in the default provider package or any of the
	 *    other provider packages that were searched.
	 * @exception NoSuchPaddingException if transformation contains a
	 * 	scheme that is not available.
	 */
	public static final Cipher getInstance(
		String transformation)
		throws NoSuchAlgorithmException, NoSuchPaddingException
	{
		Provider[] prov = Security.getProviders();

		for (int i = 0; i != prov.length; i++)
		{
			try
			{
				return getInstance(transformation, prov[i]);
			}
			catch (Exception e)
			{
				// ignore
			}
		}

		throw new NoSuchAlgorithmException(
			"No Cipher could be found for algorithm "
				+ transformation);
	}

	/**
	 * Creates a Cipher object that implements the specified transformation,
	 * as supplied by the specified provider.
	 *
	 * @param transformation the name of the transformation, e.g.,
	 * 	DES/CBC/PKCS5Padding.  See Appendix A in the Java Cryptography
	 * 	Extension API Specification & Reference for information about
	 * 	standard transformation names.
	 * @param provider the name of the cipher provider 
	 * @return a cipher that implements the requested transformation
	 * @exception NoSuchAlgorithmException if the requested algorithm is
	 *    not available
	 * @exception NoSuchProviderException if the requested provider is
	 *    not available
	 * @exception NoSuchPaddingException if the requested padding mechanism
	 *    is not available
	 * @see Provider
	 */
	public static final Cipher getInstance(
		String transformation,
		String provider)
		throws NoSuchAlgorithmException, NoSuchProviderException,
			NoSuchPaddingException
	{
		Provider	prov;

		prov = Security.getProvider(provider);
		if (prov == null)
		{
			throw new NoSuchProviderException(
					"Provider " + provider + " not found");
		}

		return getInstance(transformation, prov);
	}

	/**
	 * Returns the provider of this Cipher object.
	 *
	 * @return Returns the provider of this Cipher object.
	 */
	public final Provider getProvider()
	{
		return provider;
	}

	/**
	 * Returns the algorithm name of this Cipher object.
	 * <p>
	 * This is the same name that was specified in one of the getInstance
	 * calls that created this Cipher object.
	 *
	 * @return the algorithm name of this Cipher object.
	 */
	public final String getAlgorithm()
	{
		return transformation;
	}

	/**
	 * Returns the block size (in bytes). 
	 *
	 * @return the block size (in bytes), or 0 if the underlying algorithm
	 * 		is not a block cipher 
	 */
	public final int getBlockSize()
	{
		return cipherSpi.engineGetBlockSize();
	}

	/**
	 * Returns the length in bytes that an output buffer would need to be
	 * in order to hold the result of the next update or doFinal operation,
	 * given the input length inputLen (in bytes).
	 * <p>
	 * This call takes into account any unprocessed (buffered) data from a
	 * previous update call, and padding.
	 * <p>
	 * The actual output length of the next update or doFinal call may be
	 * smaller than the length returned by this method.
	 *
	 * @param inputLen	the input length (in bytes)
	 * @return the required output buffer size (in bytes)
	 * @exception IllegalStateException if this cipher is
	 *		in a wrong state (e.g., has not yet been initialised)
	 */
	public final int getOutputSize(
		int	inputLen)
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		return cipherSpi.engineGetOutputSize(inputLen);
	}

	/**
	 * Returns the initialisation vector (IV) in a new buffer. 
	 * <p>
	 * This is useful in the case where a random IV was created, or in
	 * the context of password-based encryption or decryption, where the
	 * IV is derived from a user-supplied password.
	 *
	 * @return the initialisation vector in a new buffer, or null if the
	 * 	underlying algorithm does not use an IV, or if the IV has
	 * 	not yet been set. 
	 */
	public final byte[] getIV()
	{
		return cipherSpi.engineGetIV();
	}

	/**
	 * Returns the parameters used with this cipher. 
	 * <p>
	 * The returned parameters may be the same that were used to initialise
	 * this cipher, or may contain a combination of default and random
	 * parameter values used by the underlying cipher implementation if
	 * this cipher requires algorithm parameters but was not initialised
	 * with any.
	 *
	 * @return the parameters used with this cipher, or null if this cipher
	 *     does not use any parameters.
	 */
	public final AlgorithmParameters getParameters()
	{
		return cipherSpi.engineGetParameters();
	}

	/**
	 * Initialises this cipher with a key. 
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending
	 * on the value of opmode. 
	 * <p>
	 * If this cipher requires any algorithm parameters that cannot be
	 * derived from the give key, the underlying cipher implementation is
	 * supposed to generate the required parameters itself (using
	 * provider0specific default or random values) if it is being
	 * initialised for encryption, and raise and InvalidKeyException if
	 * it is being initialised for decryption.  The generated parameters
	 * can be retrieved using getParameters or getIV (if the parameter is
	 * an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them using the SecureRandom
	 * implementation of the highest-priority installed provider as the
	 * source of randomness.  (If none of the installed providers supply
	 * an implementation of SecureRandom, a system provided source of
	 * randomness will be used.)
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously acquired state.  In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher and
	 * initialising it.
	 *
	 * @param opmode the operation mode of this cipher (this is either
	 *    ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the key
	 * @exception InvalidKeyException if the given key is inappropriate
	 *    for initialising this cipher, or if this cipher is being
	 *    initialised for decryption and requires algorithm parameters
	 *    that cannot be determined from the given key.
	 */
	public final void init(
		int	opmode,
		Key	key)
		throws InvalidKeyException
	{
		cipherSpi.engineInit(opmode, key, new SecureRandom());
		initialised = true;
	}

	/**
	 * Initialises this cipher with a key and a source of randomness. 
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending
	 * on the value of opmode. 
	 * <p>
	 * If this cipher requires any algorithm parameters that cannot be
	 * derived from the give key, the underlying cipher implementation is
	 * supposed to generate the required parameters itself (using
	 * provider0specific default or random values) if it is being
	 * initialised for encryption, and raise and InvalidKeyException if
	 * it is being initialised for decryption.  The generated parameters
	 * can be retrieved using getParameters or getIV (if the parameter is
	 * an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them from random.
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously acquired state.  In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher and
	 * initialising it.
	 * 
	 * @param opmode the operation mode of this cipher (this is either
	 *    ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the encryption key 
	 * @param random the source of randomness 
	 * @exception InvalidKeyException if the given key is inappropriate
	 * 	for initialising this cipher, or if the cipher is being
	 * 	initialised for decryption and requires algorithm parameters
	 * 	that cannot be determined from the given key.
	 */
	public final void init(
		int		opmode,
		Key		key,
		SecureRandom	random)
		throws InvalidKeyException
	{
		cipherSpi.engineInit(opmode, key, random);
		initialised = true;
	}

	/**
	 * Initialises this cipher with a key and a set of algorithm parameters. 
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending on
	 * the value of opmode. 
	 * <p>
	 * If this cipher requires any algorithm parameters and params is
	 * null, the underlying cipher implementation is supposed to generate
	 * the required parameters by itself (using provider-specific default
	 * or random values) if it is being initialised for encryption, and
	 * raise an InvalidAlgorithmParameterException if it being
	 * initialised for decryption.  The generated parameters can be
	 * retrieved using getParameters or getIV (if the parameter is an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them using the SecureRandom
	 * implementation of the highest-priority installed provider as the
	 * source of randomness.  (If none of the installed providers supply
	 * an implementation of SecureRandom, a system provided source of
	 * randomness will be used.)
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously acquired state.  In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher and
	 * initialising it.
	 *
	 * @param opmode the operation mode of this cipher (this is either
	 *     ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the encryption key 
	 * @param params the algorithm parameters 
	 * @exception InvalidKeyException if the given key is inappropriate
	 *     for initialising this cipher.
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 *     parameters are inappropriate for this cipher, or if this
	 *     cipher is being initialised for decryption and requires
	 *     algorithm parameters and params is null.
	 */
	public final void init(
		int			opmode,
		Key 			key,
		AlgorithmParameterSpec	params)
		throws InvalidKeyException, InvalidAlgorithmParameterException
	{
		cipherSpi.engineInit(opmode, key, params, new SecureRandom());
		initialised = true;
	}

	/**
	 * Initialises this cipher with a key, a set of algorithm parameters,
	 * and a source of randomness. 
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending
	 * on the value of opmode. 
	 * <p>
	 * If this cipher requires any algorithm parameters and params is
	 * null, the underlying cipher implementation is supposed to generate
	 * the required parameters by itself (using provider-specific default
	 * or random values) if it is being initialised for encryption, and
	 * raise an InvalidAlgorithmParameterException if it being
	 * initialised for decryption.  The generated parameters can be
	 * retrieved using getParameters or getIV (if the parameter is an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them from random.
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher, and
	 * initialising it.
	 *
	 * @param opmode the operation mode of this cipher (this is either
	 *    ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the encryption key 
	 * @param params the algorithm parameters 
	 * @param random the source of randomness 
	 * @exception InvalidKeyException if the given key is inappropriate
	 *    for initialising this cipher
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 *     parameters are inappropriate for this cipher, or if this
	 *     cipher is being initialised for decryption and requires
	 *     algorithm parameters and params is null.
	 */
	public final void init(
		int			opmode,
		Key 			key,
		AlgorithmParameterSpec	params,
		SecureRandom		random)
		throws InvalidKeyException, InvalidAlgorithmParameterException
	{
		cipherSpi.engineInit(opmode, key, params, random);
		initialised = true;
	}

	/**
	 * Initialises this cipher with a key and a set of algorithm parameters. 
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending on
	 * the value of opmode. 
	 * <p>
	 * If this cipher requires any algorithm parameters and params is
	 * null, the underlying cipher implementation is supposed to generate
	 * the required parameters by itself (using provider-specific default
	 * or random values) if it is being initialised for encryption, and
	 * raise an InvalidAlgorithmParameterException if it being
	 * initialised for decryption.  The generated parameters can be
	 * retrieved using getParameters or getIV (if the parameter is an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them using the SecureRandom
	 * implementation of the highest-priority installed provider as the
	 * source of randomness.  (If none of the installed providers supply
	 * an implementation of SecureRandom, a system provided source of
	 * randomness will be used.)
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher is
	 * equivalent to creating a new instance of that Cipher and initialising
	 * it.
	 *
	 * @param opmode the operation mode of this cipher (this is either
	 *     ENCRYPT_MODE or DECRYPT_MODE)
	 * @param key the encryption key
	 * @param params the algorithm parameters
	 * @exception InvalidKeyException - if the given key is
	 *    inappropriate for initialising this cipher
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 *     parameters are inappropriate for this cipher, or if this
	 *     cipher is being initialised for decryption and requires
	 *     algorithm parameters and params is null.
	 */
	public final void init(
		int			opmode,
		Key			key,
		AlgorithmParameters	params)
		throws InvalidKeyException, InvalidAlgorithmParameterException
	{
		cipherSpi.engineInit(opmode, key, params, new SecureRandom());
		initialised = true;
	}

	/**
	 * Initialises this cipher with a key, a set of algorithm parameters,
	 * and a source of randomness. 
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending on
	 * the value of opmode. 
	 * <p>
	 * If this cipher requires any algorithm parameters and params is
	 * null, the underlying cipher implementation is supposed to generate
	 * the required parameters by itself (using provider-specific default
	 * or random values) if it is being initialised for encryption, and
	 * raise an InvalidAlgorithmParameterException if it being
	 * initialised for decryption.  The generated parameters can be
	 * retrieved using getParameters or getIV (if the parameter is an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding scheme)
	 * requires any random bytes, it will get them from random. 
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher is
	 * equivalent to creating a new instance of that Cipher and initialising
	 * it.
	 *
	 * @param opmode the operation mode of this cipher (this is either
	 *      ENCRYPT_MODE or DECRYPT_MODE)
	 * @param key the encryption key
	 * @param params the algorithm parameters
	 * @param random the source of randomness
	 * @exception InvalidKeyException if the given key is
	 *    inappropriate for initialising this cipher
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 *     parameters are inappropriate for this cipher, or if this
	 *     cipher is being initialised for decryption and requires
	 *     algorithm parameters and params is null.
	 */
	public final void init(
		int			opmode,
		Key			key,
		AlgorithmParameters	params,
		SecureRandom		random)
		throws InvalidKeyException, InvalidAlgorithmParameterException
	{
		cipherSpi.engineInit(opmode, key, params, random);
		initialised = true;
	}

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The bytes in the input buffer are processed, and the result is
	 * stored in a new buffer.
	 *
	 * @param input the input buffer 
	 * @return the new buffer with the result, or null if the underlying
	 *    cipher is a block cipher and the input data is too short to
	 *    result in a new block.
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *    (e.g., has not been initialised)
	 */
	public final byte[] update(
		byte[]	input)
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input == null)
		{
			throw new IllegalArgumentException("null input buffer");
		}

		if (input.length == 0)
		{
			return null;
		}

		return cipherSpi.engineUpdate(input, 0, input.length);
	}

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset inclusive, are processed, and the result is stored in
	 * a new buffer.
	 *
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @return the new buffer with the result, or null if the underlying
	 *    cipher is a block cipher and the input data is too short to
	 *    result in a new block.
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *     (e.g., has not been initialised)
	 */
	public final byte[] update(
		byte[]	input,
		int	inputOffset,
		int	inputLen)
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input == null || inputOffset < 0
			|| inputLen > (input.length - inputOffset)
			|| inputLen < 0)
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		if (inputLen == 0)
		{
			return null;
		}

		return cipherSpi.engineUpdate(input, inputOffset, inputLen);
	}

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset inclusive, are processed, and the result is stored
	 * in the output buffer.
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown. In this case, repeat this call with
	 * a larger output buffer. Use getOutputSize to determine how big the
	 * output buffer should be.
	 * 
	 * @param input - the input buffer 
	 * @param inputOffset - the offset in input where the input starts 
	 * @param inputLen - the input length 
	 * @param output - the buffer for the result 
	 * @return the number of bytes stored in output
	 * @exception IllegalStateException if this cipher is in a wrong
	 *     state (e.g., has not been initialised)
	 * @exception ShortBufferException if the given output buffer is
	 *     too small to hold the result.
	 */
	public final int update(
		byte[]	input,
		int	inputOffset,
		int	inputLen,
		byte[]	output)
		throws IllegalStateException, ShortBufferException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input == null || output == null || inputOffset < 0
			|| inputLen > (input.length - inputOffset)
			|| inputLen < 0)
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		if (inputLen == 0)
		{
			return 0;
		}

		return cipherSpi.engineUpdate(input, inputOffset, inputLen, output, 0);
	}

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, are processed, and the result is stored in the output
	 * buffer, starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown. In this case, repeat this call with
	 * a larger output buffer. Use getOutputSize to determine how big the
	 * output buffer should be.
	 * 
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * @return the number of bytes stored in output 
	 * @exception IllegalStateException if this cipher is in a wrong
	 *				state (e.g., has not been initialised) 
	 * @exception ShortBufferException if the given output buffer is
	 *				too small to hold the result 
	 */
	public final int update(
		byte[]	input,
		int	inputOffset,
		int	inputLen,
		byte[]	output,
		int	outputOffset)
		throws IllegalStateException, ShortBufferException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input == null || output == null
			|| inputOffset < 0
			|| outputOffset < 0
			|| inputLen > (input.length - inputOffset)
			|| inputLen < 0)
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		if (inputLen == 0)
		{
			return 0;
		}

		return cipherSpi.engineUpdate(input, inputOffset, inputLen,
			output, outputOffset);
	}

	/**
	 * Finishes a multiple-part encryption or decryption operation,
	 * depending on how this cipher was initialised.
	 * <p>
	 * Input data that may have been buffered during a previous update
	 * operation is processed, with padding (if requested) being applied.
	 * The result is stored in a new buffer.
	 * <p>
	 * A call to this method resets this cipher object to the state it
	 * was in when previously initialised via a call to init.  That is,
	 * the object is reset and available to encrypt or decrypt (depending
	 * on the operation mode that was specified in the call to init) more
	 * data.
	 *
	 * @returns the new buffer with the result
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *		(e.g., has not been initialised)
	 * @exception IllegalBlockSizeException if this cipher is a block
	 * 	cipher, no padding has been requested (only in encryption
	 *		mode), and the total input length of the data processed by
	 *		this cipher is not a multiple of block size 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *		and (un)padding has been requested, but the decrypted data is
	 *		not bounded by the appropriate padding bytes.
	 */
	public final byte[] doFinal()
		throws IllegalStateException, IllegalBlockSizeException,
			BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		return cipherSpi.engineDoFinal(null, 0, 0);
	}

	/**
	 * Finishes a multiple-part encryption or decryption operation,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * Input data that may have been buffered during a previous update
	 * operation is processed, with padding (if requested) being applied.
	 * The result is stored in the output buffer, starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown. In this case, repeat this call with
	 * a larger output buffer. Use getOutputSize to determine how big the
	 * output buffer should be. 
	 * <p>
	 * A call to this method resets this cipher object to the state it
	 * was in when previously initialised via a call to init.  That is,
	 * the object is reset and available to encrypt or decrypt (depending
	 * on the operation mode that was specified in the call to init) more
	 * data.
	 *
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * @return the number of bytes stored in output 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *     (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *     cipher, no padding has been requested (only in encryption mode),
	 *     and the total input length of the data processed by this cipher
	 *     is not a multiple of block size 
	 * @exception ShortBufferException if the given output buffer is too
	 *     small to hold the result 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *     and (un)padding has been requested, but the decrypted data is
	 *     not bounded by the appropriate padding bytes 
	 */
	public final int doFinal(
		byte output[],
		int outputOffset)
		throws IllegalStateException, IllegalBlockSizeException,
				ShortBufferException, BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (outputOffset < 0)
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		return cipherSpi.engineDoFinal(null, 0, 0, output,
			outputOffset);
	}

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes a
	 * multiple-part operation. The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The bytes in the input buffer, and any input bytes that may have 
	 * been buffered during a previous update operation, are processed,
	 * with padding (if requested) being applied. The result is stored in
	 * a new buffer. 
	 * <p>
	 * A call to this method resets this cipher object to the state it
	 * was in when previously initialised via a call to init.  That is,
	 * the object is reset and available to encrypt or decrypt (depending
	 * on the operation mode that was specified in the call to init) more
	 * data.
	 *
	 * @param input - the input buffer 
	 * @returns the new buffer with the result 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *      (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *      cipher, no padding has been requested (only in encryption mode),
	 *      and the total input length of the data processed by this cipher
	 *      is not a multiple of block size 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *      and (un)padding has been requested, but the decrypted data is
	 *      not bounded by the appropriate padding bytes 
	 */
	public final byte[] doFinal(
		byte input[])
		throws IllegalStateException, IllegalBlockSizeException,
							BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input != null)
		{
			return cipherSpi.engineDoFinal(input, 0,
				input.length);
		}
		else
		{
			return cipherSpi.engineDoFinal(null, 0, 0);
		}
	}

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes
	 * a multiple-part operation.  The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in a new buffer. 
	 * <p>
	 * A call to this method resets this cipher object to the state it
	 * was in when previously initialised via a call to init.  That is,
	 * the object is reset and available to encrypt or decrypt (depending
	 * on the operation mode that was specified in the call to init) more
	 * data.
	 *
	 * @param input - the input buffer 
	 * @param inputOffset - the offset in input where the input starts 
	 * @param inputLen - the input length 
	 * @return the new buffer with the result 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *      (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *      cipher, no padding has been requested (only in encryption mode),
	 *      and the total input length of the data processed by this cipher
	 *      is not a multiple of block size 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *      and (un)padding has been requested, but the decrypted data is
	 *      not bounded by the appropriate padding bytes 
	 */
	public final byte[] doFinal(
		byte	input[],
		int	inputOffset,
		int	inputLen)
		throws IllegalStateException, IllegalBlockSizeException,
							BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (inputOffset < 0
			|| (input != null && inputLen > (input.length - inputOffset)))
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		return cipherSpi.engineDoFinal(input, inputOffset, inputLen);
	}

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes a
	 * multiple-part operation. The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in the output buffer. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown.  In this case, repeat this call
	 * with a larger output buffer. Use getOutputSize to determine how big
	 * the output buffer should be. 
	 * <p>
	 * A call to this method resets this cipher object to the state it
	 * was in when previously initialised via a call to init.  That is,
	 * the object is reset and available to encrypt or decrypt (depending
	 * on the operation mode that was specified in the call to init) more
	 * data.
	 * 
	 * @param input - the input buffer 
	 * @param inputOffset - the offset in input where the input starts 
	 * @param inputLen - the input length 
	 * @param output - the buffer for the result 
	 * @return the number of bytes stored in output 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *     (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *     cipher, no padding has been requested (only in encryption mode),
	 *     and the total input length of the data processed by this cipher
	 *     is not a multiple of block size 
	 * @exception ShortBufferException if the given output buffer is too
	 *     small to hold the result 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *     and (un)padding has been requested, but the decrypted data is
	 *     not bounded by the appropriate padding bytes 
	 */
	public final int doFinal(
		byte	input[],
		int	inputOffset,
		int	inputLen,
		byte	output[])
		throws IllegalStateException, ShortBufferException,
				IllegalBlockSizeException, BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (output == null
			|| inputOffset < 0
			|| (input != null && inputLen > (input.length - inputOffset)))
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		return cipherSpi.engineDoFinal(input, inputOffset, inputLen,
			output, 0);
	}

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes a
	 * multiple-part operation. The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in the output buffer,
	 * starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown.  In this case, repeat this call
	 * with a larger output buffer. Use getOutputSize to determine how big
	 * the output buffer should be. 
	 * <p>
	 * A call to this method resets this cipher object to the state it
	 * was in when previously initialised via a call to init.  That is,
	 * the object is reset and available to encrypt or decrypt (depending
	 * on the operation mode that was specified in the call to init) more
	 * data.
	 *
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * @return the number of bytes stored in output 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *    (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *     cipher, no padding has been requested (only in encryption mode),
	 *     and the total input length of the data processed by this cipher
	 *     is not a multiple of block size 
	 * @exception ShortBufferException if the given output buffer is too
	 *     small to hold the result 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *     and (un)padding has been requested, but the decrypted data is
	 *     not bounded by the appropriate padding bytes 
	 */
	public final int doFinal(
		byte	input[],
		int	inputOffset,
		int	inputLen,
		byte	output[],
		int	outputOffset)
		throws IllegalStateException, ShortBufferException,
				IllegalBlockSizeException, BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (output == null
			|| inputOffset < 0
			|| outputOffset < 0
			|| (input != null && inputLen > (input.length - inputOffset)))
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		return cipherSpi.engineDoFinal(input, inputOffset, inputLen,
			output, outputOffset);
	}
}
