/*
 * $Id: Cipher.java,v 1.9 1999/01/13 23:36:33 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/javax/crypto/Cipher.java,v $
 * $Revision: 1.9 $
 * $Date: 1999/01/13 23:36:33 $
 * $State: Exp $
 */

package javax.crypto;

import java.util.*;

import au.net.aba.lang.*;
import au.net.aba.security.*;
import au.net.aba.security.spec.*;

/**
 * A parent class for encryption/decryption objects.
 */
public class Cipher
{
	public final static String ident = "$Id: Cipher.java,v 1.9 1999/01/13 23:36:33 leachbj Exp $";

	public static final int ENCRYPT_MODE = 1;
	public static final int DECRYPT_MODE = 2;

	private boolean	initialised;

	private CipherSpi	cipherSpi;
	// private Provider	provider;	// only the ABA provider is supported
	private String		transformation;

	/**
	 * Creates a Cipher object. 
	 * 
	 * @param cipherSpi the delegate 
	 * @param provider the provider 
	 * @param transformation the transformation 
	 */
	protected Cipher(
		CipherSpi	cipherSpi,
		Provider	provider,
		String		transformation)
	{
		this.initialised = false;
		this.cipherSpi = cipherSpi;
		// this.provider = provider;
		this.transformation = transformation;
	}

	/**
	 * Generates a Cipher object implementing the specified algorithm from
	 * the specified provider.
	 *
	 * For block ciphers, this method returns a block cipher in ECB mode,
	 * with no padding. A more specialised getInstance method should be
	 * called if a block cipher implementing a different mode and padding
	 * scheme is desired. 
	 * <p>
	 * Only the ABA provider is supported.
	 *
	 * @exception NoSuchAlgorithmException - if the algorithm is not
	 * 	available in the environment.
	 * @exception NoSuchProviderException - if the provider is not
	 * 	available in the environment.
	 */
	public static Cipher getInstance(
		String	transformation,
		String	provider)
	throws NoSuchAlgorithmException, NoSuchProviderException
	{
		CipherSpi	spi = null;
		Provider	prov = null;	// Security.getProvider(provider);

		try
		{
			spi = (CipherSpi)Class.forName(
					"au.net.aba.crypto.provider."
					    + transformation).newInstance();
		}
		catch (Exception e)
		{
			StringTokenizer	st;
			String		algorithm;

			st = new StringTokenizer(transformation, "/");

			algorithm = st.nextToken();

			try
			{
				spi = (CipherSpi)Class.forName(
					"au.net.aba.crypto.provider."
					    + algorithm).newInstance();
			}
			catch (Exception ex)
			{
				throw new NoSuchAlgorithmException(algorithm);
			}

			if (st.countTokens() != 0)
			{
				try
				{
					spi.engineSetMode(st.nextToken());
					spi.engineSetPadding(st.nextToken());
				}
				catch (Exception ex)
				{
					throw new NoSuchAlgorithmException(
							transformation);
				}
			}
		}

		return new Cipher(spi, prov, transformation);
	}

	/**
	 * Returns the provider of this Cipher object.
	 *
	 * @return Returns the provider of this Cipher object.
	 */
/*
	public final Provider getProvider()
	{
		return provider;
	}
*/

	/**
	 * Returns the block size (in bytes). 
	 *
	 * @return the block size (in bytes), or 0 if the underlying algorithm
	 * 		is not a block cipher 
	 */
	public int getBlockSize()
	{
		return cipherSpi.engineGetBlockSize();
	}

	/**
	 * Returns the length in bytes that an output buffer would need to be
	 * in order to hold the result of the next update or doFinal operation,
	 * given the input length inputLen (in bytes).
	 * <p>
	 * This call takes into account any unprocessed (buffered) data from a
	 * previous update call, and padding. The actual output length of the
	 * next update or doFinal call may be smaller than the length returned
	 * by this method. 
	 *
	 * @param inputLen	the input length (in bytes)
	 * @return the required output buffer size (in bytes)
	 * @exception IllegalStateException if this cipher is
	 *		in a wrong state (e.g., has not yet been initialised)
	 */
	public int getOutputSize(
		int	inputLen)
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		return cipherSpi.engineGetOutputSize(inputLen);
	}

	/**
	 * Returns the initialisation vector (IV) in a new buffer. 
	 * <p>
	 * This is useful in the context of password-based encryption or
	 * decryption, where the IV is derived from a user-provided passphrase. 
	 *
	 * @return the initialisation vector in a new buffer, or null if the
	 * 	underlying algorithm does not use an IV, or if the IV has
	 *	not yet been set. 
	 */
	public byte[] getIV()
	{
		return cipherSpi.engineGetIV();
	}

	/**
	 * Initialises this cipher for mode opmode, using the specified key.
	 * This method may be called on a cipher in any state. Any state
	 * information (key, feedback buffer,...) is lost and reset.
	 */
	public final void init(
		int	opmode,
		Key	key)
		throws IllegalStateException, KeyException
	{
		cipherSpi.engineInit(opmode, key, new SecureRandom());
		initialised = true;
	}

	/**
	 * Initialises this cipher for mode opmode, using the specified key.
	 * This method may be called on a cipher in any state. Any state
	 * information (key, feedback buffer,...) is lost and reset.
	 */
	public final void init(
		int		opmode,
		Key		key,
		SecureRandom	random)
		throws IllegalStateException, KeyException
	{
		cipherSpi.engineInit(opmode, key, random);
		initialised = true;
	}

	public final void init(
		int			opmode,
		Key 			key,
		AlgorithmParameterSpec	params)
		throws InvalidKeyException, InvalidAlgorithmParameterException
	{
		cipherSpi.engineInit(opmode, key, params, new SecureRandom());
		initialised = true;
	}

	public final void init(
		int			opmode,
		Key 			key,
		AlgorithmParameterSpec	params,
		SecureRandom		random)
		throws InvalidKeyException, InvalidAlgorithmParameterException
	{
		cipherSpi.engineInit(opmode, key, params, random);
		initialised = true;
	}

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The bytes in the input buffer are processed, and the result is
	 * stored in a new buffer. 
	 *
	 * @param input the input buffer
	 * @return the new buffer with the result
	 * @exception IllegalStateException	if the cipher is in a wrong
	 *					state (e.g. not initialised).
	 */
	public final byte[] update(
		byte[]	input)
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input == null)
		{
			throw new IllegalArgumentException("null input buffer");
		}

		if (input.length == 0)
		{
			return null;
		}

		return cipherSpi.engineUpdate(input, 0, input.length);
	}

	/**
	 * Encrypts or decrypts the specified subarray of data. Whether the
	 * data is encrypted or decrypted depends on the cipher's
	 * initialisation state. This method will automatically allocate an
	 * output buffer of the right size. 
	 */
	public final byte[] update(
		byte[]	input,
		int	inputOffset,
		int	inputLen)
	throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input == null || inputOffset < 0
			|| inputLen > (input.length - inputOffset)
			|| inputLen < 0)
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		if (inputLen == 0)
		{
			return null;
		}

		return cipherSpi.engineUpdate(input, inputOffset, inputLen);
	}

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, are processed, and the result is stored in the output
	 * buffer, starting at outputOffset. 
	 * 
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * @return the number of bytes stored in output 
	 * @exception IllegalStateException if this cipher is in a wrong
	 *				state (e.g., has not been initialised) 
	 * @exception ShortBufferException if the given output buffer is
	 *				too small to hold the result 
	 */
	public final int update(
		byte[]	input,
		int	inputOffset,
		int	inputLen,
		byte[]	output)
	throws IllegalStateException, ShortBufferException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input == null || output == null || inputOffset < 0
			|| inputLen > (input.length - inputOffset)
			|| inputLen < 0)
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		if (inputLen == 0)
		{
			return 0;
		}

		return cipherSpi.engineUpdate(input, inputOffset, inputLen, output, 0);
	}

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, are processed, and the result is stored in the output
	 * buffer, starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown. In this case, repeat this call with
	 * a larger output buffer. Use getOutputSize to determine how big the
	 * output buffer should be.
	 * 
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * @return the number of bytes stored in output 
	 * @exception IllegalStateException if this cipher is in a wrong
	 *				state (e.g., has not been initialised) 
	 * @exception ShortBufferException if the given output buffer is
	 *				too small to hold the result 
	 */
	public final int update(
		byte[]	input,
		int	inputOffset,
		int	inputLen,
		byte[]	output,
		int	outputOffset)
		throws IllegalStateException, ShortBufferException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input == null || output == null
			|| inputOffset < 0
			|| outputOffset < 0
			|| inputLen > (input.length - inputOffset)
			|| inputLen < 0)
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		if (inputLen == 0)
		{
			return 0;
		}


		return cipherSpi.engineUpdate(input, inputOffset, inputLen,
			output, outputOffset);
	}

	/**
	 * Finishes a multiple-part encryption or decryption operation,
	 * depending on how this cipher was initialised.
	 * <p>
	 * Input data that may have been buffered during a previous update
	 * operation is processed, with padding (if requested) being applied.
	 * The result is stored in a new buffer.
	 * <p>
	 * The cipher is reset to its initial state (uninitialised) after
	 * this call. 
	 *
	 * @returns the new buffer with the result
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *	(e.g., has not been initialised)
	 * @exception IllegalBlockSizeException if this cipher is a block
	 * 	cipher, no padding has been requested (only in encryption
	 *	mode), and the total input length of the data processed by
	 *	this cipher is not a multiple of block size 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *	and (un)padding has been requested, but the decrypted data is
	 *	not bounded by the appropriate padding bytes.
	 */
	public final byte[] doFinal()
		throws IllegalStateException, IllegalBlockSizeException,
			BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		return cipherSpi.engineDoFinal(null, 0, 0);
	}

	/**
	 * Finishes a multiple-part encryption or decryption operation,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * Input data that may have been buffered during a previous update
	 * operation is processed, with padding (if requested) being applied.
	 * The result is stored in the output buffer, starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown. In this case, repeat this call with
	 * a larger output buffer. Use getOutputSize to determine how big the
	 * output buffer should be. 
	 *
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * @return the number of bytes stored in output 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *     (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *     cipher, no padding has been requested (only in encryption mode),
	 *     and the total input length of the data processed by this cipher
	 *     is not a multiple of block size 
	 * @exception ShortBufferException if the given output buffer is too
	 *     small to hold the result 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *     and (un)padding has been requested, but the decrypted data is
	 *     not bounded by the appropriate padding bytes 
	 */
	public final int doFinal(
		byte output[],
		int outputOffset)
		throws IllegalStateException, IllegalBlockSizeException,
				ShortBufferException, BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (outputOffset < 0)
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		return cipherSpi.engineDoFinal(null, 0, 0,
							output, outputOffset);
	}

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes a
	 * multiple-part operation. The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The bytes in the input buffer, and any input bytes that may have 
	 * been buffered during a previous update operation, are processed,
	 * with padding (if requested) being applied. The result is stored in
	 * a new buffer. 
	 *
	 * @param input - the input buffer 
	 * @returns the new buffer with the result 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *      (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *      cipher, no padding has been requested (only in encryption mode),
	 *      and the total input length of the data processed by this cipher
	 *      is not a multiple of block size 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *      and (un)padding has been requested, but the decrypted data is
	 *      not bounded by the appropriate padding bytes 
	 */
	public final byte[] doFinal(
		byte input[])
		throws IllegalStateException, IllegalBlockSizeException,
							BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (input != null)
		{
			return cipherSpi.engineDoFinal(input, 0, input.length);
		}
		else
		{
			return cipherSpi.engineDoFinal(null, 0, 0);
		}
	}

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes
	 * a multiple-part operation.  The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in a new buffer. 
	 *
	 * @param input - the input buffer 
	 * @param inputOffset - the offset in input where the input starts 
	 * @param inputLen - the input length 
	 * @return the new buffer with the result 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *      (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *      cipher, no padding has been requested (only in encryption mode),
	 *      and the total input length of the data processed by this cipher
	 *      is not a multiple of block size 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *      and (un)padding has been requested, but the decrypted data is
	 *      not bounded by the appropriate padding bytes 
	 */
	public final byte[] doFinal(
		byte	input[],
		int	inputOffset,
		int	inputLen)
		throws IllegalStateException, IllegalBlockSizeException,
							BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (inputOffset < 0
			|| inputLen > (input.length - inputOffset))
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		return cipherSpi.engineDoFinal(input, inputOffset, inputLen);
	}

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes a
	 * multiple-part operation. The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in the output buffer. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown.  In this case, repeat this call
	 * with a larger output buffer. Use getOutputSize to determine how big
	 * the output buffer should be. 
	 * 
	 * @param input - the input buffer 
	 * @param inputOffset - the offset in input where the input starts 
	 * @param inputLen - the input length 
	 * @param output - the buffer for the result 
	 * @return the number of bytes stored in output 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *     (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *     cipher, no padding has been requested (only in encryption mode),
	 *     and the total input length of the data processed by this cipher
	 *     is not a multiple of block size 
	 * @exception ShortBufferException if the given output buffer is too
	 *     small to hold the result 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *     and (un)padding has been requested, but the decrypted data is
	 *     not bounded by the appropriate padding bytes 
	 */
	public final int doFinal(
		byte	input[],
		int	inputOffset,
		int	inputLen,
		byte output[])
		throws IllegalStateException, ShortBufferException,
				IllegalBlockSizeException, BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (output == null
			|| inputOffset < 0
			|| (input != null && inputLen > (input.length - inputOffset)))
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		return cipherSpi.engineDoFinal(input, inputOffset, inputLen,
							output, 0);
	}

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes a
	 * multiple-part operation. The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in the output buffer,
	 * starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown.  In this case, repeat this call
	 * with a larger output buffer. Use getOutputSize to determine how big
	 * the output buffer should be. 
	 *
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * @return the number of bytes stored in output 
	 * @exception IllegalStateException if this cipher is in a wrong state
	 *    (e.g., has not been initialised) 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *     cipher, no padding has been requested (only in encryption mode),
	 *     and the total input length of the data processed by this cipher
	 *     is not a multiple of block size 
	 * @exception ShortBufferException if the given output buffer is too
	 *     small to hold the result 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *     and (un)padding has been requested, but the decrypted data is
	 *     not bounded by the appropriate padding bytes 
	 */
	public final int doFinal(
		byte	input[],
		int	inputOffset,
		int	inputLen,
		byte	output[],
		int	outputOffset)
		throws IllegalStateException, ShortBufferException,
				IllegalBlockSizeException, BadPaddingException
	{
		if (!initialised)
		{
			throw new IllegalStateException("Cipher has not been initialised");
		}

		if (output == null
			|| inputOffset < 0
			|| outputOffset < 0
			|| (input != null && inputLen > (input.length - inputOffset)))
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		return cipherSpi.engineDoFinal(input, inputOffset, inputLen,
							output, outputOffset);
	}
}
