/*
 * $Id: CipherInputStream.java,v 1.7 1998/10/26 06:24:17 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/javax/crypto/CipherInputStream.java,v $
 * $Revision: 1.7 $
 * $Date: 1998/10/26 06:24:17 $
 * $State: Exp $
 */

package javax.crypto;

import java.io.*;

/**
 * A stream filter that decrypts bytes as they are read.
 *
 * @see StreamDecryptor
 */
public class CipherInputStream extends FilterInputStream
{
	public final static String ident = "$Id: CipherInputStream.java,v 1.7 1998/10/26 06:24:17 leachbj Exp $";

	private boolean finalCalled = false;
	private int	blockSize = 0;

	//==================================
	// Constructors
	//==================================

	/**
	 * Construct an SecureInputStream.
	 *
	 * @param in		The input stream to decrypt.
	 * @param cipher	The decrypting/encrypting object.
	 */
	public CipherInputStream(
		InputStream	in,
		Cipher		cipher)
	{
		super(in);
		this.cipher = cipher;

		blockSize = cipher.getBlockSize();
		if (blockSize > 1)
		{
			this.buffer = new byte[cipher.getOutputSize(blockSize)];
			this.rawBuffer = new byte[blockSize];
			this.bpos = this.buffer.length;
		}
		this.onebyte = new byte[1];
	}


	//==================================
	// Public Interface
	//==================================

	/**
	 * Read a byte from the stream and decrypt it. This will block if
	 * no byte is available.
	 *
	 * @return		The decrypted next byte from the stream,
	 *			or -1 if the end of the stream has been
	 *			reached.
	 * @exception		IOException If the byte cannot be decrypted
	 *			or an I/O error occurs.
	 */
	public int read()
		throws IOException
	{
		if (buffer != null && bpos < buffer.length)
		{
			return buffer[bpos++] & 0xff;
		}
		else
		{
			if (read(onebyte, 0, 1) == -1)
			{
				return -1;
			}

			return onebyte[0] & 0xff;
		}
	}

	/**
	 * Reads up to b.length bytes of data from this input stream into an
	 * array of bytes. 
	 * 
	 * The read method of InputStream calls the read method of three
	 * arguments with the arguments b, 0, and b.length. 
	 */
	public int read(
		byte b[])
		throws IOException
	{
		return read(b, 0, b.length);
	}

	/**
	 * Read an array of bytes from the stream and decrypt them.
	 * If no bytes are available, this will block until at least
	 * one byte can be read.
	 *
	 * @param b		The byte[] to read into.
	 * @param off		The index within the byte[] to place data.
	 * @param len		The maximum number of bytes to read.
	 * @return		The number of bytes actually read.
	 * @exception		IOException If any byte cannot be decrypted
	 *			or an I/O error occurs.
	 */
	public int read(
		byte b[],
		int off,
		int len)
		throws IOException
	{
		if (buffer == null)
		{
			// a stream cipher - no buffering
			byte[] buf = new byte[len];
			int count = super.read(buf, 0, len);

			// eof?
			if (count < 0) return count;

			try
			{
				return cipher.update(buf, 0, count, b, off);
			}
			catch (ShortBufferException e)
			{
				throw new IOException(e.getMessage());
			}
		}

		if (bpos >= buffer.length)
		{
			if (readBlock() == -1)
			{
				return -1;
			}
		}

		// copy any buffered data across
		int n;
		if (len < buffer.length - bpos)
		{
			n = len;
		}
		else
		{
			n = buffer.length - bpos;
		}

		if (b != null)
		{
			System.arraycopy(buffer, bpos, b, off, n);
		}
		bpos += n;

		return n;
	}

	public void close()
		throws IOException
	{
		if (!finalCalled)    // discard what's left and finalise Cipher
		{
			int	outSize;

			finalCalled = true;

			bpos = 0;

			outSize = cipher.getOutputSize(0);
			if (outSize > buffer.length)
			{
				buffer = new byte[outSize];
			}

			try
			{
				cipher.doFinal(buffer, 0);
			}
			catch (Exception e)
			{
				throw new IOException(e.toString());
			}
		}

		super.close();
	}

	//==================================
	// Private Implementation
	//==================================

	/*
	 * The decryption object that is used to cipher the
	 * byte stream.
	 */
	private Cipher cipher = null;

	/*
	 * Buffer for the current block.
	 */
	private byte[] buffer = null;
	private byte[] rawBuffer = null;
	private byte[] onebyte = null;
	private int bpos = 0;

	private int readFully(
		byte[] in,
		int off,
		int len)
		throws IOException
	{
		int n = 0;
		while (n < len)
		{
			int count = super.read(in, off + n, len - n);
			if (count < 0)
			{
				if (n > 0)
				{
					return n;
				}
				else
				{
					return -1;
				}
			}
			n += count;
		}
		return n;
	}

	private int readBlock()
		throws IOException
	{
		// buffer a block, or some multiple thereof
		int	n, max;
		
		max = (available() / blockSize) * blockSize;
		if (rawBuffer.length < max || blockSize < max)
		{
			rawBuffer = new byte[max];
			buffer = new byte[cipher.getOutputSize(max)];
		}
		else
		{
			rawBuffer = new byte[blockSize];
			buffer = new byte[cipher.getOutputSize(blockSize)];
		}

		n = readFully(rawBuffer, 0, rawBuffer.length);
		if (n == -1)
		{
			if (!finalCalled)
			{
				int	outSize;

				finalCalled = true;

				bpos = 0;

				outSize = cipher.getOutputSize(0);
				if (outSize > buffer.length)
				{
					buffer = new byte[outSize];
				}

				try
				{
					n = cipher.doFinal(buffer, 0);
				}
				catch (Exception e)
				{
					throw new IOException(e.toString());
				}

				if (n == 0)
				{
					return -1;
				}

				if (n < buffer.length)
				{
					bpos = buffer.length - n;
					System.arraycopy(
						buffer, 0, buffer, bpos, n);
				}
			}
			
			return n;
		}

		if (n < max)
		{
			// EOF read before end of block
			throw new IOException("EOF read mid-block!");
		}

		try
		{
			// reset to the start of plaintext
			bpos = 0;

			// de-crypt the block
			n = cipher.update(rawBuffer, 0, rawBuffer.length, buffer, 0);

			if (n < buffer.length)
			{
				bpos = buffer.length - n;
				System.arraycopy(buffer, 0, buffer, bpos, n);
			}

			/*
			 * this will happen if the cipher needs more priming
			 */
			if (n == 0)
			{
				return readBlock();
			}

			return n;
		}
		catch (ShortBufferException e)
		{
			throw new IOException(e.getMessage());
		}
	}
}

