/*
 * $Id: CipherInputStream.java,v 1.15 1998/10/27 05:11:14 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/CipherInputStream.java,v $
 * $Revision: 1.15 $
 * $Date: 1998/10/27 05:11:14 $
 * $State: Exp $
 */

package javax.crypto;

import java.io.*;

/**
 * A CipherInputStream is composed of an InputStream and a Cipher so that
 * read() methods return data that are read in from the underlying InputStream but
 * have been additionally processed by the Cipher. The Cipher must be fully
 * initialised before being used by a CipherInputStream. 
 * <p>
 * For example, if the Cipher is initialised for decryption, the CipherInputStream
 * will attempt to read in data and decrypt them, before returning the decrypted
 * data. 
 * <p>
 * This class adheres strictly to the semantics, especially the failure semantics,
 * of its ancestor classes java.io.FilterInputStream and java.io.InputStream. This
 * class has exactly those methods specified in its ancestor classes, and
 * overrides them all. Moreover, this class catches all exceptions that are not
 * thrown by its ancestor classes. In particular, the skip() method skips
 * only data that have been processed by the encapsulated Cipher. 
 * <p>
 * It is crucial for a programmer using this class not to use methods that are not
 * defined or overridden in this class (such as a new method or constructor that is
 * later added to one of the super classes), because the design and implementation
 * of those methods are unlikely to have considered security impact with regard to
 * CipherInputStream. 
 * 
 * @see InputStream
 * @see FilterInputStream
 * @see Cipher
 * @see CipherOutputStream
 */
public class CipherInputStream extends FilterInputStream
{
	public final static String ident = "$Id: CipherInputStream.java,v 1.15 1998/10/27 05:11:14 leachbj Exp $";

	private boolean		finalCalled = false;
	private int		blockSize = 0;

	private byte[]		rawBuffer = null;

	private int		bStart = 0;
	private int		bEnd = 0;
	private byte[]		buffer = null;

	private static int	RAW_BUFFER_SIZE = 1024;

	//==================================
	// Constructors
	//==================================

	/**
	 * Constructs a CipherInputStream from an InputStream and a Cipher.
	 *
	 * @param in		The input stream.
	 * @param cipher	The decrypting/encrypting object.
	 */
	public CipherInputStream(
		InputStream	in,
		Cipher		cipher)
	{
		super(in);
		this.cipher = cipher;

		buffer = new byte[cipher.getOutputSize(RAW_BUFFER_SIZE)];
		rawBuffer = new byte[RAW_BUFFER_SIZE];
		bEnd = bStart = 0;
	}

	/**
	 * Constructs a CipherInputStream from an InputStream without specifying
	 * a Cipher. This has the effect of constructing a CipherInputStream
	 * using a NullCipher.
	 *
	 * @param in		The input stream.
	 */
	protected CipherInputStream(
		InputStream in)
	{
		this(in, new NullCipher());
	}

	//==================================
	// Public Interface
	//==================================

	/**
	 * Reads the next byte of data from this input stream. The value byte
	 * is returned as an int in the range 0 to 255. If no byte is available
	 * because the end of the stream has been reached, the value -1 is
	 * returned. This method blocks until input data is available, the end
	 * of the stream is detected, or an exception is thrown. 
	 *
	 * @return the next byte of data, or -1 if the end of the stream
	 *     is reached.
	 * @exception IOException if an I/O error occurs.
	 * @see #read(byte[], int, int)
	 */
	public int read()
		throws IOException
	{
		if (bStart == bEnd)
		{
			if (readBlock() < 0)
			{
				return -1;
			}
		}

		return buffer[bStart++] & 0xff;
	}

	/**
	 * Reads up to b.length bytes of data from this input stream into an array
	 * of bytes.
	 * <p>
	 * The read method of InputStream calls the read method of three
	 * arguments with the arguments b, 0, and b.length.
	 *
	 * @param b the buffer into which the data is read. 
	 * @return the total number of bytes read into the buffer, or -1 if
	 *    there is no more data because the end of the stream has been
	 *    reached.
	 * @exception IOException if an I/O error occurs.
	 * @see #read(byte[], int, int)
	 */
	public int read(
		byte b[])
		throws IOException
	{
		return read(b, 0, b.length);
	}

	/**
	 * Reads up to len bytes of data from this input stream into an array
	 * of bytes. This method blocks until some input is available. If the
	 * first argument is null, up to len bytes are read and discarded.
	 *
	 * @param b the buffer into which the data is read. 
	 * @param off the start offset of the data. 
	 * @param len the maximum number of bytes read. 
	 * @return the total number of bytes read into the buffer, or -1 if
	 *    there is no more data because the end of the stream has been
	 *    reached.
	 * @exception IOException if an I/O error occurs.
	 * @see #read()
	 */
	public int read(
		byte	b[],
		int	off,
		int	len)
		throws IOException
	{
		if (bStart >= bEnd)
		{
			if (readBlock() == -1)
			{
				return -1;
			}
		}

		// copy any buffered data across
		int n;

		if (len < bEnd - bStart)
		{
			n = len;
		}
		else
		{
			n = bEnd - bStart;
		}

		if (n != 0)
		{
			/*
			 * Copy in the read data if required
			 */
			if (b != null)
			{
				System.arraycopy(buffer, bStart, b, off, n);
			}
		}

		bStart += n;

		return n;
	}

	/**
	 * Skips n bytes of input that can be read from this input stream
	 * without blocking.
	 * <p>
	 * Fewer bytes than requested might be skipped.  The actual number of
	 * bytes skipped is equal to n or the result of a call to available,
	 * whichever is smaller.  If n is less than zero, no bytes are
	 * skipped.
	 * <p>
	 * The actual number of bytes skipped is returned.
	 *
	 * @param n the number of bytes to be skipped. 
	 * @return the actual number of bytes skipped.
	 * @exception IOException if an I/O error occurs.
	 */
	public long skip(
		long	n)
		throws IOException
	{
		int	count;

		count = bEnd - bStart;

		if (n > count)
		{
			bStart = bEnd;
			return count;
		}

		bStart += (int)n;
		return n;
	}

	/**
	 * Returns the number of bytes that can be read from this input stream
	 * without blocking. The available method of InputStream returns 0.
	 * This method <b>should</b> be overridden by subclasses.
	 *
	 * @return the number of bytes that can be read from this input stream
	 *     without blocking.
	 * @exception IOException if an I/O error occurs. 
	 */
	public int available()
		throws IOException
	{
		return bEnd - bStart;
	}
		
	/**
	 * Closes this input stream and releases any system resources associated
	 * with the stream. 
	 * <p>
	 * The close method of CipherInputStream calls the close method of its
	 * underlying input stream.
	 *
	 * @exception IOException - if an I/O error occurs.
	 */
	public void close()
		throws IOException
	{
		if (!finalCalled)    // discard what's left and finalise Cipher
		{
			int	outSize;

			finalCalled = true;

			outSize = cipher.getOutputSize(0);
			if (buffer != null)
			{
				if (outSize > buffer.length)
				{
					buffer = new byte[outSize];
				}

				try
				{
					cipher.doFinal(buffer, 0);
				}
				catch (Exception e)
				{
					throw new IOException(e.toString());
				}
			}
			else
			{
				try
				{
					cipher.doFinal();
				}
				catch (Exception e)
				{
					throw new IOException(e.toString());
				}
			}
		}

		super.close();
	}

	/**
	 * Tests if this input stream supports the mark and reset methods,
	 * which it does not.
	 *
	 * @return false, since this class does not support the mark and
	 *     reset methods.
	 * @see #mark(int)
	 * @see #reset()
	 */
	public boolean markSupported()
	{
		return false;
	}

	//==================================
	// Private Implementation
	//==================================

	/*
	 * The decryption object that is used to cipher the
	 * byte stream.
	 */
	private Cipher cipher = null;

	/*
	 * Buffer for the current block.
	 */

	private int readBlock()
		throws IOException
	{
		int	n, avail;
		
		avail = super.available();
		if (avail > rawBuffer.length)
		{
			avail = rawBuffer.length;
		}
		else if (avail == 0)
		{
			avail = 1;
		}

		n = super.read(rawBuffer, 0, avail);
		if (n == -1)
		{
			if (!finalCalled)
			{
				int	outSize;

				finalCalled = true;

				outSize = cipher.getOutputSize(0);
				if (outSize > buffer.length)
				{
					buffer = new byte[outSize];
				}

				try
				{
					n = cipher.doFinal(buffer, 0);
				}
				catch (RuntimeException e)
				{
					throw e;
				}
				catch (Exception e)
				{
					throw new IOException(e.toString());
				}

				if (n == 0)
				{
					return -1;
				}

				bEnd = n;
				bStart = 0;
			}
			
			return n;
		}

		try
		{
			// reset to the start of plaintext

			// de-crypt the block
			n = cipher.update(rawBuffer, 0, n, buffer, 0);

			/*
			 * this will happen if the cipher needs more priming
			 */
			if (n == 0)
			{
				return readBlock();
			}
			else
			{
				bStart = 0;
				bEnd = n;
			}

			return n;
		}
		catch (ShortBufferException e)
		{
			throw new IOException(e.getMessage());
		}
	}
}
