/*
 * $Id: CipherOutputStream.java,v 1.12 1998/10/29 06:39:08 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/CipherOutputStream.java,v $
 * $Revision: 1.12 $
 * $Date: 1998/10/29 06:39:08 $
 * $State: Exp $
 */

package javax.crypto;

import java.io.*;

/**
 * A CipherOutputStream is composed of an OutputStream and a Cipher so that
 * write() methods first process the data before writing them out to the
 * underlying OutputStream. The cipher must be fully initialised before being
 * used by a CipherOutputStream. 
 * <p>
 * For example, if the cipher is initialised for encryption, the
 * CipherOutputStream will attempt to encrypt data before writing out the
 * encrypted data. 
 * <p>
 * This class adheres strictly to the semantics, especially the failure
 * semantics, of its ancestor classes java.io.OutputStream and
 * java.io.FilterOutputStream. This class has exactly those methods specified
 * in its ancestor classes, and overrides them all. Moreover, this class catches
 * all exceptions that are not thrown by its ancestor classes. 
 * <p>
 * It is crucial for a programmer using this class not to use methods that are
 * not defined or overridden in this class (such as a new method or constructor
 * that is later added to one of the super classes), because the design and
 * implementation of those methods are unlikely to have considered security
 * impact with regard to CipherOutputStream.
 *
 * @see OutputStream
 * @see FilterOutputStream
 * @see Cipher
 * @see CipherInputStream
 */
public class CipherOutputStream extends FilterOutputStream
{
	public final static String ident = "$Id: CipherOutputStream.java,v 1.12 1998/10/29 06:39:08 leachbj Exp $";

	/*
	 * The encryption object that is used to cipher the
	 * byte stream.
	 */
	private Cipher cipher = null;

	boolean flushed = false;	/* have we been flushed, or not */
	byte[]	oneByte = new byte[1];

	/**
	 * Constructs a CipherOutputStream from an OutputStream and a Cipher.
	 *
	 * @param out		The output stream.
	 * @param cipher	The encrypting object.
	 */
	public CipherOutputStream(
		OutputStream	out,
		Cipher		cipher)
	{
		super(out);
		this.cipher = cipher;
	}

	/**
	 * Constructs a CipherOutputStream from an OutputStream without
	 * specifying a Cipher. This has the effect of constructing a
	 * CipherOutputStream using a NullCipher.
	 *
	 * @param out		The output stream.
	 */
	public CipherOutputStream(
		OutputStream	out)
	{
		this(out, new NullCipher());
	}

	//==================================
	// Public Interface
	//==================================

	/**
	 * Writes the specified byte to this output stream.
	 *
	 * @param b		The byte to write.
	 * @exception		IOException If an I/O error occurs.
	 */
	public void write(
		int b)
		throws IOException
	{
		oneByte[0] = (byte)b;
		write(oneByte, 0, 1);
	}

	/**
	 * Writes b.length bytes from the specified byte array to this output
	 * stream. 
	 * <p>
	 * The write method of CipherOutputStream calls the write method of
	 * three arguments with the three arguments b, 0, and b.length. 
	 *
	 * @param b the data. 
	 * @exception  IOException if an I/O error occurs. 
	 * @see #write(byte[], int, int)
	 */
	public void write(
		byte[]	b)
		throws IOException
	{
		write(b, 0, b.length);
	}

	/**
	 * Writes len bytes from the specified byte array starting at offset
	 * off to this output stream.
	 *
	 * @param b		The bytes to be written.
	 * @param off		The index of the first byte to write.
	 * @param len		The number of bytes to write.
	 * @exception		IOException If an I/O error occurs.
	 */
	public void write(
		byte b[],
		int off,
		int len)
		throws IOException
	{
		byte[] enc;

		if ((enc = cipher.update(b, off, len)) != null)
		{
			out.write(enc, 0, enc.length);
		}
	}

	/**
	 * Flushes this output stream by forcing any buffered output bytes
	 * that have already been processed by the encapsulated cipher object
	 * to be written out.
	 * <p>
	 * Any bytes buffered by the encapsulated cipher and waiting to be
	 * processed by it will not be written out. For example, if the
	 * encapsulated cipher is a block cipher, and the total number of
	 * bytes written using one of the write methods is less than the
	 * cipher's block size, no bytes will be written out.
	 *
	 * @exception IOException if an I/O error occurs
	 */
	public void flush()
		throws IOException
	{
		super.flush();
	}

	/**
	 * Closes this output stream and releases any system resources
	 * associated with this stream.
	 * <p>
	 * This method invokes the doFinal method of the encapsulated cipher
	 * object, which causes any bytes buffered by the encapsulated cipher
	 * to be processed. The result is written out by calling the flush
	 * method of this output stream.
	 * <p>
	 * This method resets the encapsulated cipher object to its initial
	 * state and calls the close method of the underlying output stream.
	 *
	 * @exception IOException if an I/O error occurs
	 */
	public void close()
		throws IOException
	{
		try
		{
			byte[]	enc;

			if ((enc = cipher.doFinal()) != null)
			{
				out.write(enc, 0, enc.length);
			}
		}
		catch (Exception e)
		{
			throw new IOException(e.getMessage());
		}

		flush();

		super.close();
	}
}
