/*
 * $Id: CipherOutputStream.java,v 1.5 1998/10/20 04:03:46 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/javax/crypto/CipherOutputStream.java,v $
 * $Revision: 1.5 $
 * $Date: 1998/10/20 04:03:46 $
 * $State: Exp $
 */

package javax.crypto;

import java.io.*;

/**
 * A stream filter that encrypts bytes as they are written.
 *
 * @see StreamEncryptor
 */
public class CipherOutputStream extends FilterOutputStream
{
	public final static String ident = "$Id: CipherOutputStream.java,v 1.5 1998/10/20 04:03:46 leachbj Exp $";

	boolean flushed = false;	/* have we been flushed, or not */

	/**
	 * Construct an SecureOutputStream.
	 *
	 * @param out		The output stream to encrypt.
	 * @param encryptor	The encrypting object.
	 */
	public CipherOutputStream(
		OutputStream	out,
		Cipher		cipher)
	{
		super(out);
		this.cipher = cipher;
	}


	//==================================
	// Public Interface
	//==================================

	/**
	 * Encrypt a byte and write it to the stream.
	 *
	 * @param b		The byte to write.
	 * @exception		IOException If the byte cannot be encrypted
	 *			or written.
	 */
	public void write(
		int b)
		throws IOException
	{
		byte[]	val, enc;

		val = new byte[1];
		val[0] = (byte)b;

		if ((enc = cipher.update(val)) != null)
		{
			super.write(enc, 0, enc.length);
		}
	}

	/**
	 * Writes b.length bytes from the specified byte array to this output
	 * stream. 
	 * <p>
	 * The write method of CipherOutputStream calls the write method of
	 * three arguments with the three arguments b, 0, and b.length. 
	 *
	 * @param b the data. 
	 * @exception  IOException if an I/O error occurs. 
	 * @see write
	 */
	public void write(
		byte[]	b)
		throws IOException
	{
		write(b, 0, b.length);
	}

	/**
	 * Encrypt an array of bytes and write it to the stream.
	 *
	 * @param b		The bytes to be written.
	 * @param off		The index of the first byte to write.
	 * @param len		The number of bytes to write.
	 * @exception		IOException If the byte cannot be encrypted
	 *			or written.
	 */
	public void write(
		byte b[],
		int off,
		int len)
		throws IOException
	{
		byte[] enc;

		if ((enc = cipher.update(b, off, len)) != null)
		{
			super.write(enc, 0, enc.length);
		}
	}

	/**
	 * Flushes this output stream and forces any processed buffered
	 * output bytes to be written out. 
	 *
	 * @exception IOException if an I/O error occurs
	 */
	public void flush()
		throws IOException
	{
		super.flush();
	}

	/**
	 * Closes this output stream and releases any system resources
	 * associated with this stream.
	 * <p>
	 * The close method of CipherOutputStream calls its flush method,
	 * and then calls the close method of its underlying output stream. 
	 *
	 * @exception IOException if an I/O error occurs
	 */
	public void close()
		throws IOException
	{
		try
		{
			byte[]	enc;

			if ((enc = cipher.doFinal()) != null)
			{
				super.write(enc, 0, enc.length);
			}
		}
		catch (Exception e)
		{
			throw new IOException(e.getMessage());
		}

		flush();

		super.close();
	}

	//==================================
	// Private Implementation
	//==================================

	/*
	 * The encryption object that is used to cipher the
	 * byte stream.
	 */
	private Cipher cipher = null;
}

