/*
 * $Id: CipherSpi.java,v 1.9 1999/01/07 00:08:25 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/CipherSpi.java,v $
 * $Revision: 1.9 $
 * $Date: 1999/01/07 00:08:25 $
 * $State: Exp $
 */

package javax.crypto;

import java.security.*;

import au.net.aba.security.*;
import au.net.aba.security.spec.*;

/**
 * This class defines the Service Provider Interface (SPI) for the
 * Cipher class. All the abstract methods in this class must be
 * implemented by each cryptographic service provider who wishes to
 * supply the implementation of a particular cipher algorithm.
 * <p>
 * In order to create an instance of Cipher, which encapsulates an
 * instance of this CipherSpi class, an application calls one of the
 * getInstance factory methods of the Cipher engine class and specifies
 * the requested transformation. Optionally, the application may also
 * specify the name of a provider.
 * <p>
 * A transformation is a string that describes the operation (or set of
 * operations) to be performed on the given input, to produce some
 * output.  A transformation always includes the name of a cryptographic
 * algorithm (e.g., DES), and may be followed by a feedback mode and
 * padding scheme.
 * <p>
 * A transformation is of the form:
 * <ul>
 * <li>"algorithm/mode/padding" or 
 * <li>"algorithm" 
 * </ul>
 * <p>
 * (in the latter case, provider-specific default values for the mode
 * and padding scheme are used). For example, the following is a valid
 * transformation:
 * <pre>
 * Cipher c = Cipher.getInstance("DES/CBC/PKCS5Padding");
 * </pre>
 * <p>
 * A provider may supply a separate class for each combination of
 * algorithm/mode/padding, or may decide to provide more generic classes
 * representing sub-transformations corresponding to algorithm or
 * algorithm/mode or algorithm//padding (note the double slashes), in
 * which case the requested mode and/or padding are set automatically by
 * the getInstance methods of Cipher, which invoke the engineSetMode and
 * engineSetPadding methods of the provider's subclass of CipherSpi.
 * <p>
 * A Cipher property in a provider master class may have one of the
 * following formats:
 * <ul>
 * <li><pre>
 * // provider's subclass of "CipherSpi" implements "algName" with
 * // pluggable mode and padding
 * Cipher.algName
 * </pre>
 * <li><pre>
 * // provider's subclass of "CipherSpi" implements "algName" in
 * // the
 * // specified "mode", with pluggable padding
 * Cipher.algName/mode
 * </pre>
 * <li><pre>
 * // provider's subclass of "CipherSpi" implements "algName" with
 * // the
 * // specified "padding", with pluggable mode
 * Cipher.algName//padding
 * </pre>
 * <li><pre>
 * // provider's subclass of "CipherSpi" implements "algName" with
 * // the
 * // specified "mode" and "padding"
 * Cipher.algName/mode/padding
 * </pre>
 * </ul>
 * <p>
 * For example, a provider may supply a subclass of CipherSpi that
 * implements DES/ECB/PKCS5Padding, one that implements
 * DES/CBC/PKCS5Padding, one that implements DES/CFB/PKCS5Padding, and
 * yet another one that implements DES/OFB/PKCS5Padding. That provider
 * would have the following Cipher properties in its master class:
 * <ul>
 * <li><pre>Cipher.DES/ECB/PKCS5Padding</pre>
 * <li><pre>Cipher.DES/CBC/PKCS5Padding</pre>
 * <li><pre>Cipher.DES/CFB/PKCS5Padding</pre>
 * <li><pre>Cipher.DES/OFB/PKCS5Padding</pre>
 * </ul>
 * <p>
 * Another provider may implement a class for each of the above modes
 * (i.e., one class for ECB, one for CBC, one for CFB, and one for OFB),
 * one class for PKCS5Padding, and a generic DES class that subclasses
 * from CipherSpi. That provider would have the following Cipher
 * properties in its master class:
 * <ul>
 * <li><pre>Cipher.DES</pre>
 * </ul>
 * <p>
 * The getInstance factory method of the Cipher engine class follows
 * these rules in order to instantiate a provider's implementation of
 * CipherSpi for a transformation of the form "algorithm":
 * <ol>
 * <li>Check if the provider has registered a subclass of CipherSpi for
 * the specified "algorithm".
 * <p>
 * If the answer is YES, instantiate this class, for whose mode and
 * padding scheme default values (as supplied by the provider) are used.
 * <p>
 * If the answer is NO, throw a NoSuchAlgorithmException exception. 
 * </ol>
 * <p>The getInstance factory method of the Cipher engine class follows
 * these rules in order to instantiate a provider's implementation of
 * CipherSpi for a transformation of the form "algorithm/mode/padding":
 * <ol>
 * <li>Check if the provider has registered a subclass of CipherSpi for
 * the specified "algorithm/mode/padding" transformation.
 * <p>
 * If the answer is YES, instantiate it. 
 * <p>
 * If the answer is NO, go to the next step.
 * <li>Check if the provider has registered a subclass of CipherSpi for
 * the sub-transformation "algorithm/mode".
 * <p>
 * If the answer is YES, instantiate it, and call
 * engineSetPadding(padding) on the new instance.
 * <p>
 * If the answer is NO, go to the next step.
 * <li>Check if the provider has registered a subclass of CipherSpi for
 * the sub-transformation "algorithm//padding" (note the double
 * slashes). 
 * <p>
 * If the answer is YES, instantiate it, and call engineSetMode(mode) on
 * the new instance.
 * <p>
 * If the answer is NO, go to the next step.
 * <li>Check if the provider has registered a subclass of CipherSpi for
 * the sub-transformation "algorithm". 
 * <p>
 * If the answer is YES, instantiate it, and call engineSetMode(mode)
 * and engineSetPadding(padding) on the new instance. 
 * <p>
 * If the answer is NO, throw a NoSuchAlgorithmException exception.
 *
 * @see KeyGenerator
 * @see SecretKey
 */
public abstract class CipherSpi
{
	public final static String ident = "$Id: CipherSpi.java,v 1.9 1999/01/07 00:08:25 leachbj Exp $";

	public CipherSpi()
	{
	}

	/**
	 * Sets the mode of this cipher. 
	 *
	 * @param mode the cipher mode 
	 * @exception NoSuchAlgorithmException	if the requested cipher mode
	 *					does not exist 
	 */
	protected abstract void engineSetMode(String mode)
		throws NoSuchAlgorithmException;

	/**
	 * Sets the padding mechanism of this cipher.
	 *
	 * @param padding the padding mechanism
	 * @exception NoSuchPaddingException	if the requested padding 
	 *					mechanism does not exist.
	 */
	protected abstract void engineSetPadding(String padding)
		throws NoSuchPaddingException;

	/**
	 * Returns the block size (in bytes). 
	 *
	 * @return the block size (in bytes), or 0 if the underlying
	 *	algorithm is not a block cipher 
	 */
	protected abstract int engineGetBlockSize();

	/**
	 * Returns the length in bytes that an output buffer would need to be
	 * in order to hold the result of the next update or doFinal operation,
	 * given the input length inputLen (in bytes). 
	 * <p>
	 * This call takes into account any unprocessed (buffered) data from a
	 * previous update call, and padding. 
	 * <p>
	 * The actual output length of the next update or doFinal call may be
	 * smaller than the length returned by this method. 
	 *
	 * @param inputLen the input length (in bytes) 
	 * @return the required output buffer size (in bytes)
	 */
	protected abstract int engineGetOutputSize(int inputLen);

	/**
	 * Returns the initialisation vector (IV) in a new buffer. 
	 * <p>
	 * This is useful in the context of password-based encryption or
	 * decryption, where the IV is derived from a user-provided passphrase. 
	 *
	 * @return the initialisation vector in a new buffer, or null if the
	 * 	underlying algorithm does not use an IV, or if the IV has
	 * 	not yet been set. 
	 */
	protected abstract byte[] engineGetIV();

	/**
	 * Returns the parameters used with this cipher. 
	 * <p>
	 * The returned parameters may be the same that were used to initialise
	 * this cipher, or may contain the default set of parameters or a set of
	 * randomly generated parameters used by the underlying cipher
	 * implementation (provided that the underlying cipher implementation
	 * uses a default set of parameters or creates new parameters if it needs
	 * parameters but was not initialised with any).
	 *
	 * @return the parameters used with this cipher, or null if this cipher
	 *     does not use any parameters.
	 */
	protected abstract AlgorithmParameters engineGetParameters();

	/**
	 * Initialises this cipher with a key and a source of randomness.  
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending
	 * on the value of opmode.
	 * <p>
	 * If this cipher requires any algorithm parameters that cannot be
	 * derived from the given key, the underlying cipher implementation
	 * is supposed to generate the required parameters itself (using
	 * provider-specific default or random values) if it is being
	 * initialised for encryption, and raise an InvalidKeyException if it
	 * is being initialised for decryption. The generated parameters can
	 * be retrieved using engineGetParameters or engineGetIV (if the
	 * parameter is an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them from random.
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher and
	 * initialising it

	 * @param opmode the operation mode of this cipher (this is either
	 *			ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the encryption key 
	 * @param random the source of randomness 
	 * @exception InvalidKeyException if the given key is inappropriate
	 *			for initialising this cipher 
	 */
	protected abstract void engineInit(int opmode,
						Key key, SecureRandom random)
		throws InvalidKeyException;

	/**
	 * Initialises this cipher with a key, a set of algorithm parameters,
	 * and a source of randomness.
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending
	 * on the value of opmode.
	 * <p>
	 * If this cipher requires any algorithm parameters and params is
	 * null, the underlying cipher implementation is supposed to generate
	 * the required parameters itself (using provider-specific default or
	 * random values) if it is being initialised for encryption, and
	 * raise an InvalidAlgorithmParameterException if it is being
	 * initialised for decryption. The generated parameters can be
	 * retrieved using engineGetParameters or engineGetIV (if the
	 * parameter is an IV).
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them from random.
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher and
	 * initialising it.
	 * <p>
	 * @param opmode the operation mode of this cipher (this is either
	 *			ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the encryption key 
	 * @param params the algorithm parameters
	 * @param random the source of randomness 
	 *
	 * @exception InvalidKeyException if the given key is inappropriate
	 * 	for initialising this cipher 
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 * 	parameters are inappropriate for this cipher, or if this
	 * 	cipher is being initialised fro decryption and requires
	 * 	algorithm parameters and params is null
	 */
	protected abstract void engineInit(int opmode,
			Key key, AlgorithmParameterSpec params,
						SecureRandom random)
		throws InvalidKeyException, InvalidAlgorithmParameterException;

	/**
	 * Initialises this cipher with a key, a set of algorithm parameters,
	 * and a source of randomness.
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending
	 * on the value of opmode.
	 * <p>
	 * If this cipher requires any algorithm parameters and params is
	 * null, the underlying cipher implementation is supposed to generate
	 * the required parameters itself (using provider-specific default or
	 * random values) if it is being initialised for encryption, and
	 * raise an InvalidAlgorithmParameterException if it is being
	 * initialised for decryption. The generated parameters can be
	 * retrieved using engineGetParameters or engineGetIV (if the
	 * parameter is an IV). 
	 * <p>
	 * If this cipher (including its underlying feedback or padding
	 * scheme) requires any random bytes (e.g., for parameter
	 * generation), it will get them from random. 
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher
	 * is equivalent to creating a new instance of that Cipher and
	 * initialising it.
	 * <p>
	 * @param opmode the operation mode of this cipher (this is either
	 *    ENCRYPT_MODE or DECRYPT_MODE)
	 * @param key the encryption key
	 * @param params the algorithm parameters
	 * @param random the source of randomness
	 * @exception InvalidKeyException if the given key is
	 *    inappropriate for initialising this cipher
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 * 	parameters are inappropriate for this cipher, or if this
	 * 	cipher is being initialised fro decryption and requires
	 * 	algorithm parameters and params is null
	 */
	protected abstract void engineInit(int opmode, Key key,
		AlgorithmParameters params, SecureRandom random)
		throws InvalidKeyException, InvalidAlgorithmParameterException;

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, are processed, and the result is stored in a new buffer.
	 *
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @return the new buffer with the result, or null if the underlying
	 *    cipher is a block cipher and the input data is too short to
	 *    result in a new block. 
	 */
	protected abstract byte[] engineUpdate(
					byte[] input, int inputOff, int inputLen);

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, are processed, and the result is stored in the output
	 * buffer, starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown. In this case, repeat this call with
	 * a larger output buffer. Use getOutputSize to determine how big the
	 * output buffer should be. 
	 * 
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * 
	 * @exception  ShortBufferException if the given output buffer is too
	 *     small to hold the result 
	 *
	 * @return the number of bytes stored in output 
	 */
	protected abstract int engineUpdate(
					byte[] input, int inputOff, int inputLen,
						byte[] output, int outputOff)
		throws ShortBufferException;

	/**
	 * Encrypts or decrypts data in a single-part operation, or
	 * finishes a multiple-part operation. The data is encrypted or
	 * decrypted, depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in a new buffer. 
	 * <p>
	 * The cipher is reset to its initial state (uninitialised) after
	 * this call. 
	 *
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 *
	 * @exception IllegalBlockSizeException if this cipher is a block
	 * 	cipher, no padding has been requested (only in encryption
	 *	mode), and the total input length of the data processed by
	 *	this cipher is not a multiple of block size 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 * 	and (un)padding has been requested, but the decrypted data is
	 *	not bounded by the appropriate padding bytes.
	 *
	 * @return the new buffer with the result 
	 */
	protected abstract byte[] engineDoFinal(
				byte input[], int inputOffset, int inputLen)
		throws IllegalBlockSizeException, BadPaddingException;

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes
	 * a multiple-part operation. The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in the output buffer,
	 * starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown.  In this case, repeat this call
	 * with a larger output buffer. Use getOutputSize to determine how
	 * big the output buffer should be. 
	 * 
	 * @param input the input buffer 
	 * @param inputOffset - the offset in input where the input starts 
	 * @param inputLen - the input length 
	 * @param output - the buffer for the result 
	 * @param outputOffset - the offset in output where the result is stored
	 * 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *    cipher, no padding has been requested (only in encryption mode),
	 *    and the total input length of the data processed by this cipher
	 *    is not a multiple of block size 
	 * @exception ShortBufferException if the given output buffer is too
	 *    small to hold the result 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *    and (un)padding has been requested, but the decrypted data is
	 *    not bounded by the appropriate padding bytes 
	 * @returns the number of bytes stored in output 
	 */
	protected abstract int engineDoFinal(
				byte[] input, int inputOff, int inputLen,
						byte[] output, int outputOff)
		throws ShortBufferException, IllegalBlockSizeException,
							BadPaddingException;
}
