/*
 * $Id: CipherSpi.java,v 1.5 1999/01/07 00:48:56 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/javax/crypto/CipherSpi.java,v $
 * $Revision: 1.5 $
 * $Date: 1999/01/07 00:48:56 $
 * $State: Exp $
 */

package javax.crypto;

import au.net.aba.security.*;
import au.net.aba.security.spec.*;

/**
 * A parent class for the provider implementations of encryption/decryption
 * objects.
 */
public abstract class CipherSpi
{
	public final static String ident = "$Id: CipherSpi.java,v 1.5 1999/01/07 00:48:56 leachbj Exp $";

	public CipherSpi()
	{
	}

	/**
	 * Sets the mode of this cipher. 
	 *
	 * @param mode the cipher mode 
	 * @exception NoSuchAlgorithmException	if the requested cipher mode
	 *					does not exist 
	 */
	protected abstract void engineSetMode(String mode)
		throws NoSuchAlgorithmException;

	/**
	 * Sets the padding mechanism of this cipher.
	 *
	 * @param padding the padding mechanism
	 * @exception NoSuchPaddingException	if the requested padding 
	 *					mechanism does not exist.
	 */
	protected abstract void engineSetPadding(String padding)
		throws NoSuchPaddingException;

	/**
	 * Returns the block size (in bytes). 
	 *
	 * @return the block size (in bytes), or 0 if the underlying
	 *	algorithm is not a block cipher 
	 */
	protected abstract int engineGetBlockSize();

	/**
	 * Returns the length in bytes that an output buffer would need to be
	 * in order to hold the result of the next update or doFinal operation,
	 * given the input length inputLen (in bytes). 
	 * <p>
	 * This call takes into account any unprocessed (buffered) data from a
	 * previous update call, and padding. 
	 * <p>
	 * The actual output length of the next update or doFinal call may be
	 * smaller than the length returned by this method. 
	 *
	 * @param inputLen the input length (in bytes) 
	 * @return the required output buffer size (in bytes)
	 */
	protected abstract int engineGetOutputSize(int inputLen);

	/**
	 * Returns the initialisation vector (IV) in a new buffer. 
	 * <p>
	 * This is useful in the context of password-based encryption or
	 * decryption, where the IV is derived from a user-provided passphrase. 
	 *
	 * @return the initialisation vector in a new buffer, or null if the
	 * 	underlying algorithm does not use an IV, or if the IV has
	 *	not yet been set. 
	 */
	protected abstract byte[] engineGetIV();

	/**
	 * Initialises this cipher with a key and a source of randomness. The
	 * cipher is initialised for encryption or decryption, depending on
	 * the value of opmode. 
	 * <p>
	 * If this cipher requires an initialisation vector (IV), it will get
	 * it from random. The random IV can be retrieved using getIV. This
	 * behaviour should only be used in encryption mode, however. When
	 * initialising a cipher that requires an IV for decryption, the IV
	 * (same IV that was used for encryption) must be provided explicitly
	 * as a parameter, in order to get the correct result. 
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher is
	 * equivalent to creating a new instance of that Cipher, and
	 * initialising it. 
	 * 
	 * @param opmode the operation mode of this cipher (this is either
	 *			ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the encryption key 
	 * @param random the source of randomness 
	 *
	 * @exception InvalidKeyException if the given key is inappropriate
	 *			for initialising this cipher 
	 */
	protected abstract void engineInit(int opmode,
						Key key, SecureRandom random)
		throws InvalidKeyException;

	/**
	 * Initialises this cipher with a key, a set of algorithm parameters,
	 * and a source of randomness. 
	 * <p>
	 * The cipher is initialised for encryption or decryption, depending on
	 * the value of opmode. 
	 * <p>
	 * If this cipher (including its underlying feedback or padding scheme)
	 * requires any random bytes, it will get them from random. 
	 * <p>
	 * Note that when a Cipher object is initialised, it loses all
	 * previously-acquired state. In other words, initialising a Cipher is
	 * equivalent to creating a new instance of that Cipher, and
	 * initialising it. 
	 * 
	 * @param opmode the operation mode of this cipher (this is either
	 *			ENCRYPT_MODE or DECRYPT_MODE) 
	 * @param key the encryption key 
	 * @param params the algorithm parameters
	 * @param random the source of randomness 
	 *
	 * @exception InvalidKeyException if the given key is inappropriate
	 *			for initialising this cipher 
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 *     parameters are inappropriate for this cipher 
	 */
	protected abstract void engineInit(int opmode,
			Key key, AlgorithmParameterSpec params,
						SecureRandom random)
		throws InvalidKeyException, InvalidAlgorithmParameterException;

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, are processed, and the result is stored in a new buffer.
	 *
	 * @param input the input buffer 
         * @param inputOffset the offset in input where the input starts 
         * @param inputLen the input length 
         * @return the new buffer with the result, or null if the underlying
	 *    cipher is a block cipher and the input data is too short to
	 *    result in a new block. 
	 */
	protected abstract byte[] engineUpdate(
					byte[] in, int inOff, int inLen);

	/**
	 * Continues a multiple-part encryption or decryption operation
	 * (depending on how this cipher was initialised), processing another
	 * data part. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, are processed, and the result is stored in the output
	 * buffer, starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown. In this case, repeat this call with
	 * a larger output buffer. Use getOutputSize to determine how big the
	 * output buffer should be. 
	 * 
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 * @param output the buffer for the result 
	 * @param outputOffset the offset in output where the result is stored 
	 * 
	 * @exception  ShortBufferException if the given output buffer is too
	 *     small to hold the result 
	 *
	 * @return the number of bytes stored in output 
	 */
	protected abstract int engineUpdate(
					byte[] in, int inOff, int inLen,
							byte[] out, int outOff)
		throws ShortBufferException;

	/**
	 * Encrypts or decrypts data in a single-part operation, or
	 * finishes a multiple-part operation. The data is encrypted or
	 * decrypted, depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in a new buffer. 
	 * <p>
	 * The cipher is reset to its initial state (uninitialised) after
	 * this call. 
	 *
	 * @param input the input buffer 
	 * @param inputOffset the offset in input where the input starts 
	 * @param inputLen the input length 
	 *
	 * @exception IllegalBlockSizeException if this cipher is a block
	 * 	cipher, no padding has been requested (only in encryption
	 *	mode), and the total input length of the data processed by
	 *	this cipher is not a multiple of block size 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 * 	and (un)padding has been requested, but the decrypted data is
	 *	not bounded by the appropriate padding bytes.
	 *
	 * @return the new buffer with the result 
	 */
	protected abstract byte[] engineDoFinal(
				byte input[], int inputOffset, int inputLen)
		throws IllegalBlockSizeException, BadPaddingException;

	/**
	 * Encrypts or decrypts data in a single-part operation, or finishes
	 * a multiple-part operation. The data is encrypted or decrypted,
	 * depending on how this cipher was initialised. 
	 * <p>
	 * The first inputLen bytes in the input buffer, starting at
	 * inputOffset, and any input bytes that may have been buffered during
	 * a previous update operation, are processed, with padding (if
	 * requested) being applied. The result is stored in the output buffer,
	 * starting at outputOffset. 
	 * <p>
	 * If the output buffer is too small to hold the result, a
	 * ShortBufferException is thrown.  In this case, repeat this call
	 * with a larger output buffer. Use getOutputSize to determine how
	 * big the output buffer should be. 
	 * 
	 * @param input the input buffer 
	 * @param inputOffset - the offset in input where the input starts 
	 * @param inputLen - the input length 
	 * @param output - the buffer for the result 
	 * @param outputOffset - the offset in output where the result is stored
	 * 
	 * @exception IllegalBlockSizeException if this cipher is a block
	 *    cipher, no padding has been requested (only in encryption mode),
	 *    and the total input length of the data processed by this cipher
	 *    is not a multiple of block size 
	 * @exception ShortBufferException if the given output buffer is too
	 *    small to hold the result 
	 * @exception BadPaddingException if this cipher is in decryption mode,
	 *     and (un)padding has been requested, but the decrypted data is
	 *     not bounded by the appropriate padding bytes 
	 * @returns the number of bytes stored in output 
	 */
	protected abstract int engineDoFinal(
				byte[] in, int inOff, int inLen,
							byte[] out, int outOff)
		throws ShortBufferException, IllegalBlockSizeException,
							BadPaddingException;
}
