/*
 * $Id: KeyGenerator.java,v 1.10 1998/10/29 06:17:02 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/KeyGenerator.java,v $
 * $Revision: 1.10 $
 * $Date: 1998/10/29 06:17:02 $
 * $State: Exp $
 */
package javax.crypto;

import java.security.*;
import java.util.*;

import au.net.aba.security.InvalidAlgorithmParameterException;
import au.net.aba.security.spec.AlgorithmParameterSpec;

/**
 * This class provides the functionality of a (symmetric) key generator. 
 * <p>
 * Key generators are constructed using one of the getInstance class methods
 * of this class. 
 * <p>
 * KeyGenerator objects are reusable, i.e., after a key has been generated, the
 * same KeyGenerator object can be re-used to generate further keys. 
 * <p>
 * There are two ways to generate a key: in an algorithm-independent manner,
 * and in an algorithm-specific manner. The only difference between the two is
 * the initialisation of the object: 
 * <p>
 *   Algorithm-Independent Initialisation 
 * <p>
 *   All key generators share the concepts of a "strength" and a source of
 *   randomness. The measure of strength is universally shared by all
 *   algorithms, though it is interpreted differently for different algorithms.
 *   There is an init method in this KeyGenerator class that takes these two
 *   universally shared types of arguments. There is also one that takes just a
 *   strength argument, and uses a system-provided source of randomness,
 *   and one that takes just a source of randomness. 
 * <p>
 *   Since no other parameters are specified when you call the above
 *   algorithm-independent init methods, it is up to the provider what to do
 *   about the algorithm-specific parameters (if any) to be associated with
 *   each of the keys. 
 * <p>
 *   Algorithm-Specific Initialisation 
 * <p>
 *   For situations where a set of algorithm-specific parameters already
 *   exists, there are two init methods that have an
 *   AlgorithmParameterSpec argument. One also has a SecureRandom
 *   argument, while the source of randomness is system-provided for the
 *   other. 
 * <p>
 * In case the client does not explicitly initialise the KeyGenerator (via a
 * call to an init method), each provider must supply (and document) a default
 * initialisation. 
 *
 * @see SecretKey
 */
public class KeyGenerator
{
	public final static String ident = "$Id: KeyGenerator.java,v 1.10 1998/10/29 06:17:02 leachbj Exp $";

	KeyGeneratorSpi spi;
	Provider	provider;
	String		algorithm;

	/**
	 * Creates a KeyGenerator object using the given provider,
	 * KeyGeneratorSpi and for the specified algorithm. 
	 *
	 * @param spi the KeyGeneratorSpi to use
	 * @param provider the provider to use
	 * @param algorithm the standard name of the key algorithm. See
	 *	Appendix A in the Java Cryptography Architecture API
	 *	Specification & Reference for information about standard
	 *	algorithm names.
	 */
	protected KeyGenerator(
		KeyGeneratorSpi spi,
		Provider provider,
		String algorithm)
	{
		this.spi = spi;
		this.provider = provider;
		this.algorithm = algorithm;
	}

	/**
	 * Returns the algorithm name of this KeyGenerator object. 
	 * 
	 * This is the same name that was specified in one of the getInstance
	 * calls that created this KeyGenerator object.
	 *
	 * @return the algorithm name of this KeyGenerator object.
	 */
	public final String getAlgorithm()
	{
		return algorithm;
	}

	/**
	 * create an instance of a key factory object given the provider.
	 *
	 * @param prov the provider object.
	 * @param algorithm the algorithm we want.
	 */
	private static final KeyGenerator getInstance(
		String		algorithm,
		Provider	prov)
		throws NoSuchAlgorithmException
	{
		KeyGenerator	fact;
		KeyGeneratorSpi	spi;
		String		className;

		className = prov.getProperty("KeyGenerator." + algorithm);
		if (className == null)
		{
			throw new NoSuchAlgorithmException(algorithm);
		}

		try
		{
			spi = (KeyGeneratorSpi)Class.forName(
						className).newInstance();
		}
		catch (Exception e)
		{
			throw new NoSuchAlgorithmException(
						algorithm + ": " + e);
		}

		return new KeyGenerator(spi, prov, algorithm);
	}

	/**
	 * Generates a KeyGenerator object for the specified algorithm. If the
	 * default provider package provides an implementation of the requested
	 * key generator, an instance of KeyGenerator containing that
	 * implementation is returned. If the requested key generator is not
	 * available in the default provider package, other provider packages are
	 * searched.
	 *
         * @param algorithm the standard name of the requested key algorithm.
	 *    See Appendix A in the Java Cryptography Extension API
	 *    Specification & Reference for information about standard
         *    algorithm names.
	 * @return the new KeyGenerator object
         * @exception NoSuchAlgorithmException if a key generator for
         *     the specified algorithm is not available in the default provider
         *     package or any of the other provider packages that were searched.
	 */
	public static KeyGenerator getInstance(
		String algorithm)
		throws NoSuchAlgorithmException
	{
		Provider[]	prov;
		KeyGeneratorSpi	keyGenSpi;

		prov = Security.getProviders();

		for (int i = 0; i != prov.length; i++)
		{
			try
			{
				return getInstance(algorithm, prov[i]);
			}
			catch (Exception e)
			{
				// ignore
			}
		}

		throw new NoSuchAlgorithmException(
			"No KeyGenerator could be found for algorithm "
				+ algorithm);
	}

	/**
	 * Generates a KeyGenerator object for the specified key algorithm from
	 * the specified provider.
	 *
	 * @param algorithm the standard name of the requested key algorithm.
	 * See Appendix A in the Java Cryptography Extension API
	 * Specification & Reference for information about standard
	 * algorithm names.
	 * @param provider the name of the provider
	 * @exception NoSuchAlgorithmException if a key generator for the
	 *		requested algorithm is not available.
	 * @exception NoSuchProviderException if the requested provider is
	 *		not available 
	 * @see Provider
	 */
	public static KeyGenerator getInstance(
		String algorithm,
		String provider)
		throws NoSuchAlgorithmException, NoSuchProviderException
	{
		Provider	prov;
		KeyGeneratorSpi	keyGenSpi;

		prov = Security.getProvider(provider);
		if (prov == null)
		{
			throw new NoSuchProviderException(provider);
		}

		return getInstance(algorithm, prov);
	}

	/**
	 * Returns the provider of this KeyGenerator object. 
	 *
	 * @return the provider of this KeyGenerator object 
	 */
	public final Provider getProvider()
	{
		return provider;
	}

	/**
	 * Initialises the key generator. 
	 *
	 * @param random the source of randomness for this generator.
	 */
	public final void init(
		SecureRandom random)
	{
		spi.engineInit(random);
	}

	/**
	 * Initialises the key generator with the specified parameter set,
	 * using a system-provided source of randomness. 
	 *
	 * @param params the key generation parameters 
	 * @exception InvalidAlgorithmParameterException if the given
	 *	parameters are inappropriate for this key generator.
	 */
	public final void init(
		AlgorithmParameterSpec params)
		throws InvalidAlgorithmParameterException
	{
		init(params, new SecureRandom());
	}

	/**
	 * Initialises the key generator with the specified parameter set
	 * and a user-provided source of randomness. 
	 *
	 * @param params the key generation parameters
	 * @param random the source of randomness for this key generator
	 * @exception InvalidAlgorithmParameterException if params is
	 *	inappropriate for this key generator.
	 */
	public final void init(
		AlgorithmParameterSpec	params,
		SecureRandom		random)
		throws InvalidAlgorithmParameterException
	{
		spi.engineInit(params, random);
	}

	/**
	 * Initialises this key generator for a certain strength, using a
	 * system-provided source of randomness.
	 *
	 * @param strength the strength of the key. This is an algorithm-specific
	 *    metric specified in number of bits.
	 */
	public final void init(
		int	strength)
	{
		spi.engineInit(strength, new SecureRandom());
	}

	/**
	 * Initialises this key generator for a certain strength, using a
	 * user-provided source of randomness.
	 *
	 * @param strength the strength of the key. This is an algorithm-specific
	 *     metric specified in number of bits.
	 * @param random the source of randomness for this key generator
	 */
	public final void init(
		int		strength,
		SecureRandom	random)
	{
		spi.engineInit(strength, random);
	}

	/**
	 * Generates a secret key.
	 *
	 * @return the new key
	 */
	public SecretKey generateKey()
	{
		return spi.engineGenerateKey();
	}
}
