/*
 * $Id: KeyGenerator.java,v 1.6 1998/10/26 01:46:18 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/javax/crypto/KeyGenerator.java,v $
 * $Revision: 1.6 $
 * $Date: 1998/10/26 01:46:18 $
 * $State: Exp $
 */
package javax.crypto;

import java.util.*;

import au.net.aba.security.*;
import au.net.aba.security.spec.*;

public class KeyGenerator
{
	public final static String ident = "$Id: KeyGenerator.java,v 1.6 1998/10/26 01:46:18 leachbj Exp $";

	KeyGeneratorSpi spi;
	Provider provider;
	String	algorithm;

	/**
	 * Creates a KeyGenerator object using the given provider,
	 * KeyGeneratorSpi and for the specified algorithm. 
	 *
	 * @param spi the KeyGeneratorSpi to use
	 * @param provider the provider to use
	 * @param algorithm the standard name of the key algorithm. See
	 *	Appendix A in the Java Cryptography Architecture API
	 *	Specification & Reference for information about standard
	 *	algorithm names.
	 */
	protected KeyGenerator(
		KeyGeneratorSpi spi,
		Provider provider,
		String algorithm)
	{
		this.spi = spi;
		this.provider = provider;
		this.algorithm = algorithm;
	}

	/**
	 * Returns the standard name of the key algorithm. See Appendix A in
	 * the Java Cryptography Architecture API Specification & Reference
	 * for information about standard algorithm names. 
	 *
	 * @return the standard algorithm name.
	 */
	public final String getAlgorithm()
	{
		return algorithm;
	}

	/**
	 * Generates a KeyGenerator object for the specified key algorithm.
	 *
	 * @param algorithm the standard name of the requested key algorithm.
	 *	See Appendix A in the Java Cryptography Architecture API
	 *	Specification & Reference for information about standard
	 *	algorithm names. 
	 * @return the new KeyGenerator object
	 * @exception NoSuchAlgorithmException if a key generator for the
	 *		requested algorithm is not available.
	 */
	public static KeyGenerator getInstance(
		String algorithm)
		throws NoSuchAlgorithmException
	{
		try
		{
			return getInstance(algorithm, null);
		}
		catch (NoSuchProviderException e)
		{
			throw new NoSuchAlgorithmException("Cannot find default provider.");
		}
	}

	/**
	 * Generates a KeyGenerator object for the specified key algorithm from
	 * the specified provider.
	 *
	 * @param algorithm the standard name of the requested key algorithm.
	 *	See Appendix A in the Java Cryptography Architecture API
	 *	Specification & Reference for information about standard
	 *	algorithm names. 
	 * @return the new KeyGenerator object
	 * @exception NoSuchAlgorithmException if a key generator for the
	 *		requested algorithm is not available.
	 * @exception NoSuchProviderException if the requested provider is
	 *		not available 
	 * @see Provider
	 */
	public static KeyGenerator getInstance(
		String algorithm,
		String provider)
	throws NoSuchAlgorithmException, NoSuchProviderException
	{
		KeyGeneratorSpi	keyGenSpi;

		try
		{
			StringTokenizer tok;
 
			tok = new StringTokenizer(algorithm, "/");
			if (tok.countTokens() == 1)
			{
				keyGenSpi = (KeyGeneratorSpi)Class.forName(
					"au.net.aba.crypto.provider."
					+ algorithm
					+ "KeyGenerator").newInstance();
			}
			else
			{
				keyGenSpi = (KeyGeneratorSpi)Class.forName(
					"au.net.aba.crypto.provider."
					+ tok.nextToken()
					+ "KeyGenerator").newInstance();
			}

		}
		catch (Exception e)
		{
			throw new NoSuchAlgorithmException(
				"No KeyGenerator could be found for algorithm "
				+ algorithm
				+ " for provider "
				+ provider);
		}

		Provider defaultProvider = null;
		return new KeyGenerator(keyGenSpi, defaultProvider, algorithm);
	}

	/**
	 * Returns the provider of this KeyGenerator object. 
	 *
	 * @return the provider of this KeyGenerator object 
	 */
	public final Provider getProvider()
	{
		return provider;
	}

	/**
	 * Initialises the key generator. 
	 *
	 * @param random the source of randomness for this generator.
	 */
	public void init(
		SecureRandom random)
	{
		spi.engineInit(random);
	}

	/**
	 * Initialises the key generator with the specified parameter set,
	 * using a system-provided source of randomness. 
	 *
	 * @param params the key generation parameters 
	 * @exception InvalidAlgorithmParameterException if the given
	 *	parameters are inappropriate for this key generator.
	 */
	public void init(
		AlgorithmParameterSpec params)
		throws InvalidAlgorithmParameterException
	{
		// XXX null should be the system provided SecureRandom
		init(params, null);
	}

	/**
	 * Initialises the key generator with the specified parameter set
	 * and a user-provided source of randomness. 
	 *
	 * @param params the key generation parameters
	 * @param random the source of randomness for this key generator
	 * @exception InvalidAlgorithmParameterException if params is
	 *	inappropriate for this key generator.
	 */
	public void init(
		AlgorithmParameterSpec	params,
		SecureRandom		random)
		throws InvalidAlgorithmParameterException
	{
		spi.engineInit(params, random);
	}

	/**
	 * Generates a secret key.
	 *
	 * @return the new key
	 */
	public SecretKey generateKey()
	{
		return spi.engineGenerateKey();
	}
}
