/*
 * $Id: Mac.java,v 1.6 1998/10/26 01:47:01 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/Mac.java,v $
 * $Revision: 1.6 $
 * $Date: 1998/10/26 01:47:01 $
 * $State: Exp $
 */

package javax.crypto;

import java.security.*;
import java.security.*;
import java.security.spec.*;

/**
 * This class provides the functionality of a "message authentication code" (MAC).
 * <p>
 * A MAC provides a way to check the integrity of information transmitted over or
 * stored in an unreliable medium, based on a secret key. Typically, message
 * authentication codes are used between two parties that share a secret key in
 * order to validate information transmitted between these parties. 
 * <p>
 * A MAC mechanism that is based on cryptographic hash functions is referred to
 * as HMAC. HMAC can be used with any cryptographic hash function, e.g., MD5 or
 * SHA-1, in combination with a secret shared key. HMAC is specified in RFC 2104. 
 */
public class Mac
{
	private MacSpi		macSpi;
	private Provider	provider;
	private String		algorithm;

	private boolean	initialised;

	/**
	 * Creates a MAC object.
	 *
	 * @param macSpi the delegate 
	 * @param provider the provider 
	 * @param algorithm the algorithm 
	 */
	protected Mac(
		MacSpi		macSpi,
		Provider	provider,
		String		algorithm)
	{
		this.macSpi = macSpi;
		this.provider = provider;
		this.algorithm = algorithm;
		this.initialised = false;
	}

	/**
	 * Returns the standard name of the MAC algorithm. See Appendix A in the
	 * Java Cryptography Extension API Specification & Reference for
	 * information about standard algorithm names.
	 *
	 * @return the standard algorithm name.
	 */
	public final String getAlgorithm()
	{
		return algorithm;
	}

	/**
	 * create an instance of a Mac object given the provider.
	 *
	 * @param prov the provider object.
	 * @param algorithm the algorithm we want.
	 */
	private static final Mac getInstance(
		String		algorithm,
		Provider	prov)
		throws NoSuchAlgorithmException
	{
		Mac		mac;
		MacSpi		spi;
		String		className;

		className = prov.getProperty("Mac." + algorithm);
		if (className == null)
		{
			throw new NoSuchAlgorithmException(algorithm);
		}

		try
		{
			spi = (MacSpi)Class.forName(className).newInstance();
		}
		catch (Exception e)
		{
			throw new NoSuchAlgorithmException(
						algorithm + ": " + e);
		}

		mac = new Mac(spi, prov, algorithm);

		return mac;
	}

	/**
	 * Generates an Mac object that implements the specified MAC algorithm.
	 * If the default provider package provides an implementation of the
	 * requested MAC algorithm, an instance of Mac containing that
	 * implementation is returned. If the algorithm is not available in the
	 * default provider package, other provider packages are searched.
	 *
	 * @param algorithm the standard name of the requested MAC algorithm.
	 *    See Appendix A in the Java Cryptography Extension API Specification
	 *    & Reference for information about standard algorithm names. 
	 * @return the new Mac object.
	 * @exception NoSuchAlgorithmException if the requested MAC algorithm
	 *	is not available.
	 */
	public static final Mac getInstance(
		String algorithm)
		throws NoSuchAlgorithmException
	{
		Provider[] prov = Security.getProviders();

		for (int i = 0; i != prov.length; i++)
		{
			try
			{
				return getInstance(algorithm, prov[i]);
			}
			catch (Exception e)
			{
				// ignore
			}
		}

		throw new NoSuchAlgorithmException(
			"No Mac could be found for algorithm "
				+ algorithm);
	}

	/**
	 * Generates an Mac object for the specified MAC algorithm from the
	 * specified provider.
	 * 
	 * @param algorithm the standard name of the requested MAC algorithm.
	 *    See Appendix A in the Java Cryptography Extension API Specification
	 *    & Reference for information about standard algorithm names. 
	 * @param provider the name of the provider. 
	 * @return the new Mac object.
	 * @exception NoSuchAlgorithmException if the requested MAC algorithm is
	 *    not available from the provider.
	 * @exception NoSuchProviderException if the requested provider is not
	 *    available.
         * @see Provider
	 */
	public static final Mac getInstance(
		String algorithm,
		String provider)
		throws NoSuchAlgorithmException, NoSuchProviderException
	{
		Provider	prov;

		prov = Security.getProvider(provider);
		if (prov == null)
		{
			throw new NoSuchProviderException(
					"Provider " + provider + " not found");
		}

		return getInstance(algorithm, prov);
	}

	/**
	 * Returns the provider of this Mac object.
	 *
	 * @param the provider of this Mac object.
	 */
	public final Provider getProvider()
	{
		return provider;
	}

	/**
	 * Returns the length of the MAC in bytes.
	 *
	 * @return the MAC length in bytes.
	 */
	public final int getMacLength()
	{
		return macSpi.engineGetMacLength();
	}

	/**
	 * Initialises this Mac object with the given (secret) key.
	 *
	 * @exception InvalidKeyException if the given key is inappropriate
	 *	for initialising this MAC.
	 * @return key the (secret) key.
	 */
	public final void init(
		Key key)
		throws InvalidKeyException
	{
		try
		{
			macSpi.engineInit(key, null);
		}
		catch (InvalidAlgorithmParameterException e)
		{
			// this should not happen
			throw new InvalidKeyException(
				"MAC requires algorithm parameters: "
				+ e.getMessage());
		}

		initialised = true;
	}

	/**
	 * Initialises this Mac object with the given (secret) key and algorithm
	 * parameters.
	 *
	 * @param key the (secret) key. 
	 * @param params the algorithm parameters. 
	 * @exception InvalidKeyException if the given key is inappropriate
	 *     for initialising this MAC.
	 * @exception InvalidAlgorithmParameterException if the given algorithm
	 *     parameters are inappropriate for this MAC.
	 */
	public final void init(
		Key key,
		AlgorithmParameterSpec params)
		throws InvalidKeyException, InvalidAlgorithmParameterException
	{
		macSpi.engineInit(key, params);
		initialised = true;
	}

	/**
	 * Processes the given byte.
	 *
	 * @param input the input byte to be processed. 
	 * @exception IllegalStateException if this Mac has not been initialised.
	 */
	public final void update(
		byte input)
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException("MAC has not been initialised");
		}

		macSpi.engineUpdate(input);
	}

	/**
	 * Processes the given array of bytes.
	 *
	 * @param input the array of bytes to be processed.
	 * @exception IllegalStateException if this Mac has not been
	 * 	initialised.
	 */
	public final void update(
		byte[] input)
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException(
				"MAC has not been initialised");
		}

		if (input == null)
		{
			throw new IllegalArgumentException("null input buffer");
		}

		if (input.length > 0)
		{
			macSpi.engineUpdate(input, 0, input.length);
		}
	}

	/**
	 * Processes the first len bytes in input, starting at offset.
	 *
	 * @param input the input buffer. 
	 * @param offset the offset in input where the input starts. 
	 * @param len the number of bytes to process. 
	 * @exception IllegalStateException if this Mac has not been initialised.
	 */
	public final void update(
		byte[]	input,
		int	offset,
		int	len)
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException(
				"MAC has not been initialised");
		}

		if (input == null || offset < 0
			|| len > (input.length - offset))
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		if (len > 0)
		{
			macSpi.engineUpdate(input, offset, len);
		}
	}

	/**
	 * Completes this MAC computation and resets this Mac object for
	 * further MAC computations with the same secret key. (In order to
	 * reuse this Mac object with a different secret key, init must be
	 * called after this call has completed.)
	 *
	 * @return the MAC result.
	 * @exception IllegalStateException if this Mac has not been
	 * 	initialised.
	 */
	public final byte[] doFinal()
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException(
				"MAC has not been initialised");
		}

		return macSpi.engineDoFinal();
	}

	/**
	 * Completes this MAC computation and resets this Mac object for
	 * further MAC computations with the same secret key. (In order to
	 * reuse this Mac object with a different secret key, init must be
	 * called after this call has completed.) The MAC result is stored
	 * in output, starting at outOffset.
	 *
	 * @param output the buffer where the MAC result is stored 
	 * @param outOffset the offset in output where the MAC is stored 
	 * @exception ShortBufferException if the given output buffer is too
	 *    small to hold the result
         * @exception IllegalStateException if this Mac has not been
         * 	initialized.
	 */
	public final void doFinal(
		byte[] output,
		int outOffset)
		throws ShortBufferException, IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException(
				"MAC has not been initialised");
		}

		if (output == null || outOffset < 0)
		{
			throw new IllegalArgumentException("Bad arguments");
		}

		byte[]	mac;

		if ((output.length - outOffset) < macSpi.engineGetMacLength())
		{
			throw new ShortBufferException(
						"Not enough space for MAC");
		}

		mac = macSpi.engineDoFinal();

		System.arraycopy(mac, 0, output, outOffset, mac.length);
	}

	/**
	 * Processes the given array of bytes to complete this MAC computation
	 * and resets this Mac object for further MAC computations with the
	 * same secret key. (In order to reuse this Mac object with a different
	 * secret key, init must be called after this call has completed.)
	 *
	 * @return the MAC result.
	 * @exception IllegalStateException if this Mac has not been
	 * 	initialised.
	 */
	public final byte[] doFinal(
		byte[] input)
		throws IllegalStateException
	{
		if (!initialised)
		{
			throw new IllegalStateException(
				"MAC has not been initialised");
		}

		if (input != null && input.length > 0)
		{
			macSpi.engineUpdate(input, 0, input.length);
		}

		return macSpi.engineDoFinal();
	}

	/**
	 * Resets this Mac object for further MAC computations with the same
	 * secret key. (In order to reuse this Mac object with a different
	 * secret key, init must be called after this call has completed.)
	 */
	public final void reset()
	{
		macSpi.engineReset();
	}

	/**
	 * Returns a clone if the provider implementation is cloneable.
	 *
	 * @return a clone if the provider implementation is cloneable.
	 * @exception CloneNotSupportedException if this is called on a
	 * 	delegate that does not support Cloneable.
	 */
	public final Object clone()
		throws CloneNotSupportedException
	{
		if (macSpi instanceof Cloneable)
		{
			return new Mac((MacSpi)macSpi.clone(),
						provider, algorithm);
		}
		else
		{
			throw new CloneNotSupportedException();
		}
	}
}
