/*
 * $Id: SealedObject.java,v 1.12 1999/02/03 01:41:13 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/SealedObject.java,v $
 * $Revision: 1.12 $
 * $Date: 1999/02/03 01:41:13 $
 * $State: Exp $
 */
package javax.crypto;

import java.io.*;

import java.security.*;

import au.net.aba.security.*;

/**
 * This class enables a programmer to create an object and protect its
 * confidentiality with a cryptographic algorithm. 
 * <p>
 * Given any Serializable object, one can create a SealedObject that encapsulates
 * the original object, in serialised format (i.e., a "deep copy"), and seals
 * (encrypts) its serialised contents, using a cryptographic algorithm such as
 * DES, to protect its confidentiality. The encrypted content can later be
 * decrypted (with the corresponding algorithm using the correct decryption key)
 * and de-serialised, yielding the original object. 
 * <p>
 * Note that the Cipher object must be fully initialised with the correct
 * algorithm, key, padding scheme, etc., before being applied to a SealedObject. 
 * <p>
 * The original object that was sealed can be recovered in two different ways: 
 * <p>
 *   by using the getObject method that takes a Cipher object. 
 * <p>
 *   This method requires a fully initialised Cipher object, initialised with
 *   the exact same algorithm, key, padding scheme, etc., that were used to
 *   seal the object. 
 * <p>
 *   This approach has the advantage that the party who unseals the sealed
 *   object does not require knowledge of the decryption key. For example,
 *   after one party has initialised the cipher object with the required
 *   decryption key, it could hand over the cipher object to another party who
 *   then unseals the sealed object. 
 * <p>
 *   by using one of the get Object methods that take a Key object. 
 * <p>
 *   In this approach, the getObject method creates a cipher object for the
 *   appropriate decryption algorithm and initialises it with the given
 *   decryption key and the algorithm parameters (if any) that were stored in
 *   the sealed object. 
 * <p>
 *   This approach has the advantage that the party who unseals the object
 *   does not need to keep track of the parameters (e.g., an IV) that were used
 *   to seal the object. 
 */
public class SealedObject implements Serializable
{
	/**
	 * @serial encoded algorithm parameters for the cipher
	 *	used to encrypt the object.
	 */
	private byte[]		encodedParams;

	/**
	 * @serial the encrypted object.
	 */
	private byte[]		encryptedContent;

	/**
	 * @serial the algorithm for the parameters used.
	 */
	private String		paramsAlg;

	/**
	 * @serial the algorithm used to seal the object.
	 */
	private String		sealAlg;

	/**
	 * Constructs a SealedObject from any Serializable object. 
	 * <p>
	 * The given object is sealed using the given Cipher, assuming that
	 * the Cipher is already properly initialised. 
	 * <p>
	 * Any algorithm parameters that may be used in the encryption operation
	 * are stored inside of the new SealedObject.
	 *
	 * @param object - the object to be sealed. 
	 * @param c - the cipher used to seal the object. 
	 * @exception IOException if an error occurs during serialisation 
	 * @exception IllegalBlockSizeException if the given cipher is a block
	 *    cipher, no padding has been requested, and the total input
	 *    length is not a multiple of the cipher's block size.
	 */
	public SealedObject(
		Serializable object,
		Cipher c)
		throws IOException, IllegalBlockSizeException
	{ 
		ByteArrayOutputStream	bOut;
		ObjectOutputStream	oOut;
		byte[]			bytes;
		AlgorithmParameters	parameters;

		bOut = new ByteArrayOutputStream();
		oOut = new ObjectOutputStream(bOut);

		oOut.writeObject(object);
		oOut.close();

		try
		{
			encryptedContent = c.doFinal(bOut.toByteArray());
		}
		catch (BadPaddingException e)
		{
			throw new IllegalBlockSizeException(e.toString());
		}

		this.sealAlg = c.getAlgorithm();

		parameters = c.getParameters();
		if (parameters != null)
		{
			encodedParams = parameters.getEncoded();
			paramsAlg = parameters.getAlgorithm();
		}
	}

	/**
	 * Returns the algorithm that was used to seal this object.
	 *
	 * @return the algorithm that was used to seal this object. 
	 */
	public final String getAlgorithm()
	{
		return sealAlg;
	}

	/**
	 * Retrieves the original (encapsulated) object. 
	 * <p>
	 * This method creates a cipher for the algorithm that had been used in
	 * the sealing operation. If the default provider package provides an
	 * implementation of that algorithm, an instance of Cipher containing
	 * that implementation is used. If the algorithm is not available in the
	 * default package, other packages are searched. The Cipher object is
	 * initialised for decryption, using the given key and the parameters
	 * (if any) that had been used in the sealing operation. 
	 * <p>
	 * The encapsulated object is unsealed and de-serialised, before it is
	 * returned.
	 *
	 * @param key the key used to unseal the object.
	 * @return the original object.
	 * @exception IOException if an error occurs during de-serialisation.
	 * @exception ClassNotFoundException if an error occurs during
	 *     de-serialisation.
	 * @exception NoSuchAlgorithmException if the algorithm to
	 *     unseal the object is not available.
	 * @exception InvalidKeyException if the given key cannot be
	 *     used to unseal the object (e.g., it has the wrong algorithm).
	 */
	public final Object getObject(
		Key	key)
		throws IOException, ClassNotFoundException,
			       NoSuchAlgorithmException, InvalidKeyException
	{
		AlgorithmParameters	parameters = null;
		Cipher			c;

		if (encodedParams != null)
		{
			parameters = AlgorithmParameters.getInstance(paramsAlg);
			parameters.init(encodedParams);
		}

		try
		{
			c = Cipher.getInstance(sealAlg);
		}
		catch (NoSuchPaddingException e)
		{
			// should not happen
			throw new NoSuchAlgorithmException(
				"Invalid padding in transformation: "
				+ e.getMessage());
		}

		try
		{
			c.init(Cipher.DECRYPT_MODE, key, parameters);
		}
		catch (InvalidAlgorithmParameterException e)
		{
			// should not happen
			throw new NoSuchAlgorithmException(
				"Invalid algorithm parameters: "
				+ e.getMessage());
		}

		try
		{
			return getObject(c);
		}
		catch (IllegalBlockSizeException ibse)
		{
			throw new IOException("Failed to decrypt: " + ibse);
		}
 		catch (BadPaddingException bpe)
		{
			throw new IOException("Failed to decrypt: " + bpe);
		}
	}

	/**
	 * Retrieves the original (encapsulated) object. 
	 * <p>
	 * The encapsulated object is unsealed (using the given Cipher,
	 * assuming that the Cipher is already properly initialised) and
	 * de-serialised, before it is returned. 
	 *
	 * @param c the cipher used to unseal the object 
	 * @return the original object. 
	 * @exception IOException if an error occurs during de-serialisation 
	 * @exception  ClassNotFoundException if an error occurs during
	 *     de-serialisation.
	 * @exception IllegalBlockSizeException if the given cipher is a block
	 *     cipher, no padding has been requested, and the total input
	 *     length is not a multiple of the cipher's block size.
	 * @exception BadPaddingException 
	 *     if the given cipher has been initialised for decryption, and
	 *     padding has been specified, but the input data does not have
	 *     proper expected padding bytes.
	 */
	public final Object getObject(
		Cipher c)
		throws IOException, ClassNotFoundException,
			IllegalBlockSizeException, BadPaddingException
	{
		/*
		 * retrieve the object.
		 */
		ByteArrayInputStream	bIn;
		ObjectInputStream	oIn;
		byte[]			data;

		try
		{
			data = c.doFinal(encryptedContent);
		}
		catch (Exception e)
		{
			throw new IOException("Processing error: " + e);
		}

		bIn = new ByteArrayInputStream(data);
		oIn = new ObjectInputStream(bIn);

		return oIn.readObject();
	}

	/**
	 * Retrieves the original (encapsulated) object. 
	 * <p>
	 * This method creates a cipher for the algorithm that had been used in
	 * the sealing operation, using an implementation of that algorithm from
	 * the given provider. The Cipher object is initialised for decryption,
	 * using the given key and the parameters (if any) that had been used in
	 * the sealing operation. 
	 * <p>
	 * The encapsulated object is unsealed and de-serialised, before it is
	 * returned.
	 * 
	 * @param key the key used to unseal the object.
	 * @param provider the name of the provider of the algorithm to unseal
	 *    the object.
	 * @return the original object.
	 * @exception IOException - if an error occurs during de-serialisation.
	 * @exception ClassNotFoundException if an error occurs during
	 *     de-serialisation.
	 * @exception NoSuchAlgorithmException if the algorithm to
	 *     unseal the object is not available.
	 * @exception NoSuchProviderException if the given provider is
	 *     not configured.
	 * @exception InvalidKeyException if the given key cannot be
	 *     used to unseal the object (e.g., it has the wrong algorithm).
	 */
	public final Object getObject(
		Key	key,
		String	provider)
		throws IOException, ClassNotFoundException,
		       NoSuchAlgorithmException, NoSuchProviderException,
		       InvalidKeyException
	{
		AlgorithmParameters	parameters = null;
		Cipher			c;

		if (encodedParams != null)
		{
			parameters = AlgorithmParameters.getInstance(paramsAlg,
				provider);

			parameters.init(encodedParams);
		}

		try
		{
			c = Cipher.getInstance(sealAlg, provider);
		}
		catch (NoSuchPaddingException e)
		{
			// should not happen
			throw new NoSuchAlgorithmException(
				"Invalid padding in transformation: "
				+ e.getMessage());
		}

		try
		{
			c.init(Cipher.DECRYPT_MODE, key, parameters);
		}
		catch (InvalidAlgorithmParameterException e)
		{
			// should not happen
			throw new NoSuchAlgorithmException(
				"Invalid algorithm parameters: "
				+ e.getMessage());
		}

		try
		{
			return getObject(c);
		}
		catch (IllegalBlockSizeException ibse)
		{
			throw new IOException("Failed to decrypt: " + ibse);
		}
 		catch (BadPaddingException bpe)
		{
			throw new IOException("Failed to decrypt: " + bpe);
		}
	}
}
