/*
 * $Id: SecretKeyFactory.java,v 1.3 1998/10/26 01:46:20 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/javax/crypto/SecretKeyFactory.java,v $
 * $Revision: 1.3 $
 * $Date: 1998/10/26 01:46:20 $
 * $State: Exp $
 */

package javax.crypto;

import au.net.aba.security.*;
import au.net.aba.security.spec.*;

public class SecretKeyFactory 
{
	private SecretKeyFactorySpi	spi;
	private Provider		provider;

	/**
	 * Creates a SecretKeyFactory object. 
	 *
	 * @param keyFacSpi the delegate 
	 * @param provider the provider 
	 */
	protected SecretKeyFactory(
		SecretKeyFactorySpi	keyFacSpi,
		Provider		provider)
	{
		this.spi = keyFacSpi;
		this.provider = provider;
	}

	/**
	 * Generates a SecretKeyFactory object for the specified secret key
	 * algorithm. If the default provider package contains a
	 * SecretKeyFactory subclass for the specified algorithm, an instance
	 * of that subclass is returned. If such a key factory is not available
	 * in the default provider package, other provider packages are
	 * searched. 
	 *
	 * @param algorithm the standard name of the requested secret key
	 *		algorithm. See Appendix A in the Java Cryptography
	 * 		Extension API Specification & Reference for information
	 * 		about standard algorithm names. 
	 * @exception NoSuchAlgorithmException if the requested algorithm is
	 *		not available in the default provider package or any of
	 *		the other provider packages that were searched. 
	 * @return a SecretKeyFactory object for the specified secret key
	 *	algorithm. 
	 */
	public static final SecretKeyFactory getInstance(
		String algorithm)
		throws NoSuchAlgorithmException
	{
		try
		{
			return getInstance(algorithm, null);
		}
		catch (NoSuchProviderException e)
		{
			throw new NoSuchAlgorithmException("Cannot find default provider.");
		}
	}

	/**
	 * Generates a SecretKeyFactory object for the specified secret key
	 * algorithm from the specified provider. 
	 *
	 * @param algorithm the standard name of the requested secret key
	 * 		algorithm. See Appendix A in the Java Cryptography
	 * 		Extension API Specification & Reference for information
	 *		about standard algorithm names. 
	 * @param provider the name of the provider. 
	 * @return a SecretKeyFactory object for the specified secret key
	 *	algorithm. 
	 * @exception NoSuchAlgorithmException if the algorithm is not
	 *	available from the specified provider. 
	 * @exception NoSuchProviderException if the provider has not been
	 *	configured. 
	 */
	public static final SecretKeyFactory getInstance(
		String algorithm,
		String provider)
		throws NoSuchAlgorithmException, NoSuchProviderException
	{
		SecretKeyFactorySpi	keyFactorySpi;

		try
		{
			keyFactorySpi = (SecretKeyFactorySpi)Class.forName(
					"au.net.aba.crypto.provider."
					+ algorithm + "KeyFactory").newInstance();
		}
		catch (Exception e)
		{
			throw new NoSuchAlgorithmException(
				"No SecretKeyFactory could be found for "
				+ "algorithm " + algorithm);
		}

		return new SecretKeyFactory(keyFactorySpi, null);
	}

	/**
	 * Returns the provider of this SecretKeyFactory object. 
	 * 
	 * @return the provider of this SecretKeyFactory object 
	 */
	public final Provider getProvider()
	{
		return provider;
	}

	/**
	 * Generates a SecretKey object from the provided key specification
	 * (key material). 
	 * @param keySpec the specification (key material) of the secret key 
	 * @exception InvalidKeySpecException if the given key specification is
	 *	inappropriate for this key factory to produce a secret
	 * @return the secret key 
	 */
	public final SecretKey generateSecret(
		KeySpec keySpec)
		throws InvalidKeySpecException
	{
		return spi.engineGenerateSecret(keySpec);
	}

	/**
	 * Returns a specification (key material) of the given key object in
	 * the requested format. 
	 *
	 * @param key the key 
	 * @param keySpec the requested format in which the key material
	 *	shall be returned 
	 * @returns the underlying key specification (key material) in the
	 *	requested format 
	 * @exception InvalidKeySpecException if the requested key
	 *	specification is inappropriate for the given key, or the given
	 *	key cannot be processed (e.g., the given key has an
	 *	unrecognised algorithm or format). 
	 */
	public final KeySpec getKeySpec(
		SecretKey key,
		Class keySpec)
		throws InvalidKeySpecException
	{
		return spi.engineGetKeySpec(key, keySpec);
	}


	/**
	 * Translates a key object, whose provider may be unknown or
	 * potentially untrusted, into a corresponding key object of this
	 * key factory. 
	 *
	 * @param key the key whose provider is unknown or untrusted 
	 * @return the translated key 
	 * @exception InvalidKeyException if the given key cannot be processed
	 *	by this key factory. 
	 */
	public final SecretKey translateKey(
		SecretKey key)
		throws InvalidKeyException
	{
		return spi.engineTranslateKey(key);
	}
}
