/*
 * $Id: DESKeySpec.java,v 1.4 1999/01/12 03:09:27 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/javax/crypto/spec/DESKeySpec.java,v $
 * $Revision: 1.4 $
 * $Date: 1999/01/12 03:09:27 $
 * $State: Exp $
 */

package javax.crypto.spec;

import au.net.aba.security.*;

import au.net.aba.security.spec.*;

import javax.crypto.*;

/**
 * A class that provides a specification for a DES key.
 *
 * @see java.security.Key
 * @see java.security.KeyFactory
 * @see java.security.spec.KeySpec
 */
public class DESKeySpec implements KeySpec
{
	public final static String ident = "$Id: DESKeySpec.java,v 1.4 1999/01/12 03:09:27 leachbj Exp $";

	private byte[]			desKey;

	private static final byte	BIT_MASK[] = {
						 -128, 64, 32, 16, 8, 4, 2 };

	public static final int DES_KEY_LEN = 8;

	/**
	 * Uses the first 8 bytes in key as the key material for the DES key
	 *
	 * @param key  the buffer with the DES key material.
	 * @exception InvalidKeyException if the given key material is shorter
	 * than 8 bytes.
	 */
	public DESKeySpec(
		byte[]	key)
		throws InvalidKeyException
	{
		if (key.length < DES_KEY_LEN)
			throw new InvalidKeyException("Not enough key material");

		desKey = new byte[DES_KEY_LEN];

		System.arraycopy(key, 0, desKey, 0, DES_KEY_LEN);
	}

	/**
	 * Uses the first 8 bytes in key, beginning at offset, as the key
	 * material for the DES key.
	 *
	 * @param key  the buffer with the DES key material.
	 * @param offset  the offset in key, where the DES key material starts.
	 * @exception InvalidKeyException if the given key material, starting at
	 * offset, is shorter than 8 bytes.
	 */
	public DESKeySpec(
		byte[]	key,
		int	offset)
		throws InvalidKeyException
	{
		if (key.length - offset < DES_KEY_LEN)
			throw new InvalidKeyException("Not enough key material");

		desKey = new byte[DES_KEY_LEN];

		System.arraycopy(key, offset, desKey, 0, DES_KEY_LEN);
	}

	/**
	 * Returns the DES key material.
	 *
	 * @return the DES key material.
	 */
	public byte[] getKey()
	{
		return desKey;
	}

	/**
	 * Checks if the given DES key, starting at offset, is parity-adjusted.
	 *
	 * @param key the DES key material.
	 * @param offset the offset in key, where the DES key material starts.
	 * @return true if the given DES key material is parity-adjusted, false
	 *	otherwise.
	 * @exception InvalidKeyException if the given key material, starting
	 *	at offset, is shorter than 8 bytes.
	 */
	public static boolean isParityAdjusted(
		byte	key[],
		int	offset)
		throws InvalidKeyException
	{
		if (key.length - offset < DES_KEY_LEN)
			throw new InvalidKeyException("Not enough key material");

		for (int i = 0; i < DES_KEY_LEN; i++)
		{
			int count = 0;

			for (int j = 0; j < BIT_MASK.length; j++)
			{
				/*
				 * we increment for every "on" bit
				 */
				if ((key[i + offset] & BIT_MASK[j]) != 0)
				{
					count++;
				}
			}

			if ((count & 1) == 1)
			{
				if ((key[i + offset] & 1) == 1)
				{
					return false;
				}
			}
			else if ((key[i + offset] & 1) != 1)
			{
				return false;
			}
		}

		return true;
	}

	/**
 	 * DES has a number of weak keys.  This method will check
	 * if the given DES key material is weak or semi-weak.
	 * <p>
	 * See <a href="http://www.counterpane.com/applied.html">"Applied
	 * Cryptography"</a> by Bruce Schneier for more information.
	 *
	 * @return true if the given DES key material is weak or semi-weak,
	 * 	false otherwise.
	 * @exception InvalidKeyException if the given key material, starting at
	 * 	offset, is shorter than 8 bytes.
	 */
	public static boolean isWeak(
		byte[] key,
		int offset)
		throws InvalidKeyException
	{
		if (key.length - offset < DES_KEY_LEN)
			throw new InvalidKeyException("Not enough key material");

		nextkey: for (int i = 0; i < N_WEAK_KEYS; i++)
		{
			boolean weak = false;
			for (int j = 0; j < DES_KEY_LEN; j++)
			{
				if (key[j + offset] != (byte)weak_keys[
							    i * DES_KEY_LEN + j])
				{
					continue nextkey;
				}
			}
			return true;
		}
		return false;
	}

	/* Table of weak and semi-weak keys taken from Schneier pp281 */
	static private int N_WEAK_KEYS = 16;
	static private short[] weak_keys =
	{
		/* weak keys */
		0x01,0x01,0x01,0x01, 0x01,0x01,0x01,0x01,
		0x1f,0x1f,0x1f,0x1f, 0x0e,0x0e,0x0e,0x0e,
		0x0e,0xe0,0xe0,0xe0, 0xf1,0xf1,0xf1,0xf1,
		0xfe,0xfe,0xfe,0xfe, 0xfe,0xfe,0xfe,0xfe,
		/* semi-weak keys */
		0x01,0xfe,0x01,0xfe, 0x01,0xfe,0x01,0xfe,
		0x1f,0xe0,0x1f,0xe0, 0x0e,0xf1,0x0e,0xf1,
		0x01,0xe0,0x01,0xf1, 0x01,0xf1,0x01,0xf1,
		0x1f,0xfe,0x1f,0xfe, 0x0e,0xfe,0xef,0xfe,
		0x01,0x1f,0x01,0x1f, 0x01,0x0e,0x10,0x0e,
		0xe0,0xfe,0xe0,0xfe, 0xf1,0xfe,0xf1,0xfe,
		0xfe,0x01,0xfe,0x01, 0xfe,0x01,0xfe,0x01,
		0xe0,0x1f,0xe0,0x1f, 0xf1,0x0e,0xf1,0x0e,
		0xe0,0x01,0xe0,0x01, 0xf1,0x01,0xf1,0x01,
		0xfe,0x1f,0xfe,0x1f, 0xfe,0x0e,0xfe,0x0e,
		0x1f,0x01,0x1f,0x01, 0x0e,0x01,0x0e,0x01,
		0xfe,0xe0,0xfe,0xe0, 0xfe,0xf1,0xfe,0xf1
	};
}
