/*
 * $Id: DESedeKeySpec.java,v 1.4 1999/01/12 03:09:28 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.0/src/javax/crypto/spec/DESedeKeySpec.java,v $
 * $Revision: 1.4 $
 * $Date: 1999/01/12 03:09:28 $
 * $State: Exp $
 */

package javax.crypto.spec;

import au.net.aba.security.*;

import au.net.aba.security.spec.*;

import javax.crypto.*;

/**
 * A class that provides a specification for a DESede key.
 */
public class DESedeKeySpec implements KeySpec
{
	public final static String ident = "$Id: DESedeKeySpec.java,v 1.4 1999/01/12 03:09:28 leachbj Exp $";

	private byte[]	desEDEKey;

	public static final int DES_EDE_KEY_LEN = 24;

	/**
	 * Uses the first 24 bytes in key as the DESede key material.
	 *
	 * @param key the buffer with the DES-EDE key material
	 * @exception InvalidKeyException if the given key material
	 *	is shorter than 24 bytes
	 */
	public DESedeKeySpec(
		byte[]	key)
		throws InvalidKeyException
	{
		if (key.length < DES_EDE_KEY_LEN)
			throw new InvalidKeyException("Not enough key material");

		desEDEKey = new byte[DES_EDE_KEY_LEN];

		System.arraycopy(key, 0, desEDEKey, 0, DES_EDE_KEY_LEN);
	}

	/**
	 * Uses the first 24 bytes in key, beginning at offset, as the DES-EDE
	 * key.
	 *
	 * @param key the buffer with the DES-EDE key material
	 * @param offset the offset in key, where the DES-EDE key starts 
	 * @exception InvalidKeyException if the given key material, starting
	 *	at offset, is shorter than 24 bytes
	 */
	public DESedeKeySpec(
		byte[]	key,
		int	offset)
		throws InvalidKeyException
	{
		if (key.length - offset < DES_EDE_KEY_LEN)
			throw new InvalidKeyException("Not enough key material");

		desEDEKey = new byte[DES_EDE_KEY_LEN];

		System.arraycopy(key, offset, desEDEKey, 0, DES_EDE_KEY_LEN);
	}

	/**
	 * Returns the DESede key.
	 *
	 * @return the DESede key.
	 */
	public byte[] getKey()
	{
		return desEDEKey;
	}

	/**
	 * Checks if the given DESede key, starting at offset, is
	 * parity-adjusted.
	 *
	 * @return true if the given DES-EDE key is parity-adjusted,
	 *	false otherwise
	 * @exception InvalidKeyException if the given key material, starting
	 *	at offset, is shorter than 24 bytes
	 */
	public static boolean isParityAdjusted(
		byte	key[],
		int	offset)
		throws InvalidKeyException
	{
		if (key.length - offset < DES_EDE_KEY_LEN)
			throw new InvalidKeyException("Not enough key material");

		if (DESKeySpec.isParityAdjusted(key, offset)
		   && DESKeySpec.isParityAdjusted(key, offset + 8)
		   && DESKeySpec.isParityAdjusted(key, offset + 16))
		{
			return true;
		}

		return false;
	}
}
