/*
 * $Id: DHParameterSpec.java,v 1.4 1998/10/28 23:20:02 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/DHParameterSpec.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/28 23:20:02 $
 * $State: Exp $
 */

package javax.crypto.spec;

import java.math.*;

import java.security.spec.*;

/**
 * This class specifies the set of parameters used with the Diffie-Hellman
 * algorithm, as specified in PKCS #3: Diffie-Hellman Key-Agreement Standard. 
 *
 * A central authority generates parameters and gives them to the two entities
 * seeking to generate a secret key. The parameters are a prime p, a base g,
 * and optionally the length in bits of the private value, l. 
 *
 * It is possible that more than one instance of parameters may be generated
 * by a given central authority, and that there may be more than one central
 * authority. Indeed, each individual may be its own central authority, with
 * different entities having different parameters. 
 *
 * @see javax.crypto.KeyAgreement
 * @see java.security.AlgorithmParameters
 * @see java.security.spec.AlgorithmParameterSpec
 */
public class DHParameterSpec
	implements AlgorithmParameterSpec
{
	public final static String ident = "$Id: DHParameterSpec.java,v 1.4 1998/10/28 23:20:02 leachbj Exp $";

	private BigInteger	p;
	private BigInteger	g;
	private int		l;

	/**
	 * Specifies the Diffie-Hellman parameters, using a prime modulus p
	 * and a base generator g.
	 *
	 * @param p - the prime modulus
	 * @param g - the base generator
	 */
	public DHParameterSpec(
		BigInteger p,
		BigInteger g)
	{
		this.p = p;
		this.g = g;
	}

	/**
	 * Specifies the Diffie-Hellman parameters, using a prime modulus p, a
	 * base generator g, and the size in bits, l, of the random exponent
	 * (private value).
	 *
	 * @param p the prime modulus 
	 * @param g the base generator 
	 * @param l the size in bits of the random exponent (private value) 
	 */
	public DHParameterSpec(
		BigInteger	p,
		BigInteger	g,
		int		l)
	{
		this.p = p;
		this.g = g;
		this.l = l;
	}

	/**
	 * Returns the prime modulus p.
	 *
	 * @return the prime modulus p
	 */
	public java.math.BigInteger getP()
	{
		return p;
	}

	/**
	 * Returns the base generator g.
	 *
	 * @return the base generator g
	 */
	public BigInteger getG()
	{
		return g;
	}

	/**
	 * Returns the size in bits, l, of the random exponent (private value).
	 *
	 * @return the size in bits, l, of the random exponent (private value),
	 *	or 0 if this size has not been set
	 */
	public int getL()
	{
		return l;
	}
}
