/*
 * $Id: PBEKeySpec.java,v 1.5 1998/10/28 23:20:04 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/PBEKeySpec.java,v $
 * $Revision: 1.5 $
 * $Date: 1998/10/28 23:20:04 $
 * $State: Exp $
 */

package javax.crypto.spec;

import au.net.aba.security.spec.*;

/**
 * A user-chosen password that can be used with password-based encryption. 
 * <p>
 * The password can be viewed as some kind of raw key material, from which
 * the encryption mechanism to which it is passed derives a cryptographic key. 
 * <p>
 * Different PBE mechanisms may consume different bits of each password
 * character. For example, the PBE mechanism defined in PKCS #5 looks at only
 * the low order 8 bits of each character, whereas PKCS #12 looks at all 16 bits
 * of each character. 
 * <p>
 * You convert the password characters to a PBE key by creating an instance of
 * the appropriate secret-key factory. For example, a secret-key factory for
 * PKCS #5 will construct a PBE key from only the low order 8 bits of each
 * password character, whereas a secret-key factory for PKCS #12 will take all
 * 16 bits of each character. 
 * <p>
 * Also note that this class stores passwords as char arrays instead of String
 * objects (which would seem more logical), because the String class is immutable
 * and there is no way to overwrite its internal value when the password stored in
 * it is no longer needed. Hence, this class requests the password as a char 
 * array, so it can be overwritten when done. 
 *
 * @see javax.crypto.SecretKeyFactory
 * @see PBEParameterSpec
 */
public class PBEKeySpec
	implements KeySpec
{
	public final static String ident = "$Id: PBEKeySpec.java,v 1.5 1998/10/28 23:20:04 leachbj Exp $";

	private char[]	password;

	/**
	 * Constructor that takes a password.
	 * <p>
	 * Note that the given password is cloned before it is stored in
	 * the new PBEKeySpec object.
	 *
	 * @param password the password used for password-based encryption 
	 */
	public PBEKeySpec(
		char[] password)
	{
		char[]	tmp;

		tmp = new char[password.length];
		System.arraycopy(password, 0, tmp, 0, tmp.length);

		this.password = tmp;
	}

	/**
	 * Retrieves the password.
	 * <p>
	 * Note that this method returns a reference to the password. It is
	 * the caller's responsibility to zero out the password information
	 * after it is no longer needed.
	 *
	 * @return the password
	 */
	public final char[] getPassword()
	{
		return password;
	}
}
