/*
 * $Id: RC2ParameterSpec.java,v 1.4 1998/10/26 01:47:07 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/RC2ParameterSpec.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/26 01:47:07 $
 * $State: Exp $
 */

package javax.crypto.spec;

import java.security.*;
import java.security.spec.*;

/**
 * This class specifies the parameters used with the RC2 algorithm. 
 * <p>
 * The parameters consist of an effective key size and optionally an 8-byte
 * initialisation vector (IV) (only in feedback mode). 
 * <p>
 * This class can be used to initialise a Cipher object that implements the RC2
 * algorithm. 
 */
public class RC2ParameterSpec
	implements AlgorithmParameterSpec
{
	public final static String ident = "$Id: RC2ParameterSpec.java,v 1.4 1998/10/26 01:47:07 leachbj Exp $";

	private int	effectiveKeyBits;
	private byte[]	iv;

	/**
	 * Constructs a parameter set for RC2 from the given effective key size
	 * (in bits).
	 *
	 * @param effectiveKeyBits the effective key size in bits.
	 */
	public RC2ParameterSpec(
		int effectiveKeyBits)
	{
		this.effectiveKeyBits = effectiveKeyBits;
		this.iv = null;
	}

	/**
	 * Constructs a parameter set for RC2 from the given effective key size
	 * (in bits) and an 8-byte IV.
	 *
	 * @param effectiveKeyBits the effective key size in bits.
	 * @param iv the buffer with the 8-byte IV.
	 */
	public RC2ParameterSpec(
		int	effectiveKeyBits,
		byte[]	iv)
	{
		this.effectiveKeyBits = effectiveKeyBits;
		this.iv = new byte[effectiveKeyBits / 8 + 1];
		System.arraycopy(iv, 0, this.iv, 0, this.iv.length);
	}

	/**
	 * Constructs a parameter set for RC2 from the given effective key size
	 * (in bits) and IV. The IV is taken from iv, starting at offset.
	 * 
	 * @param effectiveKeyBits the effective key size in bits.
	 * @param iv the buffer with the IV.
	 * @param offset the offset in iv where the 8-byte IV starts.
	 */
	public RC2ParameterSpec(
		int	effectiveKeyBits,
		byte[]	iv,
		int	offset)
	{
		this.effectiveKeyBits = effectiveKeyBits;
		this.iv = new byte[effectiveKeyBits / 8 + 1];
		System.arraycopy(iv, offset, this.iv, 0, this.iv.length);
	}

	/**
	 * Returns the effective key size in bits.
	 * 
	 * @return the effective key size in bits.
	 */
	public int getEffectiveKeyBits()
	{
		return effectiveKeyBits;
	}

	/**
	 * Returns the IV or null if this parameter set does not contain an IV.
	 *
	 * @return the IV or null if this parameter set does not contain an IV.
	 */
	public byte[] getIV()
	{
		return iv;
	}
}
