/*
 * $Id: RC5ParameterSpec.java,v 1.4 1998/10/26 01:47:07 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/RC5ParameterSpec.java,v $
 * $Revision: 1.4 $
 * $Date: 1998/10/26 01:47:07 $
 * $State: Exp $
 */

package javax.crypto.spec;

import java.security.*;

import au.net.aba.security.spec.*;

/**
 * This class specifies the parameters used with the RC5 algorithm. 
 * <p>
 * The parameters consist of a version number, a rounds count, a word size, and
 * optionally an initialisation vector (IV) (only in feedback mode). 
 * <p>
 * This class can be used to initialise a Cipher object that implements the RC5
 * algorithm as supplied by RSA Data Security, Inc., or parties authorised by
 * RSA Data Security. 
 */
public class RC5ParameterSpec
	implements AlgorithmParameterSpec
{
	private int	version;
	private int	rounds;
	private int	wordSize;
	private byte[]	iv;

	/**
	 * Constructs a parameter set for RC5 from the given version, number of
	 * rounds and word size (in bits).
	 *
	 * @param version the version.
	 * @param rounds the number of rounds.
	 * @param wordSize the word size in bits.
	 */
	public RC5ParameterSpec(
		int	version,
		int	rounds,
		int	wordSize)
	{
		this.version = version;
		this.rounds = rounds;
		this.wordSize = wordSize;
		this.iv = null;
	}

	/**
	 * Constructs a parameter set for RC5 from the given version, number of
	 * rounds, word size (in bits), and IV. Note that the size of the IV
	 * (block size) must be twice the word size.
	 *
	 * @param version the version.
	 * @param rounds the number of rounds.
	 * @param wordSize the word size in bits.
	 * @param iv the buffer with the IV.
	 */
	public RC5ParameterSpec(
		int	version,
		int	rounds,
		int	wordSize,
		byte[]	iv)
	{
		this.version = version;
		this.rounds = rounds;
		this.wordSize = wordSize;
		this.iv = new byte[2 * wordSize];
		System.arraycopy(iv, 0, this.iv, 0, this.iv.length);
	}

	/**
	 * Constructs a parameter set for RC5 from the given version, number of
	 * rounds, word size (in bits), and IV. The IV is taken from iv, 
	 * starting at offset. Note that the size of the IV (block size), 
	 * starting at offset, must be twice the word size.
	 *
	 * @param version the version.
	 * @param rounds the number of rounds.
	 * @param wordSize the word size in bits.
	 * @param iv the buffer with the IV.
	 * @param offset the offset in iv where the IV starts.
	 */
	public RC5ParameterSpec(
		int	version,
		int	rounds,
		int	wordSize,
		byte[]	iv,
		int	offset)
	{
		this.version = version;
		this.rounds = rounds;
		this.wordSize = wordSize;
		this.iv = new byte[2 * wordSize];
		System.arraycopy(iv, offset, this.iv, 0, this.iv.length);
	}

	/**
	 * Returns the version.
	 *
	 * @return the version.
	 */
	public int getVersion()
	{
		return version;
	}

	/**
	 * Returns the number of rounds.
	 * 
	 * @return the number of rounds.
	 */
	public int getRounds()
	{
		return rounds;
	}

	/**
	 * Returns the word size in bits.
	 *
	 * @return the word size in bits.
	 */
	public int getWordSize()
	{
		return wordSize;
	}

	/**
	 * Returns the IV or null if this parameter set does not contain an IV.
	 *
	 * @return the IV or null if this parameter set does not contain an IV.
	 */
	public byte[] getIV()
	{
		return iv;
	}

}
