/*
 * $Id: SecretKeySpec.java,v 1.6 1998/10/28 23:20:05 leachbj Exp $
 * $Author: leachbj $
 *
 * Copyright (C) 1996-1998 Australian Business Access Pty Ltd.
 * All rights reserved.
 * 
 * Use, modification, copying and distribution of this software is subject the
 * terms and conditions of the ABA Public Licence. See the file
 * "PUBLIC_LICENCE" for additional information.
 *
 * If you have not received a copy of the Public Licence, you must destroy all
 * copies of this file immediately. 
 *
 * $Source: /aba/CVSROOT/jdk1.1/src/javax/crypto/spec/SecretKeySpec.java,v $
 * $Revision: 1.6 $
 * $Date: 1998/10/28 23:20:05 $
 * $State: Exp $
 */

package javax.crypto.spec;

import java.security.*;

import au.net.aba.security.spec.*;

import javax.crypto.*;

/**
 * This class specifies a secret key in a provider-independent fashion. It can be
 * used to construct a SecretKey from a byte array, without having to go through a
 * (provider-based) SecretKeyFactory. 
 * <p>
 * This class is only useful for raw secret keys that can be represented as a
 * byte array and have no key parameters associated with them, e.g., DES or
 * Triple DES keys. 
 *
 * @see SecretKey
 * @see SecretKeyFactory
 */
public class SecretKeySpec implements KeySpec, SecretKey
{
	public final static String ident = "$Id: SecretKeySpec.java,v 1.6 1998/10/28 23:20:05 leachbj Exp $";

	/**
	 * @serial the algorithm the key is for.
	 */
	String	algorithm;

	/**
	 * @serial the key material
	 */
	byte[]	key;

	/**
	 * Constructs a secret key from the given byte array. 
	 * <p>
	 * This constructor does not check if the given bytes indeed specify a
	 * secret key of the specified algorithm. For example, if the algorithm
	 * is DES, this constructor does not check if key is 8 bytes long, and
	 * also does not check for weak or semi-weak keys. In order for those
	 * checks to * be performed, an algorithm-specific key specification
	 * class (in this case: DESKeySpec) must be used.
	 *
	 * @param key the key material of the secret key. 
	 * @param algorithm the name of the associated algorithm. 
	 */
	public SecretKeySpec(
		byte[] key,
		String algorithm)
	{
		this.key = new byte[key.length];
		System.arraycopy(key, 0, this.key, 0, key.length);

		this.algorithm = algorithm;
	}

	/**
	 * Constructs a secret key from the given byte array, using the first len
	 * bytes of key, starting at offset. 
	 * <p>
	 * This constructor does not check if the given bytes indeed specify a
	 * secret key of the specified algorithm. For example, if the algorithm is
	 * DES, this constructor does not check if key is 8 bytes long, and also
	 * does not check for weak or semi-weak keys. In order for those checks
	 * to be performed, an algorithm-specific key specification class (in this
	 * case: DESKeySpec) must be used.
	 *
         * @param key the key material of the secret key. 
         * @param offset the offset in key where the key material starts. 
         * @param len the length of the key material. 
         * @param algorithm the standard name of the associated key algorithm. See
	 * Appendix A in the Java Cryptography Extension API Specification &
	 * Reference for information about standard algorithm names. 
	 */
	public SecretKeySpec(
		byte[]	key,
		int	offset,
		int	len,
		String	algorithm)
	{
		this.key = new byte[len];
		System.arraycopy(key, offset, this.key, 0, len);

		this.algorithm = algorithm;
	}

	/**
	 * Returns the name of the algorithm associated with this secret key.
	 *
	 * @return the secret key algorithm.
	 */
	public String getAlgorithm()
	{
		return algorithm;
	}

	/**
	 * Returns the name of the encoding format for this secret key.
	 *
	 * @return the string "RAW".
	 */
	public String getFormat()
	{
		return "RAW";
	}

	/**
	 * Returns the key material of this secret key.
	 *
	 * @return the key material
	 */
	public byte[] getEncoded()
	{
		byte[]	tmp;

		tmp = new byte[key.length];
		System.arraycopy(key, 0, tmp, 0, key.length);

		return tmp;
	}
}
